/*
 * Copyright 2022 BytePlus Pte. Ltd.
 * SPDX-License-Identifier: MIT
 */

package com.ss.bytertc.engine.flutter.render;

import android.content.Context;
import android.util.Log;
import android.view.SurfaceView;
import android.view.TextureView;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.RestrictTo;

import com.ss.bytertc.engine.RTCVideo;
import com.ss.bytertc.engine.RemoteVideoRenderConfig;
import com.ss.bytertc.engine.VideoCanvas;
import com.ss.bytertc.engine.data.RemoteStreamKey;
import com.ss.bytertc.engine.data.ReturnStatus;
import com.ss.bytertc.engine.data.StreamIndex;
import com.ss.bytertc.engine.data.VideoRotation;
import com.ss.bytertc.engine.flutter.BuildConfig;
import com.ss.bytertc.engine.flutter.base.RTCType;
import com.ss.bytertc.engine.flutter.base.RTCTypeBox;
import com.ss.bytertc.engine.flutter.base.RTCVideoManager;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;

@RestrictTo(RestrictTo.Scope.LIBRARY)
class RTCSurfaceView implements PlatformView, MethodChannel.MethodCallHandler {
    private static final String TAG = "RTCSurfaceView";

    /**
     * Use {@link android.view.SurfaceView} as Render, DEFAULT
     */
    public static final int VIEW_TYPE_SURFACE = 0;
    /**
     * Use {@link android.view.TextureView} as Render
     */
    public static final int VIEW_TYPE_TEXTURE = 1;

    public static final int CANVAS_TYPE_LOCAL = 0;
    public static final int CANVAS_TYPE_REMOTE = 1;
    public static final int CANVAS_TYPE_PUBLIC_STREAM = 2;
    public static final int CANVAS_TYPE_ECHO_TEST = 3;

    @NonNull
    private final View mRenderView;

    private int mCanvasType;

    public RTCSurfaceView(BinaryMessenger messenger, Context context, int viewId, Object args) {
        RTCTypeBox arguments = new RTCTypeBox(args, "RTCSurfaceView");
        mRenderView = createRenderView(context, arguments);

        MethodChannel methodChannel = new MethodChannel(messenger, "com.bytedance.ve_rtc_surfaceView" + viewId);
        methodChannel.setMethodCallHandler(this);

        setVideoCanvas(arguments);
    }

    // region PlatformView
    @Override
    public View getView() {
        return mRenderView;
    }

    @Override
    public void dispose() {
        if (mCanvasType == CANVAS_TYPE_ECHO_TEST) {
            EchoTestViewHolder.setView(null);
        }
    }
    // endregion

    // region MethodChannel.MethodCallHandler
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        RTCTypeBox arguments = new RTCTypeBox(call.arguments, call.method);
        switch (call.method) {
            case "setVideoCanvas": {
                int retValue = setVideoCanvas(arguments);
                result.success(retValue);
                break;
            }
            case "updateVideoCanvas": {
                int retValue = updateVideoCanvas(arguments);
                result.success(retValue);
                break;
            }
            case "setZOrderOnTop": {
                if (mRenderView instanceof SurfaceView) {
                    ((SurfaceView) mRenderView).setZOrderOnTop(arguments.optBoolean("onTop"));
                }
                result.success(ReturnStatus.RETURN_STATUS_SUCCESS.value());
                break;
            }
            case "setZOrderMediaOverlay": {
                if (mRenderView instanceof SurfaceView) {
                    ((SurfaceView) mRenderView).setZOrderMediaOverlay(arguments.optBoolean("isMediaOverlay"));
                }
                result.success(ReturnStatus.RETURN_STATUS_SUCCESS.value());
                break;
            }
            default:
                result.notImplemented();
                break;
        }
    }

    private int setVideoCanvas(RTCTypeBox arguments) {
        mCanvasType = arguments.optInt("canvasType", -1);
        if (BuildConfig.DEBUG) {
            Log.d(TAG, "RTCSurfaceView: canvasType=" + mCanvasType);
        }
        switch (mCanvasType) {
            case CANVAS_TYPE_LOCAL:
                return setupLocalVideo(arguments);
            case CANVAS_TYPE_REMOTE:
                return setupRemoteVideo(arguments);
            case CANVAS_TYPE_PUBLIC_STREAM:
                return setupPublicStreamVideo(arguments);
            case CANVAS_TYPE_ECHO_TEST:
                return setupEchoTestVideo(arguments);
            default:
                Log.e(TAG, "Unknown canvasType: " + mCanvasType);
                return ReturnStatus.RETURN_STATUS_FAILURE.value();
        }
    }

    private int updateVideoCanvas(RTCTypeBox arguments) {
        mCanvasType = arguments.optInt("canvasType", -1);
        if (BuildConfig.DEBUG) {
            Log.d(TAG, "RTCSurfaceView: canvasType=" + mCanvasType);
        }
        switch (mCanvasType) {
            case CANVAS_TYPE_LOCAL:
                return updateLocalVideo(arguments);
            case CANVAS_TYPE_REMOTE:
                return updateRemoteVideo(arguments);
            case CANVAS_TYPE_PUBLIC_STREAM:
                return setupPublicStreamVideo(arguments);
            case CANVAS_TYPE_ECHO_TEST:
                return setupEchoTestVideo(arguments);
            default:
                Log.e(TAG, "Unknown canvasType: " + mCanvasType);
                return ReturnStatus.RETURN_STATUS_FAILURE.value();
        }
    }

    private int setupLocalVideo(RTCTypeBox arguments) {
        StreamIndex streamIndex = StreamIndex.fromId(arguments.optInt("streamIndex"));
        VideoCanvas videoCanvas = RTCType.toVideoCanvas(arguments);
        videoCanvas.renderView = mRenderView;
        return RTCVideoManager.getRTCVideo().setLocalVideoCanvas(streamIndex, videoCanvas);
    }

    private int updateLocalVideo(RTCTypeBox arguments) {
        StreamIndex streamIndex = StreamIndex.fromId(arguments.optInt("streamIndex"));
        int renderMode = arguments.optInt("renderMode");
        int backgroundColor = arguments.optInt("backgroundColor");

        RTCVideo rtcVideo = RTCVideoManager.getRTCVideo();
        return rtcVideo.updateLocalVideoCanvas(streamIndex, renderMode, backgroundColor);
    }

    private int setupRemoteVideo(RTCTypeBox arguments) {
        final RemoteStreamKey streamKey = RTCType.toRemoteStreamKey(arguments);
        VideoCanvas videoCanvas = RTCType.toVideoCanvas(arguments);
        videoCanvas.renderView = mRenderView;
        return RTCVideoManager.getRTCVideo().setRemoteVideoCanvas(streamKey, videoCanvas);
    }

    private int updateRemoteVideo(RTCTypeBox arguments) {
        RTCVideo rtcVideo = RTCVideoManager.getRTCVideo();
        final RemoteStreamKey streamKey = RTCType.toRemoteStreamKey(arguments);
        RemoteVideoRenderConfig renderConfig = new RemoteVideoRenderConfig();
        renderConfig.renderMode = arguments.optInt("renderMode");
        renderConfig.backgroundColor = arguments.optInt("backgroundColor");
        renderConfig.renderRotation = VideoRotation.fromId(arguments.optInt("renderRotation"));
        return rtcVideo.updateRemoteStreamVideoCanvas(streamKey, renderConfig);
    }

    private int setupPublicStreamVideo(RTCTypeBox arguments) {
        final String uid = arguments.optString("uid");
        VideoCanvas videoCanvas = RTCType.toVideoCanvas(arguments);
        videoCanvas.renderView = mRenderView;
        RTCVideo rtcVideo = RTCVideoManager.getRTCVideo();
        return rtcVideo.setPublicStreamVideoCanvas(uid, videoCanvas);
    }

    private int setupEchoTestVideo(RTCTypeBox arguments) {
        VideoCanvas videoCanvas = RTCType.toVideoCanvas(arguments);
        videoCanvas.renderView = mRenderView;
        EchoTestViewHolder.setView(videoCanvas);
        return ReturnStatus.RETURN_STATUS_SUCCESS.value();
    }

    // endregion

    @NonNull
    private static View createRenderView(Context context, RTCTypeBox arguments) {
        int viewType = arguments.optInt("viewType", VIEW_TYPE_SURFACE);
        if (viewType == VIEW_TYPE_TEXTURE) {
            return new TextureView(context);
        } else {
            SurfaceView view = new SurfaceView(context);
            view.setZOrderOnTop(arguments.optBoolean("onTop"));
            view.setZOrderMediaOverlay(arguments.optBoolean("isMediaOverlay"));
            return view;
        }
    }
}
