/*
 * Copyright 2022 BytePlus Pte. Ltd.
 * SPDX-License-Identifier: MIT
 */

#import <Flutter/FlutterCodecs.h>
#import <Flutter/FlutterBinaryMessenger.h>
#import <BytePlusRTC/objc/ByteRTCVideo.h>
#import "ByteRTCFlutterSurfaceViewFactory.h"
#import "ByteRTCFlutterStreamHandler.h"
#import "ByteRTCVideoManager+Extension.h"
#import "ByteRTCFlutterMapCategory.h"

typedef NS_ENUM(NSInteger, RTCVideoCanvasType) {
    
    RTCVideoCanvasTypeLocal = 0,
    
    RTCVideoCanvasTypeRemote = 1,
    
    RTCVideoCanvasTypePublicStream = 2,
    
    RTCVideoCanvasTypeEchoTest = 3,
};

@interface ByteRTCFlutterSurfaceViewPlugin : NSObject <FlutterPlatformView>

@property (nonatomic, strong) ByteRTCView *surfaceView;
@property (nonatomic, strong) ByteRTCFlutterMethodHandler *methodHandler;

@end

@implementation ByteRTCFlutterSurfaceViewPlugin

- (instancetype)initWithMessager:(NSObject<FlutterBinaryMessenger>*)messenger
                           frame:(CGRect)frame
                  viewIdentifier:(int64_t)viewId
                       arguments:(id)args {
    self = [super init];
    if (self) {
        self.surfaceView = [[ByteRTCView alloc] initWithFrame:frame];
        self.methodHandler = [[ByteRTCFlutterMethodHandler alloc] init];
        [self.methodHandler registerMethodChannelWithName:[NSString stringWithFormat:@"com.bytedance.ve_rtc_surfaceView%lld", viewId]
                                             methodTarget:self
                                          binaryMessenger:messenger];
        if ([args isKindOfClass:[NSDictionary class]]) {
            [self setVideoCanvas:args];
        }
    }
    return self;
}

- (UIView *)view {
    return _surfaceView;
}

- (void)dealloc {
    [self.methodHandler destroy];
}

- (ByteRTCVideo *)rtcVideo {
    return [ByteRTCVideoManager shared].rtcVideo;
}

#pragma mark - method
- (void)setVideoCanvas:(NSDictionary *)arguments result:(FlutterResult)result {
    result(@([self setVideoCanvas:arguments]));
}

- (void)updateVideoCanvas:(NSDictionary *)arguments result:(FlutterResult)result {
    result(@([self updateVideoCanvas:arguments]));
}

#pragma mark -
- (int)setVideoCanvas:(NSDictionary *)args {
    if (!self.rtcVideo) {
        return ByteRTCReturnStatusFailure;
    }
    RTCVideoCanvasType canvasType = [args[@"canvasType"] integerValue];
    switch (canvasType) {
        case RTCVideoCanvasTypeLocal:
            return [self setupLocalVideo:args];
        case RTCVideoCanvasTypeRemote:
            return [self setupRemoteVideo:args];
        case RTCVideoCanvasTypePublicStream:
            return [self setupPublicStreamVideo:args];
        case RTCVideoCanvasTypeEchoTest:
            return [self setupEchoTestVideo:args];
        default:
            return ByteRTCReturnStatusFailure;
    }
}

- (int)updateVideoCanvas:(NSDictionary *)args {
    if (!self.rtcVideo) {
        return ByteRTCReturnStatusFailure;
    }
    RTCVideoCanvasType canvasType = [args[@"canvasType"] integerValue];
    switch (canvasType) {
        case RTCVideoCanvasTypeLocal:
            return [self updateLocalVideo:args];
        case RTCVideoCanvasTypeRemote:
            return [self updateRemoteVideo:args];
        case RTCVideoCanvasTypePublicStream:
            return [self setupPublicStreamVideo:args];
        case RTCVideoCanvasTypeEchoTest:
            return [self setupEchoTestVideo:args];
        default:
            return ByteRTCReturnStatusFailure;
    }
}

- (int)setupLocalVideo:(NSDictionary *)arguments {
    ByteRTCStreamIndex streamIndex = [arguments[@"streamIndex"] integerValue];
    ByteRTCVideoCanvas *canvas = [ByteRTCVideoCanvas bf_fromMap:arguments];
    canvas.view = self.surfaceView;
    return [self.rtcVideo setLocalVideoCanvas:streamIndex
                                   withCanvas:canvas];
}

- (int)updateLocalVideo:(NSDictionary *)arguments {
    ByteRTCStreamIndex streamIndex = [arguments[@"streamIndex"] integerValue];
    ByteRTCRenderMode renderMode = [arguments[@"renderMode"] unsignedIntegerValue];
    NSUInteger backgroundColor = [arguments[@"backgroundColor"] integerValue];
    return  [self.rtcVideo updateLocalVideoCanvas:streamIndex
                                   withRenderMode:renderMode
                              withBackgroundColor:backgroundColor];
}

- (int)setupRemoteVideo:(NSDictionary *)arguments {
    ByteRTCRemoteStreamKey *streamKey = [ByteRTCRemoteStreamKey bf_fromMap:arguments];
    ByteRTCVideoCanvas *canvas = [ByteRTCVideoCanvas bf_fromMap:arguments];
    canvas.view = self.surfaceView;
    return [self.rtcVideo setRemoteVideoCanvas:streamKey withCanvas:canvas];
}

- (int)updateRemoteVideo:(NSDictionary *)arguments {
    ByteRTCRemoteStreamKey *streamKey = [ByteRTCRemoteStreamKey bf_fromMap:arguments];
    ByteRTCRemoteVideoRenderConfig *renderConfig = [[ByteRTCRemoteVideoRenderConfig alloc] init];
    renderConfig.renderMode = [arguments[@"renderMode"] unsignedIntegerValue];
    renderConfig.backgroundColor = [arguments[@"backgroundColor"] integerValue];
    renderConfig.renderRotation = [arguments[@"renderRotation"] integerValue];
    return [self.rtcVideo updateRemoteStreamVideoCanvas:streamKey withRemoteVideoRenderConfig:renderConfig];
}

- (int)setupPublicStreamVideo:(NSDictionary *)arguments {
    NSString *streamId = arguments[@"uid"];
    ByteRTCVideoCanvas *canvas = [ByteRTCVideoCanvas bf_fromMap:arguments];
    canvas.view = self.surfaceView;
    return [self.rtcVideo setPublicStreamVideoCanvas:streamId
                                          withCanvas:canvas];
}

- (int)setupEchoTestVideo:(NSDictionary *)arguments {
    [ByteRTCVideoManager shared].echoTestView = self.surfaceView;
    return ByteRTCReturnStatusSuccess;
}

@end

@interface ByteRTCFlutterSurfaceViewFactory ()

@property (nonatomic, weak) NSObject<FlutterBinaryMessenger>* messenger;

@end

@implementation ByteRTCFlutterSurfaceViewFactory

- (instancetype)initWithMessager:(NSObject<FlutterBinaryMessenger>*)messenger {
    self = [super init];
    if (self) {
        self.messenger = messenger;
    }
    return self;
}

- (NSObject<FlutterMessageCodec> *)createArgsCodec {
    return [FlutterStandardMessageCodec sharedInstance];
}

- (NSObject<FlutterPlatformView> *)createWithFrame:(CGRect)frame
                                    viewIdentifier:(int64_t)viewId
                                         arguments:(id)args {
    return [[ByteRTCFlutterSurfaceViewPlugin alloc] initWithMessager:self.messenger
                                                               frame:frame
                                                      viewIdentifier:viewId
                                                           arguments:args];
}

@end
