// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import '../src/base/bytertc_enum_convert.dart';
import 'bytertc_audio_effect_player_api.dart';
import 'bytertc_media_defines.dart';
import 'bytertc_room_api.dart';
import 'bytertc_video_event_handler.dart';
import 'bytertc_video_api.dart';

/// Type of audio device
enum AudioDeviceType {
  /// Unknown device
  unknown,

  /// Speaker or headphone
  renderDevice,

  /// Microphone
  captureDevice,

  /// Screen capturing audio device
  screenCaptureDevice,
}

/// Audio playback device
enum AudioRoute {
  /// Unknown device
  routeDefault,

  /// Wired earphones
  headset,

  /// Earpiece. Built-in device for calling
  earpiece,

  /// Speaker. Built-in device for hands-free audio playback
  speakerphone,

  /// Bluetooth earphones
  headsetBluetooth,

  /// USB device
  headsetUSB,
}

/// Audio scene type
///
/// After selecting an audio scene, RTC will automatically apply call volume/media volume based on the client's audio capture playback device and status. <br>
/// If the preset audio scene type cannot meet your business needs, please contact technical support for customization.
enum AudioScenario {
  /// Music scene. Default setting.
  ///
  /// This option is suitable for scenes that require musical expression. Such as live music. <br>
  /// Audio capture playback device and capture playback status, the mapping to the volume type is as follows:
  /// <table border>
  /// <tr><th></th><th>Mic off</th><th>Mic on</th><th>Remarks</th></tr>
  /// <tr><td>Build-in microphone and speaker/earpiece</td><td>Media volume</td><td>Call volume</td><td>/</td></tr>
  /// <tr><td>Wired headset/ USB headset/ Sound card</td><td>Media volume</td><td>Media volume</td><td></td></tr>
  /// <tr><td>Bluetooth headset</td><td>Media volume</td><td>Media volume</td><td>Media volume</td><td>Only the device's build-in microphone can be used for local audio capture, even if the Bluetooth headset has a microphone.</td></tr>
  /// </table>
  music,

  /// High-quality call scene.
  ///
  /// This option is suitable for scenarios where musical expression is required but you also want to be able to use the microphone that comes with the Bluetooth headset for audio capture. <br>
  /// This scenario allows for a balanced audio experience when playing out/using Bluetooth headphones and avoids the sudden change in listening experience caused by switching between volume types when using Bluetooth headphones.<br>
  /// Audio capture playback device and capture playback status, the mapping to the volume type is as follows: <br>
  /// <table border>
  /// <tr><th></th><th>Mic off</th><th>Mic on</th><th>Remarks</th></tr>
  /// <tr><td>Build-in microphone and speaker/earpiece</td><td>Media volume</td><td>Call volume</td><td>/</td></tr>
  /// <tr><td>Wired headset/ USB headset/ Sound card</td><td>Media volume</td><td>Media volume</td><td>/</td></tr>
  /// <tr><td>Bluetooth headset</td><td>Call volume</td><td>Call volume</td><td>Call volume</td><td>The microphone on the Bluetooth headset can be used for audio capture.</td></tr>
  /// </table>
  highQualityCommunication,

  /// Call volume-only scene.
  ///
  /// This option is suitable for calls or conference scenarios where frequent speaker switching is required.<br>
  /// In this scenario, the call volume is used throughout, regardless of the client audio capture/playback device and capture/playback status.<br>
  /// This scene maintains a uniform audio pattern without sudden volume changes in listening; echoes are eliminated to the greatest extent possible for optimum call clarity.<br>
  /// When using a Bluetooth headset, you can use the microphone that comes with the Bluetooth headset for audio capture. <br>
  /// However, this scene will depress the volume of other audio being played back using the media volume and the sound quality will deteriorate.
  communication,

  /// Pure media scene. It is generally not recommended.
  ///
  /// In this scenario, regardless of the client side audio acquisition and playback device and the acquisition and playback status, the media volume is used throughout the process.
  /// If the audio quality is unsatisfactory when you put on speaker, please contact our technical support team.
  media,

  /// Game media scene. Only suitable for game scenes.
  ///
  /// In this scenario, the Bluetooth headset uses the call volume, and other devices use the media volume.
  /// If the audio quality is unsatisfactory when you put on speaker, please contact our technical support team.
  gameStreaming,

  /// </table>
  /// High-quality chatting scenario.
  ///
  /// This scenario is the same as `highQualityCommunication`, but in one case: in this scenario, switching on/off the microphone does not change the volume type. When you talk with the built-in microphone and speaker/earpiece, the call volume is used all the time.
  ///
  /// For different audio device and status, the selected volume type is as follows:
  /// <table border>
  ///    <tr><th></th><th>Mic Off</th><th>Mic On</th><th>Remarks</th><tr>
  ///    <tr><td>Build-in microphone and speaker</td><td> Call volume </td><td> call volume </td><td>/</td></tr>
  ///    <tr><td>earpiece</td><td> Call volume </td><td> call volume </td><td>/</td></tr>
  ///    <tr><td>Wired headset/ USB headset/ external sound card</td><td> Media volume </td><td> Media Volume </td><td>/</td></tr>
  ///    <tr><td> Bluetooth Headset </td><td> Call Volume </td><td> Call Volume </td><td> Ability to use the microphone included in the Bluetooth headset for audio capture. </td></tr>
  /// </table>
  highQualityChat,
}

/// Types of voice change effects.
///
/// For more types, contact technical supporters.
enum VoiceChangerType {
  /// Original sound, no effects
  original,

  /// Giant
  giant,

  /// Chipmunk
  chipmunk,

  /// Minions
  minions,

  /// Vibrato
  vibrato,

  /// Robot
  robot,
}

/// Types of reverb effects
enum VoiceReverbType {
  /// Original sound, no effects
  original,

  /// Echo
  echo,

  /// Concert
  concert,

  /// Ethereal
  ethereal,

  /// Karaoke
  ktv,

  /// Recording studio
  studio,

  /// Virtual Stereo
  virtualStereo,

  /// Spacious
  spacious,

  /// 3D vocal
  vocal3D,
}

/// Audio Equalization effect.
enum VoiceEqualizationBandFrequency {
  /// The frequency band with a center frequency of 31Hz.
  frequency31,

  /// The frequency band with a center frequency of 62Hz.
  frequency62,

  /// The frequency band with a center frequency of 125Hz.
  frequency125,

  /// The frequency band with a center frequency of 350Hz.
  frequency250,

  /// The frequency band with a center frequency of 500Hz.
  frequency500,

  /// The frequency band with a center frequency of 1kHz.
  frequency1k,

  /// The frequency band with a center frequency of 2kHz.
  frequency2k,

  /// The frequency band with a center frequency of 4kHz.
  frequency4k,

  /// The frequency band with a center frequency of 8kHz.
  frequency8k,

  /// The frequency band with a center frequency of 16kHz.
  frequency16k,
}

/// Voice equalization effect.
class VoiceEqualizationConfig {
  /// Frequency band.
  final VoiceEqualizationBandFrequency frequency;

  /// Gain of the frequency band in dB. The range is `[-15, 15]`.
  final int gain;

  /// @nodoc
  const VoiceEqualizationConfig({
    required this.frequency,
    required this.gain,
  });

  /// @nodoc
  Map<String, dynamic> toMap() => {
        'frequency': frequency.index,
        'gain': gain,
      };
}

/// Voice reverb effect.
class VoiceReverbConfig {
  /// The room size for reverb simulation. The range is `[0.0, 100.0]`. The default value is `50.0f`. The larger the room, the stronger the reverberation.
  final double roomSize;

  /// The decay time of the reverb effect. The range is `[0.0, 100.0]`. The default value is `50.0f`.
  final double decayTime;

  /// The damping index of the reverb effect. The range is `[0.0, 100.0]`. The default value is `50.0f`.
  final double damping;

  /// The Intensity of the wet signal in dB. The range is `[-20.0, 10.0]`. The default value is `0.0f`.
  final double wetGain;

  /// The Intensity of the dry signal in dB. The range is `[-20.0, 10.0]`. The default value is `0.0f`.
  final double dryGain;

  /// The delay of the wet signal in ms. The range is `[0.0, 200.0]`. The default value is `0.0f`.
  final double preDelay;

  /// @nodoc
  const VoiceReverbConfig({
    this.roomSize = 50,
    this.decayTime = 50,
    this.damping = 50,
    this.wetGain = 0,
    this.dryGain = 0,
    this.preDelay = 0,
  });

  /// @nodoc
  Map<String, dynamic> toMap() => {
        'roomSize': roomSize,
        'decayTime': decayTime,
        'damping': damping,
        'wetGain': wetGain,
        'dryGain': dryGain,
        'preDelay': preDelay,
      };
}

/// Mixing audio playback status
enum AudioMixingState {
  /// Mixing file is loaded
  preloaded,

  /// Mixing file is playing
  playing,

  /// Mixing is paused
  paused,

  /// Mixing is stopped
  stopped,

  /// Mixing file playback failed
  failed,

  /// End of mixing file playback
  finished,

  /// PCM mixing enabled
  pcmEnabled,

  /// PCM mixing disabled
  pcmDisabled,
}

/// Audio mixing related error code
enum AudioMixingError {
  /// Normal.
  ok,

  /// Preload failed. Invalid path or the length exceeds 20s.
  preloadFailed,

  /// Mixing failed. Invalid path or fail to open the file.
  startFailed,

  /// Invalid mixing task ID.
  idNotFound,

  /// Invalid start position.
  setPositionFailed,

  /// Invalid volume. The range is `[0, 400]`.
  invalidVolume,

  /// Another file was preloaded for mixing. Call [RTCAudioEffectPlayer.unload] first.
  loadConflict,

  /// Invalid mixing type.
  idTypeNotMatch,

  /// Invalid pitch value.
  idTypeInvalidPitch,

  /// Invalid audio track.
  invalidAudioTrack,

  /// Mixing starting.
  isStarting,

  /// Invalid playback speed.
  invalidPlaybackSpeed,
}

/// Enable/disable in-ear monitoring.
enum EarMonitorMode {
  /// Disable(default setting)
  off,

  /// Enable
  on,
}

/// Audio mixing type
enum AudioMixingType {
  /// Played at the local device only.
  playout,

  /// Sent to the remote devices only.
  publish,

  /// Played at the local device and sent to the remote devices.
  playoutAndPublish,
}

/// Audio mixing playback channel type
enum AudioMixingDualMonoMode {
  /// Consistent with audio files.
  auto,

  /// Only the left channel audio in the file can be heard
  left,

  /// Only the right channel audio in the file can be heard
  right,

  /// Both the left and right channel audio in the file can be heard at the same time
  mix,
}

/// The volume callback modes.
enum AudioReportMode {
  /// Always-on(Default).
  normal,

  /// After visibly joining a room and unpublish your streams, disable the volume callback.
  disconnect,

  /// After visibly joining a room and unpublish your streams, enable the volume callback. The volume is reset to `0`.
  reset,
}

/// The audio info included in [RTCVideoEventHandler.onLocalAudioPropertiesReport]
enum AudioPropertiesMode {
  /// Only locally captured microphone audio info and locally captured screen audio info are included.
  microphone,

  /// The following information are included:
  /// + Locally captured microphone audio info and locally captured screen audio info.
  /// + Locally audio mixing info.
  audioMixing,
}

/// Configuration related to audio property information prompt
class AudioPropertiesConfig {
  /// Information prompt interval in milliseconds
  ///
  /// + `<= 0`: Turn off the prompt.
  /// + `(0,100]`: Invalid interval value, and will be automatically reset to 100ms.
  /// + `> 100`: The actual value of interval.
  int interval;

  /// Whether to enable audio spectrum detection
  bool enableSpectrum;

  /// Whether to enable Voice Activity Detection(VAD)
  bool enableVad;

  /// The volume callback modes
  AudioReportMode localMainReportMode;

  /// Sets whether to include locally mixed audio info in [RTCVideoEventHandler.onLocalAudioPropertiesReport].
  ///
  /// Locally captured microphone audio info and locally captured screen audio info are included by default.
  AudioPropertiesMode audioReportMode;

  /// The smoothing coefficient for audio attribute information prompt. The range is `(0.0, 1.0]`.
  ///
  /// The default value is `1.0`, which means the smoothing effect is off by default. Smaller the value, smoother the audio volume prompt.<br>
  /// If you want to enable the smooth effect, the recommended value is `0.3`.
  double smooth;

  /// Sets whether to return the vocal pitch of the local user.
  bool enableVoicePitch;

  /// @nodoc
  AudioPropertiesConfig({
    this.interval = 100,
    this.enableSpectrum = false,
    this.enableVad = false,
    this.localMainReportMode = AudioReportMode.normal,
    this.audioReportMode = AudioPropertiesMode.microphone,
    this.smooth = 1.0,
    this.enableVoicePitch = false,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'interval': interval,
      'enableSpectrum': enableSpectrum,
      'enableVad': enableVad,
      'localMainReportMode': localMainReportMode.index,
      'audioReportMode': audioReportMode.index,
      'smooth': smooth,
      'enableVoicePitch': enableVoicePitch,
    };
  }
}

/// Audio properties
class AudioPropertiesInfo {
  /// Linear volume. The value is in linear relation to the original volume. The higher the value, the higher the volume. The range is `[0,255]`.
  ///
  /// - `[0, 25]`: Silence
  /// - `[26, 75]`: Low volume
  /// - `[76, 204]`: Medium volume
  /// - `[205, 255]`: High volume
  final int? linearVolume;

  /// Non-linear volume in dB. The value is in proportion to the log value of the original volume. You can use the value to recognize the Active Speaker in the room. The range is `[-127, 0]`.
  ///
  /// - `[-127, -60]`: Silence
  /// - `[-59, -40]`: Low volume
  /// - `[-39, -20]`: Medium volume
  /// - `[-19, 0]`: High volume
  final int? nonlinearVolume;

  /// Voice Activity Detection(VAD)result
  ///
  /// - 1: Voice activity detected.
  /// - 0: No voice activity detected.
  /// - -1: VAD not activated.
  final int? vad;

  /// Spectrum array. The default length is 257.
  final List<double>? spectrum;

  /// The vocal pitch of the local user, in Hertz.
  ///
  /// When the following two conditions are met at the same time, the vocal pitch of the local user will be returned:
  /// - Calls [RTCVideo.enableAudioPropertiesReport], and sets the value of enableVoicePitch to `true`.
  /// - The local user's voice is included in the locally captured audio data.
  /// In other situations, `0` will be returned.
  final double? voicePitch;

  /// @nodoc
  const AudioPropertiesInfo({
    this.linearVolume,
    this.nonlinearVolume,
    this.vad,
    this.spectrum,
    this.voicePitch,
  });

  /// @nodoc
  factory AudioPropertiesInfo.fromMap(Map<dynamic, dynamic> map) {
    return AudioPropertiesInfo(
      linearVolume: map['linearVolume'],
      nonlinearVolume: map['nonlinearVolume'],
      vad: map['vad'],
      spectrum: List<double>.from(map['spectrum']),
      voicePitch: map['voicePitch'],
    );
  }
}

/// Remote audio properties
class RemoteAudioPropertiesInfo {
  /// Information about the remote stream
  final RemoteStreamKey? streamKey;

  /// Information about the audio properties
  final AudioPropertiesInfo? audioPropertiesInfo;

  /// @nodoc
  const RemoteAudioPropertiesInfo({
    this.streamKey,
    this.audioPropertiesInfo,
  });

  /// @nodoc
  factory RemoteAudioPropertiesInfo.fromMap(Map<dynamic, dynamic> map) {
    return RemoteAudioPropertiesInfo(
      streamKey: RemoteStreamKey.fromMap(map['streamKey']),
      audioPropertiesInfo:
          AudioPropertiesInfo.fromMap(map['audioPropertiesInfo']),
    );
  }
}

/// Local audio properties
class LocalAudioPropertiesInfo {
  /// Type of the stream
  final StreamIndex? streamIndex;

  /// Information about the audio properties
  final AudioPropertiesInfo? audioPropertiesInfo;

  /// @nodoc
  const LocalAudioPropertiesInfo({
    this.streamIndex,
    this.audioPropertiesInfo,
  });

  /// @nodoc
  factory LocalAudioPropertiesInfo.fromMap(Map<dynamic, dynamic> map) {
    return LocalAudioPropertiesInfo(
      streamIndex: (map['type'] as int).streamIndex,
      audioPropertiesInfo:
          AudioPropertiesInfo.fromMap(map['audioPropertiesInfo']),
    );
  }
}

/// Sound quality
enum AudioProfileType {
  /// Default sound quality
  ///
  /// The sound quality is configured via the [RoomProfile] sent from the server or set by the client.
  def,

  /// Fluent
  ///
  /// + Sample rate: 16 KHz
  /// + Mono-channel
  /// + Encoding bitrate: 32 Kbps
  ///
  /// Low resource consumption and small network packets guarantee a smooth call. It is suitable for most game scenarios, such as team-wide voice chat, group-wide voice chat, nation-wide voice chat.
  fluent,

  /// Mono-channel standard
  ///
  /// + Sample rate: 24 KHz
  /// + Encoding bitrate: 48 Kbps
  ///
  /// For scenarios where distinct voice quality is required, you can choose this mode, which achieves balanced latency, consumption, and network packets. It is suitable for educational Apps and the online Mafia Games.
  standard,

  /// Dual-channel music
  ///
  /// + Sample rate: 48 KHz
  /// + Encoding bitrate: 128 Kbps
  ///
  /// This mode provides high-resolution audio at a cost of high latency, consumption, and large network packets. It is suitable for music Apps such as co-hosting and online talent contests. <br>
  /// Not recommended for game Apps.
  hd,

  /// Dual-channel standard
  ///
  /// + Sample rate: 48 KHz
  /// + Encoding bitrate: 80 Kbps
  standardStereo,

  /// Mono-channel music
  ///
  /// + Sample rate: 48 KHz
  /// + Encoding bitrate: 64 Kbps
  hdMono,
}

/// Coordinate value of the local user's position in the rectangular coordinate system in the RTC room.
class Position {
  /// X-coordinate
  final double x;

  /// Y-coordinate
  final double y;

  /// Z-coordinate
  final double z;

  /// @nodoc
  const Position({
    this.x = 0,
    this.y = 0,
    this.z = 0,
  });

  /// @nodoc
  const Position.zero() : this();

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'x': x,
      'y': y,
      'z': z,
    };
  }
}

/// Direction orientation information
class RTCOrientation {
  /// X direction vector
  final double x;

  /// Y direction vector
  final double y;

  /// Z direction vector
  final double z;

  /// @nodoc
  const RTCOrientation({
    this.x = 0,
    this.y = 0,
    this.z = 0,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'x': x,
      'y': y,
      'z': z,
    };
  }
}

/// Three-dimensional orientation information, each pair of vectors need to be perpendicular.
class HumanOrientation {
  /// Forward orientation, the default value is {1,0,0}, i.e., the forward orientation is in the positive direction of x-axis.
  final RTCOrientation forward;

  /// Rightward orientation, the default value is {0,1,0}, i.e., the rightward orientation is in the positive direction of y-axis.
  final RTCOrientation right;

  /// Upward orientation, the default value is {0,0,1}, i.e., the upward orientation is in the positive direction of z-axis.
  final RTCOrientation up;

  /// @nodoc
  const HumanOrientation.origin()
      : forward = const RTCOrientation(x: 1, y: 0, z: 0),
        right = const RTCOrientation(x: 0, y: 1, z: 0),
        up = const RTCOrientation(x: 0, y: 0, z: 1);

  /// @nodoc
  HumanOrientation({
    required this.forward,
    required this.right,
    required this.up,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'forward': forward.toMap(),
      'right': right.toMap(),
      'up': up.toMap(),
    };
  }
}

/// Information on the user's position in the rectangular coordinate system for the spatial audio.
class PositionInfo {
  /// 3D coordinate values of the user's position in the rectangular coordinate system for the spatial audio. You need to build your own rectangular coordinate system.
  final Position position;

  /// Information on the three-dimensional orientation of the user in the rectangular coordinate system for the spatial audio. Any two of the 3D coordinate vectors of the user's position need to be perpendicular to each other.
  final HumanOrientation orientation;

  /// @nodoc
  const PositionInfo({
    this.position = const Position.zero(),
    this.orientation = const HumanOrientation.origin(),
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'position': position.toMap(),
      'orientation': orientation.toMap(),
    };
  }
}

/// The alignment mode of remote audio streams.
enum AudioAlignmentMode {
  /// Disabled.
  off,

  /// All subscribed audio streams are aligned based on the process of the background music.
  audioMixing,
}

/// Mode of volume attenuation with distance from the sound source in spatial audio
enum AttenuationType {
  /// Disable volume attenuation
  none,

  /// Linear attenuation
  linear,

  /// Exponential attenuation
  exponential,
}

/// Audio sample rate in Hz.
enum AudioSampleRate {
  /// Default value. 48000Hz.
  auto,

  /// 8000Hz
  rate8000,

  /// 16000Hz
  rate16000,

  /// 32000Hz
  rate32000,

  /// 44100Hz
  rate44100,

  /// 48000Hz
  rate48000,
}

/// Audio channel
enum AudioChannel {
  /// Default value. Dual channels.
  auto,

  /// Mono channel.
  mono,

  /// Dual channels.
  stereo,
}

/// Audio file recording source type.
enum AudioFrameSource {
  /// The audio captured by the local microphone.
  mic,

  /// The audio got by mixing all remote user's audio.
  playback,

  /// The audio got by mixing the local captured audio and all remote user's audio.
  mixed,
}

/// Audio quality.
enum AudioQuality {
  /// Low quality
  low,

  /// Medium quality
  medium,

  /// High quality
  high,

  /// Ultra high quality
  ultraHigh,
}

/// Audio recording configuration.
class AudioRecordingConfig {
  /// Absolute path of the recorded file, file name included. The App must have the write and read permission of the path.
  ///
  /// The files format is restricted to .aac and .wav.
  String absoluteFileName;

  /// Audio sample rate.
  AudioSampleRate sampleRate;

  /// Number of audio channels.
  ///
  /// If number of audio channels of recording is different than that of audio capture, the behavior is:
  /// + If the number of capture is 1, and the number of recording is 2, the recorded audio is two-channel data after copying mono-channel data. <br>
  /// + If the number of capture is 2, and the number of recording is 1, the recorded audio is recorded by mixing the audio of the two channels.
  AudioChannel channel;

  /// The source of the recording.
  AudioFrameSource frameSource;

  /// Recording quality. Only valid for .aac file.
  ///
  /// When the sample rate is 32kHz, the file (10min) size for different qualities are:
  /// + low: 1.2MB; <br>
  /// + medium: 2MB; <br>
  /// + high: 3.75MB; <br>
  /// + ultra high: 7.5MB.
  AudioQuality quality;

  /// @nodoc
  AudioRecordingConfig({
    required this.absoluteFileName,
    this.sampleRate = AudioSampleRate.rate48000,
    this.channel = AudioChannel.stereo,
    this.frameSource = AudioFrameSource.mic,
    this.quality = AudioQuality.high,
  });

  /// @nodoc
  Map<String, dynamic> toMap() => {
        'absoluteFileName': absoluteFileName,
        'sampleRate': sampleRate.value,
        'channel': channel.value,
        'frameSource': frameSource.index,
        'quality': quality.index,
      };
}

/// Audio recording config
enum AudioRecordingState {
  /// Recording exception.
  error,

  /// Recording in progress.
  processing,

  /// The recording task ends, and the file is saved.
  success,
}

/// Error code for audio recording.
enum AudioRecordingErrorCode {
  /// Normal.
  ok,

  /// No file write permissions.
  noPermission,

  /// Not in the room.
  notInRoom,

  /// Recording has already started.
  alreadyStarted,

  /// Recording has not started yet.
  notStarted,

  /// Failure. Invalid file format.
  notSupport,

  /// Other error.
  other,
}

/// ANC modes. The ANC algorithm is determined by the room profile you set when calling [RTCRoom.joinRoom].
enum AnsMode {
  /// Disable ANC.
  disable,

  /// Cancel subtle background noise.
  low,

  /// Cancel medium-level, continuous noise, such as the sound of fans or air conditioners.
  medium,

  /// Cancel loud, impulsive, and intermittent noise, such as keyboard clicking noise, a crash/clash, a bark, and chair scraping noise.
  high,

  /// Enable automatic ANC. The ANC algorithm is dynamically determined by RTC.
  automatic,
}

/// Player state.
enum PlayerState {
  /// Not started.
  idle,

  /// Preloaded
  preloaded,

  /// Opened
  opened,

  /// Playing
  playing,

  /// Paused
  paused,

  /// Stopped
  stopped,

  /// Failed
  failed,
}

/// Error code for audio player .
enum PlayerError {
  /// OK.
  ok,

  /// Format not supported.
  formatNotSupport,

  /// Invalid file path.
  invalidPath,

  /// The prerequisite is not met. Check the specific API doc.
  invalidState,

  /// Invalid position.
  invalidPosition,

  /// Invalid volume.
  invalidVolume,

  /// Invalid pitch value.
  invalidPitch,

  /// Invalid audio track.
  invalidAudioTrackIndex,

  /// Invalid playback speed.
  invalidPlaybackSpeed,

  /// Invalid effect ID. Receive this error code when you call other APIs before the audio file is preloaded or opened.
  invalidEffectId,
}
