// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'bytertc_audio_defines.dart';
import 'bytertc_video_api.dart';

/// Audio mixing configuration
class AudioEffectPlayerConfig {
  /// Audio mixing playback types.
  final AudioMixingType type;

  /// Mix playback times.
  /// + Play_count < = 0: Infinite loop
  /// + Play_count == 1: Play once (default)
  /// + Play_count > 1: Play play_count times
  final int playCount;

  /// The starting position in ms. 0 by default.
  final int startPos;

  /// The increase or decrease value compared with the original pitch of the music file. The range is `[-12, 12]`. The default value is 0. The pitch distance between two adjacent values is half a step. A positive value indicates a rising pitch, and a negative value indicates a falling pitch.
  final int pitch;

  /// @nodoc
  const AudioEffectPlayerConfig({
    this.type = AudioMixingType.playoutAndPublish,
    this.playCount = 1,
    this.startPos = 0,
    this.pitch = 0,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'type': type.index,
      'playCount': playCount,
      'startPos': startPos,
      'pitch': pitch,
    };
  }
}

/// [effectId]: The ID of [RTCAudioEffectPlayer]. Set by [RTCVideo.getAudioEffectPlayer].
///
/// [state]: Play state.
///
/// [error]: Play error.
typedef OnAudioEffectPlayerStateChangedType = void Function(
    int effectId, PlayerState state, PlayerError error);

/// Event handler for [RTCAudioEffectPlayer]. You must call [RTCAudioEffectPlayer.setEventHandler] to set the corresponding event handler to get the events.
///
/// Valid since v3.56.1
class RTCAudioEffectPlayerEventHandler {
  /// Callback for audio mixing status change.
  OnAudioEffectPlayerStateChangedType? onAudioEffectPlayerStateChanged;

  /// @nodoc
  RTCAudioEffectPlayerEventHandler({
    this.onAudioEffectPlayerStateChanged,
  });
}

/// Audio effect player.
///
/// Valid since v3.56.1
///
/// Call [RTCAudioEffectPlayer.setEventHandler] to set the callback handler to receive related callbacks.
abstract class RTCAudioEffectPlayer {
  /// Starts to play the audio effect file. <br>
  /// This API can be called multiple times with different IDs and file paths for multiple effects at the same time.
  ///
  /// [effectId]: Audio effect ID. Used for identifying the audio effect, please ensure that the audio effect ID is unique.  <br>
  /// If this API is called repeatedly with the same ID, the previous effect will stop and the next effect will start, and you'll receive [RTCAudioEffectPlayerEventHandler.onAudioEffectPlayerStateChanged].
  ///
  /// [filePath]: Audio effect file paths.<br>
  /// URL of online file, URI of local file, full path to local file, or path to local file starting with `/assets/` are supported. For URL of online file, only the https protocol is supported.<br>
  /// Recommended sample rate for audio effect files: 8KHz、16KHz、22.05KHz、44.1KHz、48KHz.<br>
  /// Local audio effect file formats supported by different platforms:
  /// <table border>
  ///    <tr><th></th><th>mp3</th><th>mp4</th><th>aac</th><th>m4a</th><th>3gp</th><th>wav</th><th>ogg</th><th>ts</th><th>wma</th></tr>
  ///    <tr><td>Android</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td></td></tr>
  ///    <tr><td>iOS/macOS</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td></td><td></td></tr>
  ///    <tr><td>Windows</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td>Y</td><td>Y</td></tr>
  /// </table>
  /// Online audio effect file formats supported by different platforms:
  /// <table border>
  ///    <tr><th></th><th>mp3</th><th>mp4</th><th>aac</th><th>m4a</th><th>3gp</th><th>wav</th><th>ogg</th><th>ts</th><th>wma</th></tr>
  ///    <tr><td>Android</td><td>Y</td><td></td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td></td><td></td></tr>
  ///    <tr><td>iOS/macOS</td><td>Y</td><td></td><td>Y</td><td>Y</td><td></td><td>Y</td><td></td><td></td><td></td></tr>
  ///    <tr><td>Windows</td><td>Y</td><td></td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td>Y</td><td>Y</td></tr>
  /// </table>
  ///
  /// [config]: See AudioEffectPlayerConfig{@link #AudioEffectPlayerConfig}.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + If the file has been loaded into memory via [RTCAudioEffectPlayer.preload], make sure that the ID here is the same as the ID set by [RTCAudioEffectPlayer.preload].
  /// + After starting to play an audio effect file, you can call the [RTCAudioEffectPlayer.stop] API to stop playing the audio effect file.
  Future<int?> start(
    int effectId, {
    required String filePath,
    AudioEffectPlayerConfig config = const AudioEffectPlayerConfig(),
  });

  /// Stops the playback of audio effect files.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + After calling the [RTCAudioEffectPlayer.start] API to start playing the audio effect file, you can call this API to stop playing the audio effect file.
  /// + After calling this API to stop playing an audio effect file, the audio effect file will be unloaded automatically.
  Future<int?> stop(int effectId);

  /// Stops playback of all audio effect files.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + After calling [RTCAudioEffectPlayer.start] to start playing audio effect files, you can call this API to stop playing all audio effect files.
  /// + After calling this API to stop playing all audio effect files, the audio effect files will be unloaded automatically.
  Future<int?> stopAll();

  /// Preloads specified music files into memory to avoid repeated loading when playing the same file frequently and reduce CPU usage.
  ///
  /// [effectId]: Audio effect ID. Used for identifying the audio effect. Please ensure that the audio effect ID is unique.<br>
  /// If this API is called repeatedly with the same ID, the later one will overwrite the previous one.<br>
  /// If you call [RTCAudioEffectPlayer.start] first and then call this API with the same ID, the SDK will stop the previous effect and then load the next one, and you will receive [RTCAudioEffectPlayerEventHandler.onAudioEffectPlayerStateChanged].<br>
  /// After calling this API to preload A.mp3, if you need to call [RTCAudioEffectPlayer.start] to play B.mp3 with the same ID, please call [RTCAudioEffectPlayer.unload] to unload A.mp3 first, otherwise SDK will report an error "loadConflict".
  ///
  /// [filePath]: The filepath of effect file. URL of online file, URI of local file, full path to local file, or path to local files starting with `/assets/` are supported. For URL of online file, only the https protocol is supported.<br>
  /// The length of the pre-loaded file must not exceed 20s.<br>
  /// Audio effect file formats supported are the same as [RTCAudioEffectPlayer.start].
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + This API just preloads the specified audio effect file, and only calls the [RTCAudioEffectPlayer.start] API to start playing the specified audio effect file.
  /// + The specified audio effect file preloaded by calling this API can be unloaded by [RTCAudioEffectPlayer.unload].
  Future<int?> preload(
    int effectId, {
    required String filePath,
  });

  /// Unloads the specified audio effect file.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// This API can be called only after [RTCAudioEffectPlayer.start] or [RTCAudioEffectPlayer.preload].
  Future<int?> unload(int effectId);

  /// Unloads all audio effect files.
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  Future<int?> unloadAll();

  /// Pauses the playback of audio effect files.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + After calling [RTCAudioEffectPlayer.start] to start playing the audio effect file, you can pause the audio effect file by calling this API.
  /// + After calling this API to pause the audio effect file, you can call [RTCAudioEffectPlayer.resume] to resume playback.
  Future<int?> pause(int effectId);

  /// Pauses the Playback of all audio effect files.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + After calling the [RTCAudioEffectPlayer.start] API to start playing audio effect files, you can pause playing all audio effect files by calling this API.
  /// + After calling this API to pause the playback of all audio effect files, you can call the [RTCAudioEffectPlayer.resumeAll] API to resume all playback.
  Future<int?> pauseAll();

  /// Resumes the playback of audio effect files.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// After calling the [RTCAudioEffectPlayer.pause] API to pause the audio effect file, you can resume playback by calling this API.
  Future<int?> resume(int effectId);

  /// Resumes the playback of all audio effect files.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// After calling the [RTCAudioEffectPlayer.pauseAll] API to pause all the audio effect files being played, you can resume playback by calling this API.
  Future<int?> resumeAll();

  /// Sets the start position of the audio effect file.
  ///
  /// [position]: The starting playback position of the audio effect file in milliseconds.<br>
  /// You can get the total duration of the audio effect file by calling [RTCAudioEffectPlayer.getDuration], the value of position should be less than the total duration of the audio effect file.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + When playing online files, calling this API may cause a delay in playback.
  /// + Call this API after [RTCAudioEffectPlayer.start].
  Future<int?> setPosition(
    int effectId, {
    required int position,
  });

  /// Gets the current position of audio effect file playback.
  ///
  /// Return value:
  /// + `>0`: Success, the current progress of audio effect file playback in milliseconds.
  /// + `<0`: Failure.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + When playing online files, calling this API may cause a delay in playback.
  /// + Call this API after [RTCAudioEffectPlayer.start].
  Future<int?> getPosition(int effectId);

  /// Adjusts the volume level of a specified audio effect.
  ///
  /// [volume]: The ratio of the volume to the original volume in % with overflow protection. The range is `[0, 400]` and the recommended range is `[0, 100]`. 100 by default.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// Call this API after [RTCAudioEffectPlayer.start].
  Future<int?> setVolume(
    int effectId, {
    required int volume,
  });

  /// Sets the volume of all audio effect, including audio effect files and PCM effects.
  ///
  /// [volume]: The ratio of the volume to the original volume in % with overflow protection. The range is `[0, 400]` and the recommended range is `[0, 100]`. 100 by default.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// This API has a lower priority than [RTCAudioEffectPlayer.setVolume], i.e. the volume of the audio effect set by [RTCAudioEffectPlayer.setVolume] is not affected by this API.
  Future<int?> setVolumeAll(int volume);

  /// Gets the current volume.
  ///
  /// Return value:
  /// + `>0`: Success, the current volume value.
  /// + `<0`: Failed.
  ///
  /// Call this API after [RTCAudioEffectPlayer.start].
  Future<int?> getVolume(int effectId);

  /// Gets the duration of the audio effect file.
  ///
  /// Return value:
  /// + `>0`: Success, the duration of the audio effect file in milliseconds.
  /// + `<0`: failed.
  ///
  /// Call this API after [RTCAudioEffectPlayer.start].
  Future<int?> getDuration(int effectId);

  /// Sets the event handler.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failed.
  void setEventHandler(RTCAudioEffectPlayerEventHandler? handler);
}
