// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'bytertc_audio_defines.dart';
import 'bytertc_video_api.dart';
import 'bytertc_video_event_handler.dart';

/// Audio mixing configuration
class AudioMixingConfig {
  /// Audio mixing playback type
  AudioMixingType type;

  /// Playback times
  ///
  /// + playCount <= 0: Infinite loop
  /// + playCount == 1:Play once(default)
  /// + playCount > 1: Play `playCount` times
  int playCount;

  /// The position of the audio file playback progress bar during audio mixing, the parameter should be an integer in milliseconds.
  int position;

  /// Frequency (ms) of audio file playback progress callback. [RTCVideoEventHandler.onAudioMixingPlayingProgress] will be triggered then according to the set value, no callback by default.
  ///
  /// + The value is a multiple of 10 greater than 0. When the value set is not divisible by 10, it will be automatically rounded up by 10. For example, if the value is set to 52ms, it will be automatically adjusted to 60ms.
  /// + If the value is less than or equals to 0, no callback will be triggered.
  int callbackOnProgressInterval;

  /// Attaches the process information of local audio file mixing to the captured audio data. Enable the function to enhance the synchronicity of the remote audio mixing.
  ///
  /// + The function is effective when mixing a single audio file. <br>
  /// + Use `true` for enabling the function and `false` for disable the function. The default is `false`.
  bool syncProgressToRecordFrame;

  /// @nodoc
  AudioMixingConfig({
    this.type = AudioMixingType.playoutAndPublish,
    this.playCount = 0,
    this.position = 0,
    this.callbackOnProgressInterval = 0,
    this.syncProgressToRecordFrame = false,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'type': type.index,
      'playCount': playCount,
      'position': position,
      'callbackOnProgressInterval': callbackOnProgressInterval,
      'syncProgressToRecordFrame': syncProgressToRecordFrame,
    };
  }
}

/// Audio mixing management
///
/// In ios application development, when using audio mixing function, you must call [setActive:withOptions:error:](https://developer.apple.com/documentation/avfaudio/avaudiosession/1616627-setactive?language=objc) to activate the app's audio session. You can deactivate the audio session after quitting audio mixing scenario.
@Deprecated(
    'Deprecated since v3.54.1, use RTCAudioEffectPlayer and RTCMediaPlayer instead')
abstract class RTCAudioMixingManager {
  /// Starts mixing audio files.
  ///
  /// [mixId] is used for identifying the mixing task.
  /// If you have already preloaded the file in memory with [RTCAudioMixingManager.preloadAudioMixing], please ensure that the mixID is the same.
  /// If this API is repeatedly called with the same ID, the previous mixing task will be stopped and a new task will start. When the previous task is stopped, you will receive [RTCVideoEventHandler.onAudioMixingStateChanged].
  ///
  /// [filePath] is the path of the mixing audio file.
  /// You can use the URL of the online file, and the absolute path of the local file. For the URL of the online file, only https protocol is supported.
  /// Recommended audio sample rates: 8KHz, 16KHz, 22.05KHz, 44.1KHz, 48KHz.
  /// Local audio file formats supported by different platforms are as follows:
  /// <table border>
  ///    <tr><th></th><th>mp3</th><th>mp4</th><th>aac</th><th>m4a</th><th>3gp</th><th>wav</th><th>ogg</th><th>ts</th><th>wma</th></tr>
  ///    <tr><td>Android</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td></td></tr>
  ///    <tr><td>iOS</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td></td><td></td></tr>
  /// </table>
  /// Online audio file formats supported by different platforms are as follows:
  /// <table border>
  ///    <tr><th></th><th>mp3</th><th>mp4</th><th>aac</th><th>m4a</th><th>3gp</th><th>wav</th><th>ogg</th><th>ts</th><th>wma</th></tr>
  ///    <tr><td>Android</td><td>Y</td><td></td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td></td><td></td></tr>
  ///    <tr><td>iOS</td><td>Y</td><td></td><td>Y</td><td>Y</td><td></td><td>Y</td><td></td><td></td><td></td></tr>
  /// </table>
  ///
  /// Notes:
  /// + You can call this API with different mixIDs and filePaths to mix multiple audio files at the same time.
  /// + After calling this API, you will receive [RTCVideoEventHandler.onAudioMixingStateChanged] about the current mixing status.<br>
  /// + Call [RTCAudioMixingManager.stopAudioMixing] to stop the mixing task.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.start, RTCMediaPlayer.open and RTCMediaPlayer.start instead')
  Future<void> startAudioMixing({
    required int mixId,
    required String filePath,
    required AudioMixingConfig config,
  });

  /// Stops the specified mixing task.
  ///
  /// Notes:
  /// + After calling this API, [RTCVideoEventHandler.onAudioMixingStateChanged] will be triggered.
  /// + After calling this API to stop mixing task, the music file will be automatically uninstalled.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.stop and RTCMediaPlayer.stop instead')
  Future<void> stopAudioMixing(int mixId);

  /// Stops playing all audio files.
  ///
  /// Notes:
  /// + After calling [startAudioMixing] to play audio files, you can call this api to stop playing all the files.
  /// + After calling this api to stop playing all audio files, you will receive [RTCVideoEventHandler.onAudioMixingStateChanged] callback to inform you that the playing has been stopped.
  /// + After you call this api to stop playing all audio files, the files will be automatically uninstalled.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.stopAll instead')
  Future<void> stopAllAudioMixing();

  /// Pauses the specified mixing task.
  ///
  /// Notes:
  /// + After calling [RTCAudioMixingManager.startAudioMixing] to start playing music files and mixing audios, you can call this API to pause the task. <br>
  /// + Call [RTCAudioMixingManager.resumeAudioMixing] to resume the task.<br>
  /// + After calling this API, [RTCVideoEventHandler.onAudioMixingStateChanged] will be triggered.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.pause and RTCMediaPlayer.pause instead')
  Future<void> pauseAudioMixing(int mixId);

  /// Pauses all audio files.
  ///
  /// Notes:
  /// + After calling [startAudioMixing] to play audio files, you can call this api to pause all the files.
  /// + After calling this api to pause all audio, you can call [resumeAllAudioMixing] to resume the playing.
  /// + After calling this api to pause all audio, you will receive [RTCVideoEventHandler.onAudioMixingStateChanged] callback to inform you that the playing has been paused.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.pauseAll instead')
  Future<void> pauseAllAudioMixing();

  /// Resumes the mixing task that you have paused.
  ///
  /// + After calling [RTCAudioMixingManager.pauseAudioMixing], you can call this API to resume the task.<br>
  /// + After calling this API, [RTCVideoEventHandler.onAudioMixingStateChanged] will be triggered.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.resume and RTCMediaPlayer.resume instead')
  Future<void> resumeAudioMixing(int mixId);

  /// Resumes playing all audio files.
  ///
  /// + After calling [pauseAudioMixing]/[pauseAllAudioMixing], you can call this api to resume playing all the files.
  /// + After calling this api to resume playing all audio files, [RTCVideoEventHandler.onAudioMixingStateChanged] callback will be triggered to inform you that the playing has been resumed.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.resumeAll instead')
  Future<void> resumeAllAudioMixing();

  /// Preloads the audio file into memory to minimize the loading cost of playing repeatedly.
  ///
  /// If you call [RTCAudioMixingManager.startAudioMixing] first and then call this API with the same ID, the previous mixing task will be stopped, and then the next file will be loaded. <br>
  /// After calling this API to preload A.mp3, if you need to call [RTCAudioMixingManager.startAudioMixing] to play B.mp3 with the same ID, make sure to call [RTCAudioMixingManager.unloadAudioMixing] to unload A.mp3 first.
  ///
  /// You can use the URL of the online file, and the absolute path of the local file. For the URL of the online file, only https protocol is supported. <br>
  /// Audio file formats supported by different platforms are as follows:
  /// <table border>
  /// <tr><th></th><th>mp3</th><th>mp4</th><th>aac</th><th>m4a</th><th>3gp</th><th>wav</th><th>ogg</th></tr>
  /// <tr><td>Android</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td></tr>
  /// <tr><td>iOS</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td></tr>
  /// </table>
  ///
  /// Notes:
  /// + You can only preload the audio file of length less than 20s, otherwise the [RTCVideoEventHandler.onAudioMixingStateChanged] will be triggered notifying the preloading failure.
  /// + You will also be notified of other states through [RTCVideoEventHandler.onAudioMixingStateChanged].
  /// +  Unload the preloaded file with [RTCAudioMixingManager.unloadAudioMixing].
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.preload instead')
  Future<void> preloadAudioMixing({
    required int mixId,
    required String filePath,
  });

  /// Uninstalls the specified music file.
  ///
  /// Whether the music file is playing or not, after calling this API to uninstall the file, the SDK will call back to notify that the mixing task has stopped. See [RTCVideoEventHandler.onAudioMixingStateChanged].
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.unload instead')
  Future<void> unloadAudioMixing(int mixId);

  /// Sets the default volume during audio mixing, which works for both audio file mixing and PCM mixing.
  ///
  /// [volume]: The ratio of the mixing volume to the original volume. The range is `[0, 400]`. The recommended range is `[0, 100]`.
  /// + 0: Mute.
  /// + 100: Original volume.
  /// + 400: Maximum volume (with overflow protection).
  ///
  /// [type]: Audio mixing type.
  ///
  /// This API does not work for the mixing ID for which the volume has been set specifically by [RTCAudioMixingManager.setAudioMixingVolume].
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.setVolumeAll instead')
  Future<void> setAllAudioMixingVolume({
    required int volume,
    required AudioMixingType type,
  });

  /// Adjusts the mixing volume, including audio files and PCM data.
  ///
  /// [volume] is the ratio of the mixing volume to the original volume. The range is `[0, 400]`. The recommended range is `[0, 100]`.
  /// + 0: Mute
  /// + 100: Original volume(default)
  /// + 400: Maximum volume (with overflow protection)
  ///
  /// Make sure you have called [RTCAudioMixingManager.preloadAudioMixing] or [RTCAudioMixingManager.startAudioMixing] before calling this API.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.setVolume and RTCMediaPlayer.setVolume instead')
  Future<void> setAudioMixingVolume({
    required int mixId,
    required int volume,
    required AudioMixingType type,
  });

  /// Gets the duration in ms of the music file.
  ///
  /// API call result:
  /// + `>0`: Success, SDK will return the duration of the music file, in milliseconds.
  /// + `<0`: Failure.
  ///
  /// Make sure you have called [RTCAudioMixingManager.preloadAudioMixing] or [RTCAudioMixingManager.startAudioMixing] before calling this API.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.getDuration and RTCMediaPlayer.getTotalDuration instead')
  Future<int?> getAudioMixingDuration(int mixId);

  /// Gets the actual time played of the audio file.
  ///
  /// Return value:
  /// + `>0`: The actual time played(in millisecond).
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + The actual time played excludes the time when the file is paused, sped up, rewound, and fast-forwarded. For example, if a song plays the first 1:30 minutes and plays another 2 minutes after being paused for 30 seconds or fast-forwarded to 2:00,the actual time played will be 3.5 minutes.
  /// + Before calling this API, you must call [RTCAudioMixingManager.startAudioMixing] to play the audio file.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCMediaPlayer.getPlaybackDuration instead')
  Future<int?> getAudioMixingPlaybackDuration(int mixId);

  /// Gets the music file playback progress in ms.
  ///
  /// API call result:
  /// + `>0`: Success, SDK will return the music file playback progress, in milliseconds.
  /// + `<0`: Failure.
  ///
  /// Make sure you have called [RTCAudioMixingManager.startAudioMixing] before calling this API.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.getPosition and RTCMediaPlayer.getPosition instead')
  Future<int?> getAudioMixingCurrentPosition(int mixId);

  /// Sets the starting position of the audio file for audio mixing.
  ///
  /// [position]: The starting position in ms. <br>
  /// You can get the length of the file with [RTCAudioMixingManager.getAudioMixingDuration]. The value of the position must be less than the length of the file.
  ///
  /// Make sure you have called [RTCAudioMixingManager.startAudioMixing] before calling this API.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCAudioEffectPlayer.setPosition and RTCMediaPlayer.setPosition instead')
  Future<void> setAudioMixingPosition({
    required int mixId,
    required int position,
  });

  /// Sets the channel mode of the current music file. Consistent wih the original file by default.
  ///
  /// Make sure you have called [RTCAudioMixingManager.startAudioMixing] before calling this API.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCMediaPlayer.setAudioDualMonoMode instead')
  Future<void> setAudioMixingDualMonoMode({
    required int mixId,
    required AudioMixingDualMonoMode mode,
  });

  /// Enables local playback of music files in a different key, mostly used in Karaoke scenarios.
  ///
  /// [pitch] is the value by which the adjusted pitch is raised/lowered from the original pitch, and takes the range `[-12, 12]`, with the default value being 0, i.e. no adjustment is made.<br>
  /// The difference in pitch between two adjacent values within the value range is a semitone, with positive values indicating an ascending tone and negative values indicating a descending tone, and the larger the absolute value you set, the more the pitch is raised or lowered.<br>
  /// Settings outside the range of values will fail and trigger [RTCVideoEventHandler.onAudioMixingStateChanged].
  ///
  /// Notes: Call this API between [RTCAudioMixingManager.startAudioMixing] and [RTCAudioMixingManager.stopAudioMixing].
  @Deprecated(
      'Deprecated since v3.54.1, use RTCMediaPlayer.setAudioPitch instead')
  Future<void> setAudioMixingPitch({
    required int mixId,
    required int pitch,
  });

  /// Sets the playback speed of the current audio file.
  ///
  /// [speed]: Ratio of playback speed to original speed in percentage. The range is `[50,200]`, the default value is `100`.  <br>
  /// Settings outside the range of values will fail and trigger [RTCVideoEventHandler.onAudioMixingStateChanged].
  ///
  /// Notes:
  /// + You should call this API after calling [RTCAudioMixingManager.startAudioMixing] and receiving [RTCVideoEventHandler.onAudioMixingStateChanged] indicating that the [AudioMixingState] is `playing` and the [AudioMixingError] is `ok`.  <br>
  /// + If you call this API after calling [RTCAudioMixingManager.stopAudioMixing]or [RTCAudioMixingManager.unloadAudioMixing], you will receive [RTCVideoEventHandler.onAudioMixingStateChanged] indicating that the [AudioMixingState] is `failed` and the [AudioMixingError] is `idNotFound`.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCMediaPlayer.setPlaybackSpeed instead')
  Future<int?> setAudioMixingPlaybackSpeed({
    required int mixId,
    required int speed,
  });

  /// If you need to call [RTCVideo.enableVocalInstrumentBalance] to adjust the volume of the audio file or PCM data used for audio mixing, you must import the original loudness value of the audio file or PCM data via this API.
  ///
  /// [loudness]: Original loudness of the audio file in lufs. The range is `[-70.0, 0.0]`.  <br>
  /// If the value is set to be less than -70.0lufs, it is automatically adjusted to -70.0lufs. If the value is set to be greater than 0.0lufs, SDK will not equalize the loudness. The default value is 1.0lufs, i.e. No loudness equalization effect.
  ///
  /// To avoid sudden volume changes during audio file playback, we recommend that you call this API before starting to play the audio file with [RTCAudioMixingManager.startAudioMixing].
  @Deprecated(
      'Deprecated since v3.54.1, use RTCMediaPlayer.setLoudness instead')
  Future<void> setAudioMixingLoudness({
    required int mixId,
    required double loudness,
  });

  /// Sets the interval of callbacks about audio file playback progress during audio mixing.
  ///
  /// You can set the interval for multiple IDs by calling this method multiple times and passing in different [mixId]s.
  ///
  /// [interval] The time interval (ms) of the audio file playback progress callback in milliseconds. <br>
  /// + The value of interval is a multiple of 10 greater than 0. When the value set is not divisible by 10, the default is rounded up by 10. For example, if the value is set to 52ms, it will be automatically adjusted to 60ms, then the SDK will trigger `onAudioMixingPlayingProgress` callback at the set interval.
  /// + If the value is less than or equals to 0, the callback will not be triggered.
  ///
  /// Notes:
  /// + Call this API between [RTCAudioMixingManager.startAudioMixing] and [RTCAudioMixingManager.stopAudioMixing].
  /// + If you want to set the interval of playback progress callbacks before the music file starts playing, you need to call [RTCAudioMixingManager.startAudioMixing] to set the interval in [AudioMixingConfig], and you can update the callback interval through this method after the audio file starts playing.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCMediaPlayer.setProgressInterval instead')
  Future<void> setAudioMixingProgressInterval({
    required int mixId,
    required int interval,
  });

  /// Gets the track index of the current audio file.
  ///
  /// API call result
  /// + `≥0`: Success, SDK will return the track index.
  /// + `<0`:Failure
  ///
  /// Make sure you have called [RTCAudioMixingManager.startAudioMixing] before calling this API.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCMediaPlayer.getAudioTrackCount instead')
  Future<int?> getAudioTrackCount(int mixId);

  /// Specifies the playback track of the current audio file.
  ///
  /// [audioTrackIndex] should be less than or equal to the returned value of [RTCAudioMixingManager.getAudioTrackCount].
  ///
  /// Make sure you have called [RTCAudioMixingManager.startAudioMixing] before calling this API.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCMediaPlayer.selectAudioTrack instead')
  Future<void> selectAudioTrack({
    required int mixId,
    required int audioTrackIndex,
  });
}
