// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'dart:typed_data';

import '../src/base/bytertc_enum_convert.dart';
import 'bytertc_audio_defines.dart';
import 'bytertc_room_api.dart';

/// Stream mixing type. (New)
enum MixedStreamType {
  /// Mix media streams on the server side.
  byServer,

  /// @nodoc(`Not available`)
  /// Intelligent stream mixing. The SDK will intelligently decide that a stream mixing task would be done on the client or the server side.
  byClient,
}

/// Events during pushing streams to CDN. (New)
enum MixedStreamEvent {
  /// @nodoc('For internal use')
  /// Undefined event.
  base,

  /// Request to start the task of pushing streams to CDN.
  start,

  /// The task to push streams to CDN started.
  startSuccess,

  /// Failed to start the task to push streams to CDN.
  startFailed,

  /// Request to update the configuration for the task to push streams to CDN.
  update,

  /// Successfully update the configuration for the task to push streams to CDN.
  updateSuccess,

  /// Failed to update the configuration for the task to push streams to CDN.
  updateFailed,

  /// Request to stop the task to push streams to CDN.
  stop,

  /// The task to push streams to CDN stopped.
  stopSuccess,

  /// Failed to stop the task to push streams to CDN.
  stopFailed,

  /// Timeout for the request to update the configuration for the task to push streams to CDN.
  changeMixType,

  /// Got the first frame of the mixed audio stream by client.
  firstAudioFrameByClientMix,

  /// Got the first frame of the mixed video stream by client.
  firstVideoFrameByClientMix,

  /// Timeout for the request to update the configuration for the task to push streams to CDN.
  updateTimeout,

  /// Timeout for the request to start the task to push streams to CDN.
  startTimeout,

  /// Error in the parameters of the request for the task to push streams to CDN.
  requestParamError,

  /// Mixing image.
  mixImage,
}

/// Errors occurring during pushing streams to CDN. (New)
enum MixedStreamErrorCode {
  /// Successfully pushed streams to target CDN.
  ok,

  /// Undefined error
  base,

  /// Invalid parameters detected by Client SDK.
  invalidParam,

  /// Program runs with an error, the state machine is in abnormal condition.
  invalidState,

  /// Invalid operation.
  invalidOperator,

  /// Request timed out. Please check network status and retry.
  timeOut,

  /// Invalid parameters detected by the server.
  invalidParamByServer,

  /// Your subscription to the stream has expired.
  subTimeoutByServer,

  /// Internal server error.
  invalidStateByServer,

  /// The server failed to push streams to CDN.
  authenticationByCDN,

  /// Signaling connection timeout error. Please check network status and retry.
  timeoutBySignaling,

  /// Fail to mix image.
  mixImageFail,

  /// Unknown error from server.
  unKnownErrorByServer,
}

/// The video codec.(New)
enum MixedStreamVideoCodecType {
  /// H.264 format. Default value.
  h264,

  /// ByteVC1 format.
  byteVC1,
}

/// Video transcoding configurations.(New)<br>
/// With invalid or empty input, the configurations will be set as the default values.
class MixedStreamVideoConfig {
  /// The video codec. The default value is `0`.
  ///
  /// These parameters cannot be updated while pushing stream to the CDN.
  final MixedStreamVideoCodecType videoCodec;

  /// The frame rate (FPS) in range of [1, 60]. The default value is 15 FPS.
  final int fps;

  /// The time interval between I-frames (second) in range of [1, 5]. The default value is 2 seconds.
  ///
  /// These parameters cannot be updated while pushing stream to the CDN.
  final int gop;

  /// The bitrate (Kbps) in range of [1, 10000]. The default value is self-adaptive.
  final int bitrate;

  /// The width (pixels) to be set. The range is [2, 1920], and must be an even number. The default value is 640 pixels.<br>
  /// If an odd number is set, the width will be adjusted to the next larger even number.
  final int width;

  /// The height (pixels) to be set. The range is [2, 1920], and must be an even number. The default value is 360 pixels.
  /// If an odd number is set, the height will be adjusted to the next larger even number.
  final int height;

  /// Whether to push streams with B frame, only supported during audio mixing on the server side.
  final bool enableBFrame;

  /// @nodoc
  const MixedStreamVideoConfig({
    this.videoCodec = MixedStreamVideoCodecType.h264,
    this.fps = 15,
    this.gop = 2,
    this.bitrate = 500,
    this.width = 360,
    this.height = 640,
    this.enableBFrame = false,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'videoCodec': videoCodec.index,
      'fps': fps,
      'gop': gop,
      'bitrate': bitrate,
      'width': width,
      'height': height,
      'enableBFrame': enableBFrame,
    };
  }
}

/// The audio codec.(New)
enum MixedStreamAudioCodecType {
  /// AAC format.
  acc,
}

/// Advanced Audio Coding (AAC) profile.(New)
enum MixedStreamAudioProfile {
  /// AAC Low-Complexity profile (AAC-LC). Default value.
  lc,

  /// HE-AAC v1 profile (AAC LC with SBR).
  hev1,

  /// HE-AAC v2 profile (AAC LC with SBR and Parametric Stereo).
  hev2,
}

/// Audio transcoding configurations.(New)<br>
/// With invalid or empty input, the configurations will be set as the default values.<br>
/// These parameters cannot be updated while pushing stream to the CDN.
class MixedStreamAudioConfig {
  /// The audio codec.
  final MixedStreamAudioCodecType audioCodec;

  /// The bitrate (Kbps) in range of [32, 192]. Defaults to 64 Kbps.
  final int bitrate;

  /// The sample rate (Hz). Supported sample rates: 32,000 Hz, 44,100 Hz, 48,000 Hz. Defaults to 48,000 Hz.
  final int sampleRate;

  /// The number of channels. Supported channels: 1 (single channel), 2 (dual channel).  Defaults to 2.
  final int channels;

  /// AAC profile. Default value is `0`.
  final MixedStreamAudioProfile audioProfile;

  /// @nodoc
  const MixedStreamAudioConfig({
    this.audioCodec = MixedStreamAudioCodecType.acc,
    this.bitrate = 64,
    this.sampleRate = 48000,
    this.channels = 2,
    this.audioProfile = MixedStreamAudioProfile.lc,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'audioCodec': audioCodec.index,
      'bitrate': bitrate,
      'sampleRate': sampleRate,
      'channels': channels,
      'audioProfile': audioProfile.index,
    };
  }
}

/// Spatial audio config when pushing to CDN.(New)
class MixedStreamSpatialConfig {
  /// Whether to enable the spatial audio effect when pushing to CDN.<br>
  /// when you enable the feature, set the `spatialPosition` of each [MixedStreamLayoutRegionConfig] for spatial audio effect.
  final bool enableSpatialRender;

  /// The spatial position of the audience.<br>
  /// The audience is the users who receive the audio stream from CDN.
  final Position audienceSpatialPosition;

  /// The orientation of the audience.<br>
  /// The audience is the users who receive the audio stream from CDN.
  final HumanOrientation audienceSpatialOrientation;

  /// @nodoc
  const MixedStreamSpatialConfig({
    this.enableSpatialRender = false,
    this.audienceSpatialPosition = const Position.zero(),
    this.audienceSpatialOrientation = const HumanOrientation.origin(),
  });

  /// @nodoc
  const MixedStreamSpatialConfig.disabled() : this();

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'enableSpatialRender': enableSpatialRender,
      'position': audienceSpatialPosition.toMap(),
      'orientation': audienceSpatialOrientation.toMap(),
    };
  }
}

/// Stream mixing content type.(New)
enum MixedStreamMediaType {
  /// The mixed stream contains audio and video.
  audioAndVideo,

  /// The mixed stream only contains audio.
  audioOnly,

  /// The mixed stream only contains video.
  videoOnly,
}

/// The render mode.(New)
enum MixedStreamRenderMode {
  /// Fill and Crop. Default value.<br>
  /// The video frame is scaled with fixed aspect ratio, until it completely fills the canvas. The region of the video exceeding the canvas will be cropped.
  hidden,

  /// Fit.<br>
  /// The video frame is scaled with fixed aspect ratio, until it fits just within the canvas. Other part of the canvas is filled with the background color.
  fit,

  /// Fill the canvas.<br>
  /// The video frame is scaled to fill the canvas. During the process, the aspect ratio may change.
  adaptive,
}

/// Stream type in the region.
enum MixedStreamVideoType {
  /// Mainstream. Streams that captured by the camera or microphone.
  main,

  /// Screen-sharing stream.
  screen,
}

/// Stream mixing region type.(New)
enum MixedStreamLayoutRegionType {
  /// The region is filled with a video stream.
  videoStream,

  /// The region is filled with a picture.
  image,
}

/// Image parameters for stream mixing.(New)
class MixedStreamLayoutRegionImageWaterMarkConfig {
  /// Width of the original image in px.
  final int imageWidth;

  /// Height of the original image in px.
  final int imageHeight;

  /// @nodoc
  const MixedStreamLayoutRegionImageWaterMarkConfig({
    this.imageWidth = 0,
    this.imageHeight = 0,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'imageWidth': imageWidth,
      'imageHeight': imageHeight,
    };
  }
}

/// Layout information for one of the video streams or image to be mixed.(New)<br>
/// After starting to push streams to CDN and mixing multiple video streams, you can set the layout information for each of them.
class MixedStreamLayoutRegionConfig {
  /// The user ID of the user who publishes the video stream. Required.
  final String uid;

  /// The room ID of the media stream. Required.<br>
  /// If the media stream is the stream forwarded by [RTCRoom.startForwardStreamToRooms], you must set the roomID to the room ID of the target room.
  final String roomId;

  /// The normalized horizontal coordinate value of the top left end vertex of the user's view, ranging within [0.0, 1.0). The default value is 0.0.
  final double locationX;

  /// The normalized vertical coordinate value of the top left end vertex of the user's view, ranging within [0.0, 1.0). The default value is 0.0.
  final double locationY;

  /// The normalized width of the user's view, ranging within [0.0, 1.0]. The default value is 1.0.
  final double widthProportion;

  /// The normalized height of the user's view, ranging within [0.0, 1.0]. The default value is 1.0.
  final double heightProportion;

  /// The layer on which the video is rendered. The range is [0, 100]. 0 for the bottom layer, and 100 for the top layer. The default value is 0.
  final int zOrder;

  /// The opacity in range of (0.0, 1.0]. The lower value, the more transparent. The default value is 1.0.
  final double alpha;

  /// The proportion of the radius to the width of the canvas. `0.0` by default.
  ///
  /// After you set the value, `width_px`, `height_px`, and `cornerRadius_px` are calculated based on `width`, `height`, `cornerRadius`, and the width of the canvas. If `cornerRadius_px < min(width_px/2, height_px/2)` is met, the value of `cornerRadius` is set valid; if not, `cornerRadius_px` is set to `min(width_px/2, height_px/2)`, and `cornerRadius` is set to the proportion of `cornerRadius_px` to the width of the canvas.
  final double cornerRadius;

  /// The stream mixing content type. The default value is `audioAndVideo`.
  final MixedStreamMediaType mediaType;

  /// The render mode. The default value is 1.
  final MixedStreamRenderMode renderMode;

  /// Whether the source user of the stream is a local user.
  final bool isLocalUser;

  /// The mixed stream type, the default value is `Main`.
  final MixedStreamVideoType streamType;

  /// Stream mixing region type.
  final MixedStreamLayoutRegionType regionContentType;

  /// The RGBA data of the mixing image. Put in null when mixing video streams.
  final Uint8List? imageWaterMark;

  /// Image parameters for stream mixing. Set it to null when mixing video streams.
  final MixedStreamLayoutRegionImageWaterMarkConfig? imageWaterMarkConfig;

  /// Spatial position.
  final Position spatialPosition;

  /// Sets whether a user applies spatial audio effects.
  final bool applySpatialAudio;

  /// @nodoc
  const MixedStreamLayoutRegionConfig({
    this.uid = '',
    this.roomId = '',
    this.locationX = 0.0,
    this.locationY = 0.0,
    this.widthProportion = 1.0,
    this.heightProportion = 1.0,
    this.zOrder = 0,
    this.alpha = 1.0,
    this.cornerRadius = 0.0,
    this.mediaType = MixedStreamMediaType.audioAndVideo,
    this.renderMode = MixedStreamRenderMode.hidden,
    this.isLocalUser = false,
    this.streamType = MixedStreamVideoType.main,
    this.regionContentType = MixedStreamLayoutRegionType.videoStream,
    this.imageWaterMark,
    this.imageWaterMarkConfig,
    this.spatialPosition = const Position.zero(),
    this.applySpatialAudio = true,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'uid': uid,
      'roomId': roomId,
      'locationX': locationX,
      'locationY': locationY,
      'widthProportion': widthProportion,
      'heightProportion': heightProportion,
      'zOrder': zOrder,
      'alpha': alpha,
      'cornerRadius': cornerRadius,
      'mediaType': mediaType.index,
      'renderMode': renderMode.value,
      'isLocalUser': isLocalUser,
      'streamType': streamType.index,
      'regionContentType': regionContentType.index,
      if (imageWaterMark != null) 'imageWaterMark': imageWaterMark,
      if (imageWaterMarkConfig != null)
        'imageWaterMarkConfig': imageWaterMarkConfig!.toMap(),
      'spatialPosition': spatialPosition.toMap(),
      'applySpatialAudio': applySpatialAudio,
    };
  }
}

/// The overall video layout of the mixed stream.(New)
///
/// After starting to push streams to CDN, you can set the layout of each video stream, the background information of the mixed stream, etc.
class MixedStreamLayoutConfig {
  /// List of user's video layout information. The specific layout of each stream is detailed in [MixedStreamLayoutRegionConfig].
  ///
  /// If the value is invalid or empty, the configurations will be set to default values.
  final List<MixedStreamLayoutRegionConfig> regions;

  /// Info passed through from the user.
  final String userConfigExtraInfo;

  /// Background-color of the mixed stream in hexadecimal values such as #FFFFFF and #000000. The default value is #000000 (black).
  ///
  /// If the value is invalid or empty, the configurations will be set to default values.
  final String backgroundColor;

  /// @nodoc
  const MixedStreamLayoutConfig({
    this.regions = const [],
    this.userConfigExtraInfo = '',
    this.backgroundColor = '#000000',
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'regions': regions.map((e) => e.toMap()).toList(growable: false),
      'userConfigExtraInfo': userConfigExtraInfo,
      'backgroundColor': backgroundColor,
    };
  }
}

/// Content of SEI sent while pushing mixed stream to CDN.
enum MixedStreamSEIContentMode {
  /// The video stream contains all the SEI information. Default value.
  def,

  /// The SEI data transmitted with non-key frames contains volume information only.
  ///
  /// This parameter only takes effect after you set [MixedStreamServerControlConfig.enableVolumeIndication] to "True".
  enableVolumeIndication,
}

/// Configurations to be set while mixing streams on the server side.
class MixedStreamServerControlConfig {
  /// Sets whether to enable the function of separately sending sound indication SEI:
  /// + True: Enable.
  /// + False: Disable. Default setting.
  ///
  /// After setting this parameter to "true", you can choose whether to send sound indication SEI only through [MixedStreamServerControlConfig.seiContentMode].
  final bool enableVolumeIndication;

  /// Sound indication interval in s. The range is [0.3,+∞). The default value is 2.
  ///
  /// If a decimal with two or more decimal places is passed in, it will be rounded to the value of the first decimal place. For example, if you pass in 0.36, it will be automatically adjusted to 0.4.
  final double volumeIndicationInterval;

  /// Effective volume within the range of [0, 255]. The default value is 0.
  ///
  /// If the value is set beyond the range, it will be automatically adjusted to the default value 0.
  final int talkVolume;

  /// Sets whether volume value is contained in the sound indication SEI:
  /// + True: Yes.
  /// + False: No. Default setting.
  final bool isAddVolumeValue;

  /// Sets SEI content.
  final MixedStreamSEIContentMode seiContentMode;

  /// SEI payload type.
  ///
  /// The default value is `100`, and the value supported is `5` and `100`.<br>
  /// During the process of pushing streams to CDN, you cannot change the parameter.
  final int seiPayloadType;

  /// SEI payload UUID.
  ///
  /// Notes:
  /// + When PayloadType is `5`, you must set PayloadUUID, or you will receive a callback indicating parameter error.
  /// + When PayloadType is not `5`, it is not required to set PayloadUUID. If filled, it will be ignored by the backend.
  /// + The length of PayloadUUID should be 32 bits, or you will receive a callback indicating parameter error.
  /// + Each character of the parameter should be within the range of [0, 9] [a, f] [A, F].
  /// + During the process of pushing streams to CDN, you cannot change the parameter.
  final String seiPayloadUUID;

  /// @nodoc
  const MixedStreamServerControlConfig({
    this.enableVolumeIndication = false,
    this.volumeIndicationInterval = 2.0,
    this.talkVolume = 0,
    this.isAddVolumeValue = false,
    this.seiContentMode = MixedStreamSEIContentMode.def,
    this.seiPayloadType = 100,
    this.seiPayloadUUID = '',
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'enableVolumeIndication': enableVolumeIndication,
      'volumeIndicationInterval': volumeIndicationInterval,
      'talkVolume': talkVolume,
      'isAddVolumeValue': isAddVolumeValue,
      'seiContentMode': seiContentMode.index,
      'seiPayloadType': seiPayloadType,
      'seiPayloadUUID': seiPayloadUUID,
    };
  }
}

/// Configurations to be set when pushing media streams to CDN.(New)
class MixedStreamConfig {
  /// The URL for live transcoding. Only supports live transcoding via RTMP. The URL should match the regular expression `/^rtmps?:\/\//`.
  ///
  /// This parameter cannot be updated while pushing stream to the CDN.
  final String pushURL;

  /// The room ID for live transcoding. The sum length of `roomID` and `userID` should not exceed 126 bytes.
  ///
  /// This parameter cannot be updated while pushing stream to the CDN.
  final String roomId;

  /// The user ID for live transcoding. The sum length of `roomID` and `userID` should not exceed 126 bytes.
  ///
  /// This parameter cannot be updated while pushing stream to the CDN.
  final String uid;

  /// Stream mixing type. Only mixing on the server side is supported.
  final MixedStreamType expectedMixingType;

  /// Video mixing related configurations.
  final MixedStreamVideoConfig videoConfig;

  /// Audio mixing related configurations.
  final MixedStreamAudioConfig audioConfig;

  /// Configurations to be set while mixing streams on the server side.
  final MixedStreamServerControlConfig serverControlConfig;

  /// The spatial audio config when pushing to CDN.
  final MixedStreamSpatialConfig spatialConfig;

  /// The overall video layout of the mixed stream.<br>
  /// After starting to push streams to CDN, you can set the layout of each video stream, the background information of the mixed stream, etc.
  final MixedStreamLayoutConfig layout;

  /// @nodoc
  const MixedStreamConfig({
    required this.pushURL,
    required this.roomId,
    required this.uid,
    this.expectedMixingType = MixedStreamType.byServer,
    this.videoConfig = const MixedStreamVideoConfig(),
    this.audioConfig = const MixedStreamAudioConfig(),
    this.serverControlConfig = const MixedStreamServerControlConfig(),
    this.spatialConfig = const MixedStreamSpatialConfig.disabled(),
    this.layout = const MixedStreamLayoutConfig(),
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'pushURL': pushURL,
      'roomId': roomId,
      'uid': uid,
      'expectedMixingType': expectedMixingType.index,
      'videoConfig': videoConfig.toMap(),
      'audioConfig': audioConfig.toMap(),
      'serverControlConfig': serverControlConfig.toMap(),
      'spatialConfig': spatialConfig.toMap(),
      'layout': layout.toMap(),
    };
  }
}

/// [eventType]: Task status.
///
/// [taskId]: Task ID.
///
/// [error]: Errors occurring when pushing stream to CDN.
///
/// [mixType]: Task type.
typedef OnMixingEventType = void Function(MixedStreamEvent eventType,
    String taskId, MixedStreamErrorCode error, MixedStreamType mixType);

/// Register this observer to receive stream mixing related callbacks from the SDK.
///
/// Valid since v3.56.1
class RTCMixedStreamObserver {
  /// Used for reporting events during pushing streams to CDNs.
  OnMixingEventType? onMixingEvent;

  /// @nodoc
  RTCMixedStreamObserver({
    this.onMixingEvent,
  });
}

/// Status when pushing a single media stream to CDN.
enum StreamSinglePushEvent {
  /// @nodoc('For internal use')
  /// Undefined status.
  base,

  /// Request to start the task to push a single stream to CDN.
  start,

  /// The task to a single stream to CDN started.
  success,

  /// Failed to start the task to push a single stream to CDN.
  failed,

  /// The task to a single stream to CDN stopped.
  stop,

  /// Timeout for the request to start the task to push a single stream to CDN.
  timeout,

  /// Invalid parameter.
  paramError,
}

/// Configurations for pushing a single media stream to CDN.
class PushSingleStreamParam {
  /// ID of the room from which the media stream is pushed.
  String roomId;

  /// ID of the user who publishes the stream.
  String uid;

  /// The target address of the CDN.
  String url;

  /// Whether the media stream is a screen-sharing stream.
  bool isScreen;

  /// @nodoc
  PushSingleStreamParam({
    required this.roomId,
    required this.uid,
    required this.url,
    this.isScreen = false,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'roomId': roomId,
      'uid': uid,
      'url': url,
      'isScreen': isScreen,
    };
  }
}

/// [eventType]: Task status
///
/// [taskId]: Task ID
///
/// [error]: Error code
typedef OnStreamPushEventType = void Function(
    StreamSinglePushEvent eventType, String taskId, int error);

/// Observer in pushing single stream to CDN
class RTCPushSingleStreamToCDNObserver {
  /// Status callback
  OnStreamPushEventType? onStreamPushEvent;

  /// @nodoc
  RTCPushSingleStreamToCDNObserver({
    this.onStreamPushEvent,
  });
}
