// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import '../src/base/bytertc_enum_convert.dart';

/// @nodoc('Not available')
/// The filter type of the music list.
enum MusicFilterType {
  /// No filter.
  none,

  /// Remove music that does not have lyrics.
  withoutLyric,

  /// Remove music that does not support scoring.
  unsupportedScore,

  /// Remove music that does not support accompany mode.
  unsupportedAccompany,

  /// Remove music that does not have a climax part.
  unsupportedClimax,
}

/// @nodoc('Not available')
/// Hot music type.
enum MusicHotType {
  /// Hot music in the content center.
  contentCenter,

  /// Hot music in your project.
  project,
}

/// @nodoc('Not available')
/// Music playing status.
enum PlayState {
  /// Playing
  playing,

  /// Paused
  paused,

  /// Stopped
  stopped,

  /// Failed
  failed,

  /// Finished
  finished,
}

/// @nodoc('Not available')
/// Audio track type of the KTV player.
enum AudioTrackType {
  /// Play the original music with vocals.
  original,

  /// Play the instrumental music without vocals.
  accompany,
}

/// @nodoc('Not available')
/// Audio play type.
enum AudioPlayType {
  /// Only play on the local side.
  local,

  /// Only play on the remote side.
  remote,

  /// Play on both local and remote side.
  localAndRemote,
}

/// @nodoc('Not available')
/// The lyrics type.
enum LyricStatus {
  /// No lyrics.
  none,

  /// KRC lyrics.
  krc,

  /// LRC lyrics.
  lrc,

  /// KRC And LRC lyrics.
  krcAndLrc,
}

/// @nodoc('Not available')
/// Download file type.
enum DownloadFileType {
  /// Audio file.
  music,

  /// KRC lyrics file.
  krc,

  /// LRC lyrics file.
  lrc,

  /// MIDI file.
  midi,
}

/// @nodoc('Not available')
/// The lyrics file's format.
enum DownloadLyricType {
  /// KRC lyrics file.
  krc,

  /// LRC lyrics file.
  lrc,
}

/// @nodoc('Not available')
/// KTV error code.
enum KTVErrorCode {
  /// Success.
  ok,

  /// Invalid AppID.
  appIdInValid,

  /// Invalid parameter.
  parasInValid,

  /// Failed to get music resources.
  getMusicFailed,

  /// Failed to get lyrics.
  getLyricFailed,

  /// The music is removed.
  musicTakedown,

  /// Failed to download the music file.
  musicDownload,

  /// Failed to download the MIDI file.
  midiDownloadFailed,

  /// The system is busy.
  systemBusy,

  /// Network anomaly.
  network,

  /// The KTV feature is not added to the room.
  notJoinRoom,

  /// Failed to parse data.
  parseData,

  /// Download error.
  download,

  /// Already downloading.
  downloading,

  /// Internal error. Contact the technical support representatives for help.
  internal,

  /// Failed to download because of insufficient disk space. Please retry after clearing cache.
  insufficientDiskSpace,

  /// Failed to download because of music decryption failure. Contact the technical support for help.
  musicDecryptionFailed,

  /// Failed to download because of music rename failure. Please retry.
  fileRenameFailed,

  /// Failed to download because of network failure. Please retry.
  downloadTimeOut,

  /// Failed to clear cache because the file is occupied or the system is abnormal. Please retry.
  clearCacheFailed,

  /// Download task canceled.
  downloadCanceled,
}

/// @nodoc('Not available')
/// KTV player error code.
enum KTVPlayerErrorCode {
  /// Success
  ok,

  /// Failed to play the music. Download first.
  fileNotExist,

  /// Failed to play the music. Check the file's format.
  fileError,

  /// Failed to play the music. Join a room first.
  notJoinRoom,

  /// Invalid parameter.
  param,

  /// Failed to play the music. Invalid path or failed to open the file.
  startError,

  /// Invalid mixing ID.
  mixIdError,

  /// Invalid position.
  positionError,

  /// Invalid volume. The range of values that can be set is `[0,400]`.
  audioVolumeError,

  /// The mix type is not supported.
  typeError,

  /// Invalid pitch.
  pitchError,

  /// Invalid audio track.
  audioTrackError,

  /// Mixing in process.
  startingError,
}

/// @nodoc('Not available')
/// Music information.
class MusicInfo {
  /// Music ID.
  final String musicId;

  /// Music name.
  final String musicName;

  /// Singer.
  final String singer;

  /// Vendor ID.
  final String vendorId;

  /// Vendor name.
  final String vendorName;

  /// Latest update timestamp in milliseconds.
  final int updateTimestamp;

  /// The URL of the music cover.
  final String posterUrl;

  /// The type of lyrics.
  final LyricStatus lyricStatus;

  /// The length of the song in milliseconds.
  final int duration;

  /// Whether scoring can be used to the song.
  final bool enableScore;

  /// The start time of the climax part in milliseconds.
  final int climaxStartTime;

  /// The end time of the climax part in milliseconds.
  final int climaxEndTime;

  /// @nodoc
  const MusicInfo({
    required this.musicId,
    required this.musicName,
    required this.singer,
    required this.vendorId,
    required this.vendorName,
    required this.updateTimestamp,
    required this.posterUrl,
    required this.lyricStatus,
    required this.duration,
    required this.enableScore,
    required this.climaxStartTime,
    required this.climaxEndTime,
  });

  /// @nodoc
  factory MusicInfo.fromMap(Map<dynamic, dynamic> map) {
    return MusicInfo(
      musicId: map['musicId'],
      musicName: map['musicName'],
      singer: map['singer'],
      vendorId: map['vendorId'],
      vendorName: map['vendorName'],
      updateTimestamp: map['updateTimestamp'],
      posterUrl: map['posterUrl'],
      lyricStatus: (map['lyricStatus'] as int?).ktvLyricStatus,
      duration: map['duration'],
      enableScore: map['enableScore'],
      climaxStartTime: map['climaxStartTime'],
      climaxEndTime: map['climaxEndTime'],
    );
  }
}

/// Hot music information.
class HotMusicInfo {
  /// Hot music type.
  final MusicHotType hotType;

  /// Hot list name.
  final String? hotName;

  /// Music information.
  final List<MusicInfo>? musicInfos;

  /// @nodoc
  const HotMusicInfo({
    required this.hotType,
    this.hotName,
    this.musicInfos,
  });

  /// @nodoc
  factory HotMusicInfo.fromMap(Map<dynamic, dynamic> map) {
    return HotMusicInfo(
      hotType: (map['hotType'] as int?).ktvMusicHotType,
      hotName: map['hotName'],
      musicInfos: (map['musicInfos'] as List?)
          ?.map((e) => MusicInfo.fromMap(e))
          .toList(growable: false),
    );
  }
}

/// Download music information.
class DownloadResult {
  /// Music ID.
  final String musicId;

  /// Download file type.
  final DownloadFileType fileType;

  /// Download file path.
  final String? filePath;

  /// @nodoc
  const DownloadResult({
    required this.musicId,
    required this.fileType,
    this.filePath,
  });

  /// @nodoc
  factory DownloadResult.fromMap(Map<dynamic, dynamic> map) {
    return DownloadResult(
      musicId: map['musicId'],
      fileType: (map['fileType'] as int?).ktvDownloadFileType,
      filePath: map['filePath'],
    );
  }
}
