// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'dart:async';

import 'package:async/async.dart';

import 'bytertc_ktv_defines.dart';
import 'bytertc_ktv_player_api.dart';

/// @nodoc('Not Available')
/// [musicInfos]: Array of the music information.
///
/// [totalSize]: The total size of the music list.
///
/// [errorCode]: Error code.
typedef OnMusicListResultType = void Function(
    List<MusicInfo>? musicInfos, int totalSize, KTVErrorCode errorCode);

/// @nodoc('Not Available')
/// [hotMusics]: Array of the hot music information.
///
/// [errorCode]: Error code.
typedef OnHotMusicResultType = void Function(
    List<HotMusicInfo>? hotMusics, KTVErrorCode errorCode);

/// @nodoc('Not Available')
/// [musicInfo]: Music information.
///
/// [errorCode]: Error code.
typedef OnMusicDetailResultType = void Function(
    MusicInfo? musicInfo, KTVErrorCode errorCode);

/// @nodoc('Not available')
/// Error code.
typedef OnClearCacheResultType = void Function(KTVErrorCode errorCode);

/// @nodoc('Not available')
/// KTV manager event handler.
class RTCKTVManagerEventHandler {
  /// Callback for the music list.
  OnMusicListResultType? onMusicListResult;

  /// Callback for searching song results.
  OnMusicListResultType? onSearchMusicResult;

  /// Callback for hot song searching results.
  OnHotMusicResultType? onHotMusicResult;

  /// Callback for detailed music information.
  OnMusicDetailResultType? onMusicDetailResult;

  /// Callback for clear Cache result callback.
  OnClearCacheResultType? onClearCacheResult;

  /// @nodoc
  RTCKTVManagerEventHandler({
    this.onMusicListResult,
    this.onSearchMusicResult,
    this.onHotMusicResult,
    this.onMusicDetailResult,
    this.onClearCacheResult,
  });
}

/// @nodoc('Not available')
/// KTV manager API class.
abstract class RTCKTVManager {
  /// Sets the KTV event handler.
  void setKTVManagerEventHandler(RTCKTVManagerEventHandler? eventHandler);

  /// Gets the music list.
  ///
  /// [pageNum]: Page number. The default value is 1.
  ///
  /// [pageSize]: The number of the music that displays on one page.
  ///
  /// [filters]: The filter type of the music list. Multiple filters can be combined by the bitwise-or operator.
  ///
  /// After calling this API, you will receive the music list through [RTCKTVManagerEventHandler.onMusicListResult] callback.
  Future<void> getMusicList({
    int pageNum,
    int pageSize,
    List<MusicFilterType> filters,
  });

  /// Searches music by keywords.
  ///
  /// [keyWord]: Keywords in string of up to 20 characters in length.
  ///
  /// [pageNum]: Page number. The default value is 1.
  ///
  /// [pageSize]: The number of the music that displays on one page.
  ///
  /// [filters]: The filter type of the music list. Multiple filters can be combined by the bitwise-or operator.
  ///
  /// After calling this API, you will receive the music list through [RTCKTVManagerEventHandler.onSearchMusicResult] callback.
  Future<void> searchMusic({
    required String keyWord,
    int pageNum,
    int pageSize,
    List<MusicFilterType> filters,
  });

  /// Gets hot music according to music types.
  ///
  /// [hotTypes]: Hot music type. Multiple hot music types can be combined by the bitwise-or operator.
  ///
  /// [filters]: The filter type of the music list. Multiple filters can be combined by the bitwise-or operator.
  ///
  /// After calling this API, you will receive the music list through [RTCKTVManagerEventHandler.onHotMusicResult] callback.
  Future<void> getHotMusic({
    List<MusicHotType> hotTypes,
    List<MusicFilterType> filters,
  });

  /// Gets detailed music information.
  ///
  /// [musicId]: Music ID.
  ///
  /// After calling this API, you will receive the music detail through [RTCKTVManagerEventHandler.onMusicDetailResult] callback.
  Future<void> getMusicDetail(String musicId);

  /// Downloads music.
  ///
  /// [musicId]: Music ID.
  ///
  /// [onReceiveProgress]: Callback for music download progress, after setting which you will receive the download progress of the current music file in %. The range is [0,100].
  ///
  /// If this API is successfully called, you will receive a [DownloadResult] object, otherwise you will receive corresponding error code in [KTVErrorCode].
  CancelableOperation<DownloadResult> downloadMusic(
    String musicId, {
    void Function(int downloadProgress)? onReceiveProgress,
  });

  /// Downloads lyrics.
  ///
  /// [musicId]: Music ID.
  ///
  /// [lyricType]: The lyrics file's format.
  ///
  /// If this API is successfully called, you will receive a [DownloadResult] object, otherwise you will receive corresponding error code in [KTVErrorCode].
  CancelableOperation<DownloadResult> downloadLyric(
    String musicId, {
    DownloadLyricType lyricType,
  });

  /// Downloads MIDI files.
  ///
  /// [musicId]: Music ID.
  ///
  /// If this API is successfully called, you will receive a [DownloadResult] object, otherwise you will receive corresponding error code in [KTVErrorCode].
  CancelableOperation<DownloadResult> downloadMidi(String musicId);

  /// Clears music cache, including music and lyrics.
  Future<void> clearCache();

  /// Sets the maximum cache for storing music files.
  ///
  /// [maxCacheSizeMB]: The maximum cache to be set in MB.
  ///
  /// If the value is set to less than or equal to 0, it will be adjusted to 1,024 MB.
  Future<void> setMaxCacheSize(int maxCacheSizeMB);

  /// Gets the KTV player.
  FutureOr<RTCKTVPlayer?> getKTVPlayer();
}
