// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'dart:collection';
import 'dart:typed_data';

import '../src/base/bytertc_enum_convert.dart';
import 'bytertc_audio_defines.dart';
import 'bytertc_video_defines.dart';

/// Stream mixing type.
enum StreamMixingType {
  /// Mix media streams on the server side.
  byServer,

  /// @nodoc(`Not available`)
  /// Intelligent stream mixing. The SDK will intelligently decide that a stream mixing task would be done on the client or the server side.
  byClient,
}

/// Events during pushing streams to CDN.
enum StreamMixingEvent {
  /// @nodoc('For internal use')
  /// Undefined event.
  base,

  /// Request to start the task of pushing streams to CDN.
  start,

  /// The task to push streams to CDN started.
  startSuccess,

  /// Failed to start the task to push streams to CDN.
  startFailed,

  /// Request to update the configuration for the task to push streams to CDN.
  update,

  /// Successfully update the configuration for the task to push streams to CDN.
  updateSuccess,

  /// Failed to update the configuration for the task to push streams to CDN.
  updateFailed,

  /// Request to stop the task to push streams to CDN.
  stop,

  /// The task to push streams to CDN stopped.
  stopSuccess,

  /// Failed to stop the task to push streams to CDN.
  stopFailed,

  /// Timeout for the request to update the configuration for the task to push streams to CDN.
  changeMixType,

  /// Got the first frame of the mixed audio stream by client.
  firstAudioFrameByClientMix,

  /// Got the first frame of the mixed video stream by client.
  firstVideoFrameByClientMix,

  /// Timeout for the request to update the configuration for the task to push streams to CDN.
  updateTimeout,

  /// Timeout for the request to start the task to push streams to CDN.
  startTimeout,

  /// Error in the parameters of the request for the task to push streams to CDN.
  requestParamError,

  /// Mixing image.
  mixImage,
}

/// Errors occurring during pushing streams to CDN.
enum TranscoderErrorCode {
  /// Successfully pushed streams to target CDN.
  ok,

  /// Undefined error
  base,

  /// Invalid parameters detected by Client SDK.
  invalidParam,

  /// Program runs with an error, the state machine is in abnormal condition.
  invalidState,

  /// Invalid operation.
  invalidOperator,

  /// Request timed out. Please check network status and retry.
  timeOut,

  /// Invalid parameters detected by the server.
  invalidParamByServer,

  /// Your subscription to the stream has expired.
  subTimeoutByServer,

  /// Internal server error.
  invalidStateByServer,

  /// The server failed to push streams to CDN.
  authenticationByCDN,

  /// Signaling connection timeout error. Please check network status and retry.
  timeoutBySignaling,

  /// Fail to mix image.
  mixImageFail,

  /// Unknown error from server.
  unKnownErrorByServer,
}

/// Stream mixing region type.
enum TranscoderLayoutRegionType {
  /// The region type is a video stream.
  videoStream,

  /// The region type is an image.
  image,
}

/// Video codec type.
enum TranscodingVideoCodec {
  /// (Default) H.264 format.
  h264,

  /// ByteVC1 format.
  byteVC1,
}

/// Audio codec type.
enum TranscodingAudioCodec {
  /// AAC format.
  aac,
}

/// Advanced Audio Coding (AAC) profile
enum AACProfile {
  /// Low-Complexity profile (AAC-LC).
  lc,

  ///  HE-AAC profile (AAC LC with SBR)
  hev1,

  /// HE-AAC v2 profile (AAC LC with SBR and Parametric Stereo)
  hev2,
}

/// Video transcoding related configurations.
class LiveTranscodingVideoConfig {
  /// Video codec type. The default value is `h264`.
  ///
  /// This parameter cannot be updated while pushing stream to the CDN.
  TranscodingVideoCodec codec;

  /// The frame rate (FPS) in range of `[1, 60]`. The default value is 15 FPS.
  int fps;

  /// The time interval between I-frames (second) in range of `[1, 5]`. The default value is 2 seconds.
  int gop;

  /// Whether to push streams with B frame.
  bool bFrame;

  /// The bitrate (Kbps) in range of `[1, 10000]`. The default value is self-adaptive.
  int kBitrate;

  /// The width (pixels) to be set. The range is `[2, 1920]`, and must be an even number. The default value is 640 pixels.
  ///
  /// If an odd number is set, the width will be adjusted to the next larger even number.
  int width;

  /// The height (pixels) to be set. The range is `[2, 1920]`, and must be an even number. The default value is 360 pixels.
  ///
  /// If an odd number is set, the height will be adjusted to the next larger even number.
  int height;

  /// @nodoc
  LiveTranscodingVideoConfig({
    this.codec = TranscodingVideoCodec.h264,
    this.fps = 30,
    this.gop = 2,
    this.bFrame = false,
    this.kBitrate = 500,
    this.width = 360,
    this.height = 640,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'codec': codec.value,
      'fps': fps,
      'gop': gop,
      'bFrame': bFrame,
      'kBitrate': kBitrate,
      'width': width,
      'height': height,
    };
  }
}

/// Audio transcoding related configurations.
class LiveTranscodingAudioConfig {
  /// Audio codec type. `aac` by default.
  TranscodingAudioCodec codec;

  /// Mixed audio bitrate in kbps within a range of `[32, 192]`. 64 Kbps by default.
  int kBitrate;

  /// The sample rate(Hz). Supported sample rates: 32,000Hz, 44,100Hz, 48,000Hz. Defaults to 48,000Hz.
  int sampleRate;

  /// Number of channels, either 1 or 2.
  int channels;

  /// Advanced Audio Coding (AAC) profile.
  AACProfile aacProfile;

  /// @nodoc
  LiveTranscodingAudioConfig({
    this.codec = TranscodingAudioCodec.aac,
    this.kBitrate = 64,
    this.sampleRate = 48000,
    this.channels = 2,
    this.aacProfile = AACProfile.lc,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'codec': codec.value,
      'kBitrate': kBitrate,
      'sampleRate': sampleRate,
      'channels': channels,
      'aacProfile': aacProfile.value,
    };
  }
}

/// Image parameters for stream mixing
class TranscoderLayoutRegionDataParam {
  /// Width of the original image in px.
  int imageWidth;

  /// Height of the original image in px.
  int imageHeight;

  /// @nodoc
  TranscoderLayoutRegionDataParam({
    required this.imageWidth,
    required this.imageHeight,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'imageWidth': imageWidth,
      'imageHeight': imageHeight,
    };
  }
}

/// Information about layout of a single video stream in a mixed stream.
class LiveTranscodingRegion {
  /// ID of the user who publishes the video stream.
  String uid;

  /// Room ID (required).
  String roomId;

  /// The offset of the user video layout relative to the left side of the canvas. Relative value, in the range `[0.0, 1.0]`.
  double x;

  /// The offset of the user video layout relative to the top of the canvas. Relative value, in the range `[0.0, 1.0]`.
  double y;

  /// The ratio of the user video width relative to the user's original video width. Relative value, in the range `[0.0, 1.0]`.
  double w;

  /// The ratio of the user video height relative to the user's original video height. Relative value, in the range `[0.0, 1.0]`.
  double h;

  /// The layer of the user video layout in the canvas. 0 is the bottom layer, the larger the value the higher the layer, the range is `[0, 100]`.
  int zorder;

  /// The transparency in range of `[0.0, 1.0]`.
  double alpha;

  /// The proportion of the radius to the width of the canvas. The default value is `0.0`.
  ///
  /// Only server-side stream mixing can set this parameter.
  ///
  /// After you set the value, `width_px`, `height_px`, and `radius_px` are calculated based on `width`, `height`, `radius`, and the width of the canvas. If `radius_px < min(width_px/2, height_px/2)` is met, the value of `radius` is set valid; if not, `radius_px` is set to `min(width_px/2, height_px/2)`, and `radius` is set to the proportion of `radius_px` to the width of the canvas.
  double cornerRadius;

  /// The stream mixing content type.
  TranscoderContentControlType contentControl;

  /// Render mode
  VideoRenderMode renderMode;

  /// Whether the source user of the stream is a local user.
  bool localUser;

  /// Whether the stream comes from screen sharing.
  bool isScreen;

  /// Stream mixing region type.
  TranscoderLayoutRegionType type;

  /// The RGBA data of the mixing image.
  Uint8List? data;

  /// Image parameters for stream mixing.
  TranscoderLayoutRegionDataParam? dataParam;

  /// Spatial position.
  Position spatialPosition;

  /// Sets whether a user applies spatial audio effects.
  bool applySpatialAudio;

  /// @nodoc
  LiveTranscodingRegion({
    required this.uid,
    required this.roomId,
    required this.x,
    required this.y,
    required this.w,
    required this.h,
    this.zorder = 0,
    this.alpha = 1.0,
    this.cornerRadius = 0.0,
    this.contentControl = TranscoderContentControlType.hasAudioAndVideo,
    this.renderMode = VideoRenderMode.hidden,
    required this.localUser,
    this.isScreen = false,
    this.type = TranscoderLayoutRegionType.videoStream,
    this.data,
    this.dataParam,
    this.spatialPosition = const Position.zero(),
    this.applySpatialAudio = true,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    HashMap<String, dynamic> dic = HashMap.of({
      'uid': uid,
      'roomId': roomId,
      'x': x,
      'y': y,
      'w': w,
      'h': h,
      'zorder': zorder,
      'alpha': alpha,
      'cornerRadius': cornerRadius,
      'contentControl': contentControl.index,
      'renderMode': renderMode.value,
      'localUser': localUser,
      'isScreen': isScreen,
      'type': type.index,
      'spatialPosition': spatialPosition.toMap(),
      'applySpatialAudio': applySpatialAudio,
    });
    if (data != null) {
      dic['data'] = data;
    }
    if (dataParam != null) {
      dic['dataParam'] = dataParam?.toMap();
    }
    return dic;
  }
}

/// Layout information about pushing streams to CDN.
class LiveTranscodingLayout {
  /// Region information.
  List<LiveTranscodingRegion> regions;

  /// SEI information with 4096 bytes in length.
  String appData;

  /// The background color to be set, in RGB hex value. The default value is #000000 (black).
  String backgroundColor;

  /// @nodoc
  LiveTranscodingLayout({
    required this.regions,
    this.appData = '',
    this.backgroundColor = '#000000',
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'appData': appData,
      'backgroundColor': backgroundColor,
      'regions': regions.map((e) => e.toMap()).toList(),
    };
  }
}

/// Spatial audio config when pushing to CDN.
class LiveTranscodingSpatialConfig {
  /// The orientation of the audience.
  ///
  /// The audience refers to users who receive the audio stream from CDN.
  final HumanOrientation audienceSpatialOrientation;

  /// The spatial position of the audience.
  ///
  /// The audience refers to users who receive the audio stream from CDN.
  final Position audienceSpatialPosition;

  /// Whether to enable the spatial audio effect when pushing to CDN.
  ///
  /// When you enable the feature, set the `audienceSpatialPosition` of each [LiveTranscodingSpatialConfig] for spatial audio effect.
  final bool enableSpatialRender;

  /// @nodoc
  const LiveTranscodingSpatialConfig({
    this.audienceSpatialOrientation = const HumanOrientation.origin(),
    this.audienceSpatialPosition = const Position.zero(),
    this.enableSpatialRender = false,
  });

  /// @nodoc
  const LiveTranscodingSpatialConfig.disabled() : this();

  /// @nodoc
  Map<String, dynamic> toMap() => {
        'orientation': audienceSpatialOrientation.toMap(),
        'position': audienceSpatialPosition.toMap(),
        'enableSpatialRender': enableSpatialRender,
      };
}

/// Configurations of pushing streams to CDN.
class LiveTranscoding {
  /// Stream mixing type. This parameter cannot be updated while pushing stream to the CDN.
  StreamMixingType mixType;

  /// The CDN url where the mixed stream is supposed to be pushed. This parameter cannot be updated while pushing stream to the CDN.
  String url;

  /// Room ID. This parameter cannot be updated while pushing stream to the CDN.
  String roomId;

  /// User ID. This parameter cannot be updated while pushing stream to the CDN.
  String uid;

  /// Video transcoding related configurations.
  LiveTranscodingVideoConfig video;

  /// Audio transcoding related configurations. This parameter cannot be updated while pushing stream to the CDN.
  LiveTranscodingAudioConfig audio;

  /// Layout information.
  LiveTranscodingLayout layout;

  /// Spatial audio configuration.
  LiveTranscodingSpatialConfig spatialConfig;

  /// @nodoc
  LiveTranscoding({
    this.mixType = StreamMixingType.byServer,
    required this.url,
    required this.roomId,
    required this.uid,
    required this.video,
    required this.audio,
    required this.layout,
    this.spatialConfig = const LiveTranscodingSpatialConfig.disabled(),
  });

  /// @nodoc
  Map<String, dynamic> toMap() => {
        'url': url,
        'roomId': roomId,
        'uid': uid,
        'mixType': mixType.index,
        'video': video.toMap(),
        'audio': audio.toMap(),
        'layout': layout.toMap(),
        'spatialConfig': spatialConfig.toMap(),
      };
}

/// [eventType]: Task status.
///
/// [taskId]: Task ID.
///
/// [error]: Errors occurring when pushing stream to CDN.
///
/// [mixType]: Task type.
typedef OnStreamMixingEventType = void Function(StreamMixingEvent eventType,
    String taskId, TranscoderErrorCode error, StreamMixingType mixType);

/// Observer in pushing streams to CDN
class RTCLiveTranscodingObserver {
  /// Status callback
  OnStreamMixingEventType? onStreamMixingEvent;

  /// @nodoc
  RTCLiveTranscodingObserver({
    this.onStreamMixingEvent,
  });
}
