// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import '../src/base/bytertc_enum_convert.dart';
import 'bytertc_room_api.dart';
import 'bytertc_room_event_handler.dart';
import 'bytertc_rts_defines.dart';
import 'bytertc_video_api.dart';
import 'bytertc_video_event_handler.dart';

/// API call result.
enum ReturnStatus {
  /// Success.
  success,

  /// Failure.
  failure,

  /// Invalid parameter.
  parameterErr,

  /// API status error.
  wrongState,

  /// Failure. The user is already in the room.
  hasInRoom,

  /// Failure. The user has already logged in.
  hasInLogin,

  /// Failure. The user is already in a pre-call test.
  hasInEchoTest,

  /// Failure. Neither video nor audio is captured.
  neitherVideoNorAudio,

  /// Failure. The roomId is already in use.
  roomIdInUse,

  /// Failure. Screen stream is not supported.
  screenNotSupport,

  /// Failure. This operation is not supported.
  notSupport,

  /// Failure. Resource overflow.
  resourceOverflow,

  /// Failure. No audio frame.
  audioNoFrame,

  /// Failure. No implemented.
  audioNotImplemented,

  /// Failure. The audio capture device does not have microphone use permission, and the initialization of the device failed.
  audioNoPermission,

  /// Failure. The device does not exist. This value is returned if there is currently no device or if the device has been removed.
  audioDeviceNotExists,

  /// Failure. The device audio format is not supported.
  audioDeviceFormatNotSupport,

  /// Failure. No device in system.
  audioDeviceNoDevice,

  /// Failure. The current device is not available and need to be replaced.
  audioDeviceCannotUse,

  /// Device initialization failed due to a system error.
  audioDeviceInitFailed,

  /// Device start-up failed due to a system error.
  audioDeviceStartFailed,

  /// Failure. The native layer is not initialized and the engine is invalid.
  nativeInValid,
}

/// @nodoc
extension RTCTypeReturnStatus on int? {
  /// @nodoc
  ReturnStatus get returnStatus {
    switch (this) {
      case 0:
        return ReturnStatus.success;
      case -1:
        return ReturnStatus.failure;
      case -2:
        return ReturnStatus.parameterErr;
      case -3:
        return ReturnStatus.wrongState;
      case -4:
        return ReturnStatus.hasInRoom;
      case -5:
        return ReturnStatus.hasInLogin;
      case -6:
        return ReturnStatus.hasInEchoTest;
      case -7:
        return ReturnStatus.neitherVideoNorAudio;
      case -8:
        return ReturnStatus.roomIdInUse;
      case -9:
        return ReturnStatus.screenNotSupport;
      case -10:
        return ReturnStatus.notSupport;
      case -11:
        return ReturnStatus.resourceOverflow;
      case -101:
        return ReturnStatus.audioNoFrame;
      case -102:
        return ReturnStatus.audioNotImplemented;
      case -103:
        return ReturnStatus.audioNoPermission;
      case -104:
        return ReturnStatus.audioDeviceNotExists;
      case -105:
        return ReturnStatus.audioDeviceFormatNotSupport;
      case -106:
        return ReturnStatus.audioDeviceNoDevice;
      case -107:
        return ReturnStatus.audioDeviceCannotUse;
      case -108:
        return ReturnStatus.audioDeviceInitFailed;
      case -109:
        return ReturnStatus.audioDeviceStartFailed;
      case -201:
        return ReturnStatus.nativeInValid;
      default:
        return ReturnStatus.failure;
    }
  }
}

/// The reason for the removal of the remote stream in the room.
enum StreamRemoveReason {
  /// The remote user stops publishing streams.
  unpublish,

  /// The remote user fails to publish streams.
  publishFailed,

  /// Keep alive failed.
  keepLiveFailed,

  /// The remote user is disconnected.
  clientDisconnected,

  /// The remote user republishes streams.
  republish,

  /// Other reasons.
  other,

  /// The remote user's publishing privilege of the Token is expired.
  publishPrivilegeExpired,
}

/// Room mode
enum RoomProfile {
  /// Communication mode(default)
  communication,

  /// @nodoc
  @Deprecated(
      'Deprecated since v3.45.1 and will be deleted in v3.52.1, use interactivePodcast instead')
  liveBroadCasting,

  /// Game voice mode.
  ///
  /// Low consumption of computing resources and network data.<br>
  /// Additional performance optimizations have been made for low-end devices:
  /// + Encodes frame length 40/60 for some low-end models.
  /// + Disables software 3A audio processing for some low-end models.
  /// Enhance iOS Compatibility with other screen recordings to avoid audio recordings being interrupted by RTC.
  game,

  /// Cloud game mode.
  ///
  /// Use this mode for game App requiring low latency. <br>
  /// In this mode, the call latency will be significantly reduced, but at the same time, weak network resistance and call sound quality will be affected to a certain extent, so before using this mode, it is highly recommended to consult technical support.
  cloudGame,

  /// Cloud render mode featuring extra-low latency.
  ///
  /// Choose this mode if it is not a game App yet requires low latency. <br>
  /// In this mode, latency will significantly decrease at the cost of lower audio quality as well as the bad performance with weak network signal.
  lowLatency,

  /// For 1 vs 1 video and audio calls
  chat,

  /// For voice calls only with 3 or more people
  chatRoom,

  /// For scenarios such as "Watch together" and "Listen together". Multiple clients are capable of playing the same videos and audios synchronically.
  ///
  /// In this mode, RTC does not involve the transportation of the sharing media but only synchronizes video/music playback across multiple clients via signaling.
  lwTogether,

  /// For game apps demanding high-resolution audio. In this mode, RTC plays audio using the media mode only.
  gameHD,

  /// For the events of co-hosting in the live-streaming.
  ///
  /// During a livestreaming using a CDN network, one host can invite another host to join the co-hosting event using RTC.
  coHost,

  /// For interactive podcasts that the host can have video and audio interactions with the audience.
  ///
  /// Using communication mode, and whether to co-host or not will not have mode switching to avoid sudden volume changes.
  interactivePodcast,

  /// For the online karaoke with high-quality audio and low latency.
  ///
  /// In these scenarios, RTC transports the accompaniment and mixed audio, such as solo and non-realtime chorus.
  ktv,

  /// For the online-chorusing scenarios requiring high-quality audio and low latency.
  ///
  /// Contact our technical specialists before you apply it to your App.
  chorus,

  /// For VR chat.
  ///
  /// This mode supports audio sampling rates up to 192 KHz, with the ability to turn on spherical stereo.<br>
  /// Available in 3.46 and later versions.
  vrChat,

  /// For scenarios of streaming live videos to only one client on the LAN. It can be applied to devices on the Internet or LAN.
  gameStreaming,

  /// For 1-to-many live video streaming for LANs.
  ///
  /// Maximum supported resolution is 8K, frame rate is 60fps, and bit rate is 100 Mbps.<br>
  /// A private media server is needed on the LAN.
  lanLiveStreaming,

  /// For meeting Apps installed on personal devices.
  meeting,

  /// For meeting Apps installed on terminals of meeting rooms.
  meetingRoom,

  /// For the online classrooms and lectures that over 10 participants in the room are allowed to join the video chat.
  classroom,
}

/// SEI data source type.
enum DataMessageSourceType {
  /// (Default) Customized information.
  def,

  /// System notification including volume indication.
  system,
}

/// SEI sending mode.
enum SEICountPerFrame {
  /// Single-SEI mode. When you send multiple SEI messages in 1 frame, they will be sent frame by frame in a queue.
  single,

  /// Multi-SEI mode. When you send multiple SEI messages in 1 frame, they will be sent together in the next frame.
  multi,
}

/// The error code indicates that the SDK has encountered a non-recoverable error. It will be send to users through [RTCVideoEventHandler.onError].
enum ErrorCode {
  /// Undefined error code
  unknown,

  /// The token used when joining the room is invalid or expired. The user is required to retrieve the token and update it by calling [RTCRoom.updateToken].
  invalidToken,

  /// An unknown error occurs when entering a room causing a failure to join the room, requiring the user to rejoin the room.
  joinRoom,

  /// The user failed to publish streams in the room because they do not have permission.
  noPublishPermission,

  /// The user failed to subscribe to streams in the room because they do not have permission.
  noSubscribePermission,

  /// The current user is removed from the room because another user with the same user ID joins the room.
  duplicateLogin,

  /// Applies to Android.
  ///
  /// The App ID is not passed in when you create the engine.
  appIdNull,

  /// The user has been removed from the room by the administrator via a OpenAPI call.
  kickedOut,

  /// The roomId passed in when you call [RTCVideo.createRTCRoom] is illegal.
  roomIdIllegal,

  /// The Token has expired.
  ///
  /// The user should rejoin the room with a new Token.
  tokenExpired,

  /// The Token provided in [RTCRoom.updateToken] is invalid.
  updateTokenWithInvalidToken,

  /// All the users have been removed from the room because the administrator dismissed the room by calling OpenAPI.
  roomDismiss,

  /// Failed to join the room. The LICENSE billing account does not use the LICENSE_AUTHENTICATE SDK.
  joinRoomWithoutLicenseAuthenticateSDK,

  /// A room with the same roomId already exists during the call echo test.
  roomAlreadyExist,

  /// The local user joins multiple rooms with different uid.
  ///
  /// In the same engine instance, users need to use the same uid to join different rooms.
  userIdDifferent,

  /// Server license expired, refused to enter the room.
  joinRoomServerLicenseExpired,

  /// Exceeds the upper limit of the concurrency allowed by the server license, and refuses to enter the room.
  joinRoomExceedsTheUpperLimit,

  /// The license parameter is wrong and refuses to enter the room.
  joinRoomLicenseParameterError,

  /// Wrong license certificate path.
  joinRoomLicenseFilePathError,

  /// The license certificate is illegal, refuse to enter the room.
  joinRoomLicenseIllegal,

  /// License certificate has expired, refused to enter the room.
  joinRoomLicenseExpired,

  /// The content of the license certificate does not match.
  joinRoomLicenseInformationNotMatch,

  /// The current certificate does not match the cached certificate.
  joinRoomLicenseNotMatchWithCache,

  /// The room has been banned before [RTCRoom.joinRoom] is called.
  joinRoomRoomForbidden,

  /// The user has been banned before joining the room.
  joinRoomUserForbidden,

  /// The license method did not load successfully. Check the corresponding extension.
  joinRoomLicenseFunctionNotFound,

  /// Failed to subscribe to streams because the total number of subscribed streams has exceeded the upper limit.
  ///
  /// In order to ensure the performance and quality in game scenarios, the server will limit the total number of audio & video streams subscribed by the user. <br>
  /// When the total number of subscribed stream has reached the maximum, continuing to subscribe to more streams will fail, and the user will receive this error notification.
  overStreamSubscribeLimit,

  /// @nodoc('For internal use')
  loadSOLib,

  /// Failed to publish camera-captured video stream because the total number of published streams has exceeded the upper limit.
  ///
  /// RTC will limit the total number of streams published in a single room, including video, audio, and screen streams. <br>
  /// Local users will fail to publish streams to the room when the maximum number of published streams in the room has been reached, and will receive this error notification.
  overStreamPublishLimit,

  /// Failed to publish screen video stream because the total number of published streams has exceeded the upper limit.
  ///
  /// RTC will limit the total number of streams published in a single room, including video, audio, and screen streams. <br>
  /// Local users will fail to publish streams to the room when the maximum number of published streams in the room has been reached, and will receive this error notification.
  @Deprecated('Deprecated since v3.54.1, use overStreamPublishLimit instead')
  overScreenPublishLimit,

  /// The total number of published video streams has exceeded the upper limit.
  ///
  /// RTC will limit the total number of streams published in a single room, including video, audio, and screen streams. <br>
  /// Local users will fail to publish streams to the room when the maximum number of published streams in the room has been reached, and will receive this error notification.
  @Deprecated('Deprecated since v3.54.1, use overStreamPublishLimit instead')
  overVideoPublishLimit,

  /// Audio/Video synchronization failed.
  ///
  /// The current audio source has been synchronized with another video source.<br>
  /// One single audio source cannot be synchronized with multiple video sources at the same time.
  invalidAudioSyncUidRepeated,

  /// The user has been removed from the room due to the abnormal status of server.
  ///
  /// SDK is disconnected with the signaling server and will not reconnect automatically. Please contact technical support.
  abnormalServerStatus,
}

/// Subscription status
enum SubscribeState {
  /// Successfully changed the subscription status
  success,

  /// Failed to change the subscription status, because the local user was not in the room.
  failedNotInRoom,

  /// Failed to change the subscription status, because the target audio/video stream was not found.
  failedStreamNotFound,

  /// Failed to change the subscription status, because the number of streams you have subscribed to has exceeded the limit.
  failedOverLimit,
}

/// The warning code indicates that the SDK has encountered a problem and is trying to recover. It only serves as a notification.
enum WarningCode {
  /// Undefined warning code
  unknown,

  /// Failed to enter the room due to a server error. The SDK will automatically retry.
  joinRoomFailed,

  /// Failed to publish streams due to a server error. The SDK will automatically retry.
  publishStreamFailed,

  /// Failed to subscribe to streams. The target stream cannot be found.
  ///
  /// The SDK will automatically retry the subscription and if it still fails you can quit and retry.
  subscribeStreamFailed404,

  /// Failed to subscribe to streams due to a server error. The SDK will automatically retry.
  subscribeStreamFailed5xx,

  /// Publishing stream after calling [RTCRoom.setUserVisibility] to set user visibility to false will trigger this warning.
  publishStreamForbidden,

  /// Failed to send custom broadcast message, because the local user is not in the room.
  sendCustomMessage,

  /// When the number of people in the room exceeds 500, the SDK will stop sending [RTCRoomEventHandler.onUserJoined] and [RTCRoomEventHandler.onUserLeave] callbacks to existing users in the room, and notify all users in the room via broadcast.
  receiveUserNotifyStop,

  /// The user has already published a stream in another room, or the user is publishing a public stream now.
  userInPublish,

  /// The room with the same roomId already exists.
  roomIdAlreadyExist,

  /// A new room with the same roomId has replaced the old one.
  oldRoomBeenReplaced,

  /// Invalid call as an echo test is currently in progress.
  inEchoTestMode,

  /// The current app does not have access to the camera
  noCameraPermission,

  /// The microphone permission is abnormal, and the current application does not obtain microphone permission.
  @Deprecated(
      'Deprecated since v3.45.1 and will be deleted in v3.52.1, use MediaDeviceWarning instead')
  noMicrophonePermission,

  /// The audio capture device failed to start.
  ///
  /// Failed to start the audio capture device, the current device may be occupied by other applications.
  @Deprecated(
      'Deprecated since v3.45.1 and will be deleted in v3.52.1, use MediaDeviceWarning instead')
  audioDeviceManagerRecordingStartFail,

  /// Audio playback device startup failure warning.
  ///
  /// It may be due to insufficient system resources or wrong parameters.
  @Deprecated(
      'Deprecated since v3.45.1 and will be deleted in v3.52.1, use MediaDeviceWarning instead')
  audioDeviceManagerPlayoutStartFail,

  /// No audio acquisition equipment available.
  ///
  /// Failed to start the audio capture device, please insert the available audio capture device.
  @Deprecated(
      'Deprecated since v3.45.1 and will be deleted in v3.52.1, use MediaDeviceWarning instead')
  noRecordingDevice,

  /// No audio playback device available.
  ///
  /// Failed to start the audio playback device, please insert the available audio playback device.
  @Deprecated(
      'Deprecated since v3.45.1 and will be deleted in v3.52.1, use MediaDeviceWarning instead')
  noPlayoutDevice,

  /// The current audio equipment has not collected valid sound data, please check and replace the audio acquisition equipment.
  @Deprecated(
      'Deprecated since v3.45.1 and will be deleted in v3.52.1, use MediaDeviceWarning instead')
  recordingSilence,

  /// Media device misoperation warning.
  @Deprecated(
      'Deprecated since v3.45.1 and will be deleted in v3.52.1, use MediaDeviceWarning instead')
  mediaDeviceOperationDenied,

  /// Switching screen audio capture source after calling [RTCRoom.publishScreen] is not allowed.
  setScreenAudioSourceTypeFailed,

  /// Setting up the screen audio sharing type with [RTCVideo.setScreenAudioStreamIndex] after calling [RTCRoom.publishScreen] is not allowed.
  setScreenAudioStreamIndexFailed,

  /// Invalid pitch value.
  invalidVoicePitch,

  /// Mixed use of old and new APIs for external audio sources.
  invalidCallForExtAudio,

  /// Applies to iOS. The specified internal rendering canvas handle is invalid.
  invalidCanvasHandle,

  /// Authorization failure for [BytePlus Audio SDK](https://go.byteplus.com/audio-sdk). Contact the technical supporters.
  invalidSamiAppKeyOrToken,

  /// Failure to get the resource for [BytePlus Audio SDK](https://go.byteplus.com/audio-sdk). Input the right DAT path or contact the technical supporters.
  invalidSamiResourcePath,

  /// Failure to load the library for [BytePlus Audio SDK](https://go.byteplus.com/audio-sdk). Use right library or contact the technical supporters.
  loadSamiLibraryFailed,

  /// The audio effect is not supported by [BytePlus Audio SDK](https://go.byteplus.com/audio-sdk). Contact the technical supporters.
  invalidSamiEffectType,
}

/// Reasons for performance-related alarms
enum PerformanceAlarmReason {
  /// The poor network causes the transmission performance to fall back.
  ///
  /// This reason is only received when sending performance fallback is turned on.
  bandwidthFallbacked,

  /// Network performance recovery, transmission performance rollback recovery.
  ///
  /// This reason is only received when sending performance fallback is turned on.
  bandwidthResumed,

  /// If the send performance fallback is not turned on, when receiving this alarm, it means that the performance is insufficient. <br>
  /// If the send performance fallback is turned on, when receiving this alarm, it means that the performance is insufficient and the send performance fallback has occurred.
  performanceFallbacked,

  /// If the send performance fallback is not turned on, when receiving this alarm, it means that the performance shortage has been restored. <br>
  /// If the send performance fallback is turned on, when receiving this alarm, it means that the performance shortage has been restored and the send performance fallback has occurred. Recovery.
  performanceResumed
}

/// Whether to enable publisher performance fallback
enum PerformanceAlarmMode {
  /// No
  normal,

  /// Yes
  simulcast,
}

/// Stream fallback options of publisher
enum PublishFallbackOption {
  /// Default setting. No fallback is allowed under limited network conditions.
  disable,

  /// Under limited network conditions, the video streams that you published will degrade sequentially from the highest-quality stream to the lowest-quality stream until it can match current network conditions.
  simulcastSmallVideoOnly
}

/// Stream fallback options of subscriber
enum SubscribeFallbackOption {
  /// Default setting. No fallback is allowed under limited network conditions.
  disable,

  /// The resolution of currently subscribed video stream will be reduced under limited network conditions.
  ///
  /// You can only select this setting after the stream publisher calls [RTCVideo.enableSimulcastMode] to enable the mode of publishing multiple streams, otherwise no fallback will occur.
  streamLow,

  /// Receive audio only under poor network conditions.
  audioOnly
}

/// Priority of the publisher. When a user encounters performance insufficiency of either the network or the device, the media stream  will fall back in the ascending order of `RemoteUserPriority`.
enum RemoteUserPriority {
  /// Low(default)
  low,

  /// No priority
  medium,

  /// High
  high,
}

/// Causes of fallback or recovery from fallback in remote subscription streams.
enum FallbackOrRecoverReason {
  /// Undefined reason. Default value.
  ///
  /// Fallback due to an unknown reason that is neither insufficient bandwidth of the network nor poor-performance of the device.
  unknown,

  /// Fallback of the subscribed stream due to insufficient bandwidth of the network.
  subscribeFallbackByBandwidth,

  /// Fallback of the subscribed stream for poor-performance of the device.
  subscribeFallbackByPerformance,

  /// Reverting from a Fallback of the subscribed stream due to the recovery of the network bandwidth.
  subscribeRecoverByBandwidth,

  /// Reverting from a Fallback of the subscribed stream due to the amelioration of the device performance.
  subscribeRecoverByPerformance,

  /// Fallback of the publishing stream due to Insufficient bandwidth of the network.
  publishFallbackByBandwidth,

  /// Fallback of the publishing stream due to poor-performance of the device.
  publishFallbackByPerformance,

  /// Reverting from a Fallback of the publishing stream due to the recovery of the network bandwidth.
  publishRecoverByBandwidth,

  /// Reverting from a Fallback of the publishing stream due to the amelioration of the device performance.
  publishRecoverByPerformance,
}

/// Local audio stream status.
enum LocalAudioStreamState {
  /// The default initial state of the local audio.
  ///
  /// SDK will return this state when the microphone stops working. The corresponding error code is `ok`.
  stopped,

  /// The local audio recording device starts successfully.
  ///
  /// SDK will return this state after successfully capturing the first audio frame. The corresponding error code is `ok`.
  recording,

  /// The first frame of the local audio was successfully encoded. The corresponding error code is `ok`.
  encoding,

  /// The local audio startup failed, and the status is called back at the following times:
  /// + The local recording device failed to start, the corresponding error code is `recordFailure`.
  /// + No recording device permission was detected, the corresponding error code is `deviceNoPermission`.
  /// + Audio encoding failed, the corresponding error code is `encodeFailure`.
  failed
}

/// Error code returned when local audio stream state changes.
enum LocalAudioStreamError {
  /// Local audio status is normal.
  ok,

  /// Unknown error cause.
  failure,

  /// No permission to start local audio recording device.
  deviceNoPermission,

  /// @nodoc('Not available')
  /// The local audio recording device is already in use.
  deviceBusy,

  /// Local audio recording failed, it is recommended that you check whether the recording device is working properly.
  recordFailure,

  /// Local audio encoding failed.
  encodeFailure,

  /// No audio recording equipment available.
  noRecordingDevice
}

/// Local video stream status and corresponding error code in [LocalVideoStreamError].
enum LocalVideoStreamState {
  /// Local video default initial state.
  ///
  /// The corresponding error code is `ok`.
  stopped,

  /// The local video recording device starts successfully.
  ///
  /// The corresponding error code is `ok`.
  recording,

  /// The first frame is encoded successfully.
  ///
  /// The corresponding error code is `ok`.
  encoding,

  /// Local video startup failed,  the status is called back at the following times:
  /// + The local acquisition device failed to start, the corresponding error code is `failure`.
  /// + No camera permission was detected, the corresponding error code is `deviceNoPermission`.
  /// + Video encoding failed, the corresponding error code is `encodeFailure`.
  failed
}

/// Error code returned when the local video state changes.
enum LocalVideoStreamError {
  /// Local video status is normal.
  ok,

  /// Local video stream publishing failed.
  failure,

  /// No permission to start local video capture device.
  deviceNoPermission,

  /// Local video capture device is occupied.
  deviceBusy,

  /// Local video capture device does not exist or has been removed.
  deviceNotFound,

  /// Local video capture failed, it is recommended to check whether the capture device is working properly.
  captureFailure,

  /// Local video encoding failure.
  encodeFailure,

  /// The local video capture device is seized by other programs during the call, causing the device connection to be interrupted.
  deviceDisconnected,
}

/// Status of the subscribed remote audio stream and corresponding reason code in [RemoteAudioStateChangeReason].
enum RemoteAudioState {
  /// Initial status, and will be returned at the following times:
  /// + Local user stops receiving the remote audio stream, the reason code is `localMuted`.
  /// + The remote user stops sending the audio stream, the reason code is `remoteMuted`.
  /// + The remote user has left the room, the reason code is `remoteOffline`.
  stopped,

  /// Start receiving the remote audio stream header package.
  starting,

  /// The remote audio stream is decoding and playing normally. The state will be returned at the following times:
  /// + The first frame of the remote audio is successfully decoded, the reason code is `localUnmuted`.
  /// + The network is back to normal from being blocked, the reason code is `networkRecovery`.
  /// + The local user resumes receiving remote audio streams, the reason code is `localUnmuted`.
  /// + The remote user resumes sending audio streams, the reason code is `remoteUnmuted`.
  decoding,

  /// Remote audio stream stalls.
  ///
  /// This state will be called back when network blocking causes packet loss greater than 40%, the reason code is `networkCongestion`.
  frozen,

  /// @nodoc('Not available')
  /// Remote audio stream playback failed.
  failed
}

/// Reasons for change in remote audio stream status
enum RemoteAudioStateChangeReason {
  /// Internal reasons.
  internal,

  /// Network blocking.
  networkCongestion,

  /// Network back to normal.
  networkRecovery,

  /// Local user stops receiving remote audio streams.
  localMuted,

  /// Local user resumes receiving remote audio streams.
  localUnmuted,

  /// Remote user stops sending audio streams.
  remoteMuted,

  /// Remote user resumes sending audio streams.
  remoteUnmuted,

  /// Remote user leaves the room.
  remoteOffline,
}

/// Remote video stream status and corresponding reason code in [RemoteVideoStateChangeReason].
enum RemoteVideoState {
  /// Initial status. This state will be called back at the following times:
  /// + Local user stops receiving remote video streams, the reason code is `localMuted`.
  /// + Remote user stops sending video streams, the reason code is `remoteMuted`.
  /// + Remote user leaves the room, the reason code is `remoteOffline`.
  stopped,

  /// Local user has received the remote video header package, the reason code is `localUnmuted`.
  starting,

  /// The remote video stream is decoding and playing normally, callback the status at the following times:
  /// + The first frame of remote video is successfully decoded, the reason code is `localUnmuted`.
  /// + The network is back to normal from being blocked, the reason code is `networkRecovery`.
  /// + The local user resumes receiving remote video streams, the reason code is `localUnmuted`.
  /// + The remote user resumes sending video stream, the reason code is `remoteUnmuted`.
  decoding,

  /// Remote video stream stalls.
  ///
  /// This state will be called back when network blocking causes packet loss greater than 40%, the reason code is `networkCongestion`.
  frozen,

  /// Remote video stream playback failed.
  ///
  /// This state will be called back when internal processing of remote video streams fails, the reason code is `internal`.
  failed,
}

/// Cause of remote video stream state change.
enum RemoteVideoStateChangeReason {
  /// Internal reasons
  internal,

  /// Network blocking.
  networkCongestion,

  /// Network back to normal.
  networkRecovery,

  /// Local user stops receiving remote video stream or local user disables video module.
  localMuted,

  /// Local user resumes receiving remote video streams or local user enables video modules.
  localUnmuted,

  /// The remote user stops sending the video stream or the remote user disables the video module.
  remoteMuted,

  /// Remote user resumes sending video stream or remote user enables video module.
  remoteUnmuted,

  /// Remote user leaves room.
  ///
  /// See [RTCRoomEventHandler.onUserUnpublishStream]/[RTCRoomEventHandler.onUserUnpublishScreen] for state change.
  remoteOffline
}

/// Media device state
enum MediaDeviceState {
  /// Start capturing.
  started,

  /// Stop capturing.
  stopped,

  /// The media device is expected to be working but no data is received.
  runtimeError,

  /// Device is plugged in.
  added,

  /// Device is removed.
  removed,

  /// Phone calls, locking screen or the other Apps interrupted the RTC call. RTC call will resume once the phone call ends or the other Apps release the media devices.
  interruptionBegan,

  /// RTC call resumed from the interruption caused by the phone calls or the other Apps.
  interruptionEnded,
}

/// Media device error
enum MediaDeviceError {
  /// Normal.
  ok,

  /// No permission to start the media device.
  deviceNoPermission,

  /// The media device is already in use.
  deviceBusy,

  /// Media device error.
  deviceFailure,

  /// The specified media device was not found.
  deviceNotFound,

  /// The media device is removed.
  deviceDisconnected,

  /// he media device is expected to be working but no data is received.
  deviceNoCallback,

  /// Device sample rate is not supported.
  unsupportedFormat,

  /// No group id parameter for iOS screen capture.
  notFindGroupId,

  /// For iOS only. An interruption caused by the app being sent to the background as the user starts the system camera app.
  notAvailableInBackground,

  /// For iOS only. An interruption caused by the video device temporarily being unavailable (for example, when used by another app).
  videoInUseByAnotherClient,

  /// For iOS only. An interruption caused when your app is running in Slide Over, Split View, or Picture in Picture mode on iPad.
  notAvailableWithMultipleForegroundApps,

  /// For iOS only. An interruption due to system pressure, such as overheating.
  notAvailableDueToSystemPressure,
}

/// Media device warning
enum MediaDeviceWarning {
  /// No warning.
  ok,

  /// Illegal operation.
  ///
  /// Calls the API for internal device when using the external device.
  operationDenied,

  /// The media device is capturing silent frames.
  captureSilence,

  /// Android-specific silence, system-level silence reporting.
  androidSysSilence,

  /// Android-specific silence disappears.
  androidSysSilenceDisappear,

  /// @nodoc('For internal use')
  /// The volume is too high and exceeds the capture range of the device, it is recommended to reduce the microphone volume or reduce the volume of the sound source.
  ///
  /// Applies to conference scenario.
  detectClipping,

  /// Echo leakage.
  ///
  /// Applies to conference scenario.
  detectLeakEcho,

  /// @nodoc('For internal use')
  /// Low signal-to-noise ratio.
  ///
  /// Applies to conference scenario.
  detectLowSNR,

  /// @nodoc('For internal use')
  /// Silence inserted during capture.
  ///
  /// Applies to conference scenario.
  detectInsertSilence,

  /// @nodoc('For internal use')
  /// Silence during capture.
  ///
  /// Applies to conference scenario.
  captureDetectSilence,

  /// @nodoc('For internal use')
  /// Silence during capture disappears.
  ///
  /// Applies to conference scenario.
  captureDetectSilenceDisappear,

  /// Howling detected.
  ///
  /// You will receive this callback in the following scenarios:
  /// + Howling is detected under the room profiles that do not support howling suppression.
  /// + Detect howling that is not suppressed under the room profiles that support howling suppression.
  ///
  /// You can only enable howling suppression when the room profile is Communication, Meeting, or MeetingRoom. <br>
  /// We recommend that you remind your users to adjust the physical distance between two devices or disable all unused devices except the connecting one.
  captureDetectHowling,

  /// Setting audio route failed because the audio route can not be changed in the current Audio Scenario.
  setAudioRouteInvalidScenario,

  /// Setting audio route failed because the routing device did not exist.
  ///
  /// Applies to Android.
  setAudioRouteNotExists,

  /// Setting audio route failed because the audio route was occupied by the system process or the other Apps.
  setAudioRouteFailedByPriority,

  /// Setting audio route failed because the audio route can only be changed in the communication audio mode.
  ///
  /// Applies to Android.
  setAudioRouteNotVoipMode,

  ///  Setting audio route failed because the audio device did not initiate.
  setAudioRouteDeviceNotStart,

  /// For iOS only. The media volume is not set in all scenarios. You cannot change the Bluetooth profile until you set the media volume in all scenarios.
  setBluetoothModeScenarioUnsupport,

  /// For iOS only. You cannot change the Bluetooth profile to HFP.
  setBluetoothModeUnsupport,
}

/// Audio/video quality feedback
enum ProblemFeedbackOption {
  /// Good
  none,

  /// Undefined problems
  otherMessage,

  /// Connection failed
  disconnected,

  /// High latency for the ear monitor
  earBackDelay,

  /// Noise on the local end
  localNoise,

  /// Audio stutter on the local end
  localAudioLagging,

  /// No sound on the local end
  localNoAudio,

  /// Too little/loud sound on the local end
  localAudioStrength,

  /// Echo noise on the local end
  localEcho,

  /// Unclear video on the local end
  localVideoFuzzy,

  /// Audio and video out of sync on the local end
  localNotSync,

  /// Video stutter on the local end
  localVideoLagging,

  /// No picture on the local end
  localNoVideo,

  /// Noise on the remote end
  remoteNoise,

  /// Audio stutter on the remote end
  remoteAudioLagging,

  /// No sound on the remote end
  remoteNoAudio,

  /// Too little/loud sound on the remote end
  remoteAudioStrength,

  /// Echo noise on the remote end
  remoteEcho,

  /// Unclear video on the remote end
  remoteVideoFuzzy,

  /// Audio and video out of sync on the remote end
  remoteNotSync,

  /// Video stutter on the remote end
  remoteVideoLagging,

  /// No picture on the remote end
  remoteNoVideo,
}

/// Room info of the video call quality feedback.
class ProblemFeedbackRoomInfo {
  /// Room ID
  final String roomId;

  /// User ID
  final String uid;

  /// @nodoc
  ProblemFeedbackRoomInfo({
    required this.roomId,
    required this.uid,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'roomId': roomId,
      'uid': uid,
    };
  }
}

/// Feedback information on video call quality
class ProblemFeedbackInfo {
  /// Text description
  String problemDesc;

  /// Room information
  List<ProblemFeedbackRoomInfo>? roomInfo;

  /// @nodoc
  ProblemFeedbackInfo({
    required this.problemDesc,
    this.roomInfo,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'problemDesc': problemDesc,
      if (roomInfo != null)
        'roomInfo': roomInfo!.map((e) => e.toMap()).toList(growable: false),
    };
  }
}

/// First frame sending status
enum FirstFrameSendState {
  /// Sending
  sending,

  /// Sent successfully
  sent,

  /// Send failed
  end,
}

/// First frame playback status
enum FirstFramePlayState {
  /// Playing
  playing,

  /// Played successfully
  played,

  /// Play failed
  end,
}

/// Custom encryption type
enum EncryptType {
  /// No built-in encryption used (default)
  customize,

  /// AES-128-CBC encryption algorithm
  aes128CBC,

  /// AES-256-CBC encryption algorithm
  aes256CBC,

  /// AES-128-ECB encryption algorithm
  aes128ECB,

  /// AES-256-ECB encryption algorithm
  aes256ECB,
}

/// Video encoding type
enum VideoCodecType {
  /// Unknown type
  auto,

  /// H.264 format
  h264,

  /// ByteVC1 format
  bytevc1
}

/// State of the black frame video stream
enum SEIStreamUpdateEvent {
  /// A black frame video stream is published from the remote user.
  ///
  /// In a voice call, when the remote user calls [RTCVideo.sendSEIMessage] to send SEI data, SDK will automatically publish a black frame video stream, and trigger this callback.
  streamAdd,

  /// The black frame video stream is removed. The timing this callback will be triggered is as following:
  /// + The remote user turns on their camera, switching the voice call to a video call.
  /// + No SEI data is sent within 1min after the remote user calls [RTCVideo.sendSEIMessage].
  streamRemove,
}

/// Stream type
enum StreamIndex {
  /// Mainstream.
  ///
  /// Video/audio captured by the the camera/microphone using internal capturing.
  main,

  /// Screen-sharing stream.
  ///
  /// Video/Audio streams from screen sharing.
  screen
}

/// Stream type for synchronization
enum SyncInfoStreamType {
  /// Audio stream
  audio,
}

/// Media stream relaying status
enum ForwardStreamState {
  /// No stream relaying.
  ///
  /// + After [RTCRoom.stopForwardStreamToRooms] is called, all target rooms are in this status.
  /// + When number of target rooms is reduced by calling [RTCRoom.updateForwardStreamToRooms], the target room for this reduction is in this status.
  idle,

  /// Relaying started.
  ///
  /// + Calling [RTCRoom.startForwardStreamToRooms] to successfully start relaying media stream to all rooms will have this status returned.
  /// + Calling [RTCRoom.updateForwardStreamToRooms] to successfully started relaying media stream to the new target rooms will have this status returned.
  success,

  /// relaying failed, see [ForwardStreamError] for details.
  ///
  /// This status will be called back in case of relaying failure after calling [RTCRoom.startForwardStreamToRooms] or [RTCRoom.updateForwardStreamToRooms].
  failure,
}

/// Error code during cross-room relaying of media streams
enum ForwardStreamError {
  /// Normal.
  ok,

  /// Parameter exception.
  invalidArgument,

  /// Invalid token.
  invalidToken,

  /// Error returned from server.
  response,

  /// relaying is interrupted because a user with the same userID joins the target room.
  remoteKicked,

  /// The server does not support cross-room relaying function.
  notSupport,
}

/// Events occurring in this target room during media stream relaying.
enum ForwardStreamEvent {
  /// The network connection between the local end and the server is disconnected and relaying is suspended.
  disconnected,

  /// Network connection between local and server is restored, and relaying service is successfully connected.
  connected,

  /// relaying is interrupted because a user with the same userID joins the target room.
  interrupt,

  /// The target room is updated by calling [RTCRoom.updateForwardStreamToRooms].
  dstRoomUpdated,

  /// API call timing error.
  ///
  /// For example, call [RTCRoom.updateForwardStreamToRooms] before [RTCRoom.startForwardStreamToRooms].
  unexpectedAPICall,
}

/// Status of local recording
enum RecordingState {
  /// Recording exception.
  error,

  /// Recording in progress.
  processing,

  /// Recorded file saved successfully.
  ///
  /// Received after calling [RTCVideo.stopFileRecording] to end the recording.
  success
}

/// Local recording error code
enum RecordingErrorCode {
  /// Recording normally.
  ok,

  /// No file write permission.
  noPermission,

  /// The current version of the SDK does not provide local recording function, please contact technical support.
  notSupport,

  /// Other exceptions.
  noOther,
}

/// Storage format of locally recorded files
enum RecordingFileType {
  /// aac format
  aac,

  /// mp4 format
  mp4,
}

/// A/V synchronization states
enum AVSyncState {
  /// A/V synchronization begins.
  streamSyncBegin,

  /// The audio stream is removed during the synchronization which, however, does not affect the sync relationship.
  audioStreamRemove,

  /// The video stream is removed during the synchronization which, however, does not affect the sync relationship.
  videoStreamRemove,

  /// @nodoc('Not available')
  /// Subscriber settings synchronization.
  ///
  /// Currently not available.
  setAVSyncStreamId,
}

/// Video/audio call test result
enum EchoTestResult {
  /// The playback of captured audio/video is received, test succeeded.
  success,

  /// Test is not completed after 60 seconds and has been stopped automatically.
  timeout,

  /// Less than 5s between the end of the last test and the start of the next test.
  intervalShort,

  /// Audio capture error.
  audioDeviceError,

  /// Video capture error.
  videoDeviceError,

  /// Audio reception error.
  audioReceiveError,

  /// Video reception error.
  videoReceiveError,

  /// Unrecoverable internal error.
  internalError,
}

/// Error code for public stream
enum PublicStreamErrorCode {
  /// Success
  success,

  /// Parameter error. Retry with the right parameters.
  pushParamError,

  /// Server in abnormal status. Retry automatically.
  pushStateError,

  /// Internal error. Please retry.
  pushInternalError,

  /// Failed to push stream. Retry automatically.
  pushError,

  /// Failed to push stream. Retry automatically in every 10s, up to 3 times.
  pushTimeOut,

  /// Failed to play a public stream because the publisher has not started publishing.
  pullNoPushStream,
}

/// The Bluetooth profiles.
///
/// For iOS only.
enum BluetoothMode {
  /// The default mode：
  /// <table border>
  /// <tr>
  ///   <th> Scenario </th>
  ///   <th> HFP </th>
  ///   <th> A2DP </th>
  /// </tr>
  /// <tr>
  ///   <th> The in-call volume is set in all scenarios. </th>
  ///   <th> HFP is supported by your Bluetooth device. </th>
  ///   <th> HFP is not supported by your Bluetooth device. </th>
  /// </tr>
  /// <tr>
  ///   <th> The media volume is set in all scenarios.</th>
  ///   <th> Audio is captured and played by your Bluetooth device. </th>
  ///   <th> Audio is captured by your iOS device and played by your Bluetooth device. </th>
  /// </tr>
  /// </table>
  auto,

  /// Advanced Audio Distribution Profile(A2DP). This profile defines how high-quality audio (stereo or mono) can be streamed from one device to another over a Bluetooth connection. Audio is captured by your iOS device and played by your Bluetooth device.
  a2dp,

  /// Hands-Free Profile(HFP). This profile defines how voice-quality, mono audio can be streamed from one device to another. Audio is captured and played by your Bluetooth device.
  hfp,
}

/// Video/audio call test configurations
class EchoTestConfig {
  /// User ID for testing.
  String uid;

  /// ID of the room that the tested user will join.
  String roomId;

  /// Token used for authenticating users when they enter the room.
  String token;

  /// Whether to test audio. The device to be tested is the default audio device.
  ///
  /// + true: Yes, and the device microphone will automatically turn on and [RTCVideoEventHandler.onLocalAudioPropertiesReport] will be triggered when the value of audioReportInterval is set to greater than 0, from which you can judge the working status of the microphone.
  /// + false: No.
  bool enableAudio;

  /// Whether to test video.
  ///
  /// + true: Yes, and the device camera will automatically turn on.
  /// + false: No.
  ///
  /// The video is published with fixed parameters: resolution 640px × 360px, frame rate 15fps.
  bool enableVideo;

  /// Volume prompt interval in ms, the default value is 100.
  ///
  /// + `<= 0`: Turn off prompt.
  /// + `(0,100]`: Invalid interval value, and will be automatically reset to 100ms.
  /// + `> 100`: The actual value of interval.
  int audioReportInterval;

  /// @nodoc
  EchoTestConfig({
    required this.uid,
    required this.roomId,
    required this.token,
    required this.enableAudio,
    required this.enableVideo,
    required this.audioReportInterval,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'uid': uid,
      'roomId': roomId,
      'token': token,
      'enableAudio': enableAudio,
      'enableVideo': enableVideo,
      'audioReportInterval': audioReportInterval,
    };
  }
}

/// User information
class UserInfo {
  /// User ID
  ///
  /// The string matches the regular expression: `[a-zA-Z0-9_@\-]{1,128}`.
  ///
  /// You must set or manage the uid yourself and ensure that each uid within the same room is unique.
  final String uid;

  /// Additional information the user passed in with a maximum length of 200 bytes, which will be called back to the remote user in [RTCRoomEventHandler.onUserJoined].
  final String extraInfo;

  /// @nodoc
  const UserInfo({
    required this.uid,
    this.extraInfo = '',
  });

  /// @nodoc
  factory UserInfo.fromMap(Map<dynamic, dynamic> map) {
    return UserInfo(
      uid: map['uid'],
      extraInfo: map['metaData'],
    );
  }

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'uid': uid,
      'metaData': extraInfo,
    };
  }
}

/// Call-related statistics
class RTCRoomStats {
  /// Cumulative call duration in seconds after the user enters the room.
  final int? duration;

  /// Cumulative number of bytes sent by local user.
  final int? txBytes;

  /// Cumulative number of bytes received by local user.
  final int? rxBytes;

  /// Uplink code rate (kbps), instantaneous value when obtaining this data.
  final int? txKBitrate;

  /// Downlink code rate (kbps), instantaneous value when obtaining this data.
  final int? rxKBitrate;

  /// The code rate (kbps) at which the audio packet is sent, instantaneous value when obtaining this data.
  final int? txAudioKBitrate;

  /// The code rate (kbps) at which the audio packet is received, instantaneous value when obtaining this data.
  final int? rxAudioKBitrate;

  /// The code rate (kbps) at which the video packet is sent, instantaneous value when obtaining this data.
  final int? txVideoKBitrate;

  /// The code rate (kbps) at which the video packet is received, instantaneous value when obtaining this data.
  final int? rxVideoKBitrate;

  /// The code rate (kbps) at which the screen stream packet is sent, instantaneous value when obtaining this data.
  final int? txScreenKBitrate;

  /// The code rate (kbps) at which the screen stream packet is received, instantaneous value when obtaining this data.
  final int? rxScreenKBitrate;

  /// Number of visible users in the current room.
  final int? userCount;

  /// The uplink packet loss rate of the current application, in the range of `[0, 1]`.
  final double? txLostrate;

  /// The downlink packet loss rate of the current application, in the range of `[0, 1]`.
  final double? rxLostrate;

  /// Round-trip time delay for client-to-server data transfer in ms.
  final int? rtt;

  /// @nodoc('Not available')
  /// System uplink network jitter in ms.
  final int? txJitter;

  /// @nodoc('Not available')
  /// System downlink network jitter in ms.
  final int? rxJitter;

  /// The code rate (kbps) sent by the cellular path, instantaneous value when obtaining this data.
  final int? txCellularKBitrate;

  /// The code rate (kbps) received by the cellular path, instantaneous value when obtaining this data.
  final int? rxCellularKBitrate;

  /// @nodoc
  const RTCRoomStats({
    this.duration,
    this.txBytes,
    this.rxBytes,
    this.txKBitrate,
    this.rxKBitrate,
    this.txAudioKBitrate,
    this.rxAudioKBitrate,
    this.txVideoKBitrate,
    this.rxVideoKBitrate,
    this.txScreenKBitrate,
    this.rxScreenKBitrate,
    this.userCount,
    this.txLostrate,
    this.rxLostrate,
    this.rtt,
    this.txJitter,
    this.rxJitter,
    this.txCellularKBitrate,
    this.rxCellularKBitrate,
  });

  /// @nodoc
  factory RTCRoomStats.fromMap(Map<dynamic, dynamic> map) {
    return RTCRoomStats(
      duration: map['duration'],
      txBytes: map['txBytes'],
      rxBytes: map['rxBytes'],
      txKBitrate: map['txKBitrate'],
      rxKBitrate: map['rxKBitrate'],
      txAudioKBitrate: map['txAudioKBitrate'],
      rxAudioKBitrate: map['rxAudioKBitrate'],
      txVideoKBitrate: map['txVideoKBitrate'],
      rxVideoKBitrate: map['rxVideoKBitrate'],
      txScreenKBitrate: map['txScreenKBitrate'],
      rxScreenKBitrate: map['rxScreenKBitrate'],
      userCount: map['userCount'],
      txLostrate: map['txLostrate'],
      rxLostrate: map['rxLostrate'],
      rtt: map['rtt'],
      txJitter: map['txJitter'],
      rxJitter: map['rxJitter'],
      txCellularKBitrate: map['txCellularKBitrate'],
      rxCellularKBitrate: map['rxCellularKBitrate'],
    );
  }
}

/// Local video stream statistics called back every 2 seconds.
///
/// After a local user successfully publishes a video stream, the SDK periodically notifies the user of the status of the published video stream during this statistics cycle through [RTCRoomEventHandler.onLocalStreamStats].
class LocalVideoStats {
  /// The sending code rate in Kbps of the video stream with the largest resolution actually published in this statistical period.
  final double? sentKBitrate;

  /// The capture frame rate in Hz for this statistical period.
  final int? inputFrameRate;

  /// The video send sample rate in Hz for this statistical period.
  final int? sentFrameRate;

  /// Encoder output frame rate in fps for the current statistical period.
  final int? encoderOutputFrameRate;

  /// The local video rendering frame rate in fps for this statistical period.
  final int? renderOutputFrameRate;

  /// The interval of this statistical period, in ms.
  ///
  /// This field is used to set the statistics period of the callback, the default setting is 2000ms.
  final int? statsInterval;

  /// The video uplink packet loss rate in this statistical period, the value range is `[0, 1]`.
  final double? videoLossRate;

  /// Round-trip time delay in ms.
  final int? rtt;

  /// The video encoding bit rate in kbps of the largest resolution video stream actually published in this statistical period.
  final int? encodedBitrate;

  /// The video encoding width in px of the video stream with the largest resolution actually published.
  final int? encodedFrameWidth;

  /// The video encoding height in px of the video stream with the largest resolution actually published.
  final int? encodedFrameHeight;

  /// The total number of video frames sent for the video stream with the largest resolution actually published during this statistical period.
  final int? encodedFrameCount;

  /// Encoding type of the video.
  final VideoCodecType? codecType;

  /// Whether the media stream is from screen sharing.
  final bool? isScreen;

  /// Video uplink network jitter in ms.
  final int? jitter;

  /// Video noise reduction mode.
  final VideoDenoiseMode? videoDenoiseMode;

  /// @nodoc
  const LocalVideoStats({
    this.sentKBitrate,
    this.inputFrameRate,
    this.sentFrameRate,
    this.encoderOutputFrameRate,
    this.renderOutputFrameRate,
    this.statsInterval,
    this.videoLossRate,
    this.rtt,
    this.encodedBitrate,
    this.encodedFrameWidth,
    this.encodedFrameHeight,
    this.encodedFrameCount,
    this.codecType,
    this.isScreen,
    this.jitter,
    this.videoDenoiseMode,
  });

  /// @nodoc
  factory LocalVideoStats.fromMap(Map<dynamic, dynamic> map) {
    return LocalVideoStats(
      sentKBitrate: map['sentKBitrate'],
      inputFrameRate: map['inputFrameRate'],
      sentFrameRate: map['sentFrameRate'],
      encoderOutputFrameRate: map['encoderOutputFrameRate'],
      renderOutputFrameRate: map['renderOutputFrameRate'],
      statsInterval: map['statsInterval'],
      videoLossRate: map['videoLossRate'],
      rtt: map['rtt'],
      encodedBitrate: map['encodedBitrate'],
      encodedFrameWidth: map['encodedFrameWidth'],
      encodedFrameHeight: map['encodedFrameHeight'],
      encodedFrameCount: map['encodedFrameCount'],
      codecType: (map['codecType'] as int?).videoCodecType,
      isScreen: map['isScreen'],
      jitter: map['jitter'],
      videoDenoiseMode: (map['videoDenoiseMode'] as int?).videoDenoiseMode,
    );
  }
}

/// Remote video stream statistics called back every 2 seconds.
///
/// After a local user successfully subscribed to a remote video stream, the SDK periodically notifies the user of the status of the received video stream during this statistics cycle through [RTCRoomEventHandler.onRemoteStreamStats].
class RemoteVideoStats {
  /// Remote video stream width
  final int? width;

  /// Remote video stream height
  final int? height;

  /// The video downlink packet loss rate in % in the range of `[0, 1]` for this statistical period.
  final double? videoLossRate;

  /// The received bit rate of the video in kbps for this statistical period.
  final double? receivedKBitrate;

  /// The frame rate of the video decoder output in fps for this statistical period.
  final int? decoderOutputFrameRate;

  /// The frame rate of the video rendering in fps during the statistical period.
  final int? renderOutputFrameRate;

  /// Number of stalls in the statistical cycle.
  final int? stallCount;

  /// The total duration of video stalls in the statistics period in ms.
  final int? stallDuration;

  /// Delay in ms from the completion of encoding at the sender-side to the completion of decoding and rendering at the receiver-side
  final int? e2eDelay;

  /// Whether the media stream is from screen sharing.
  final bool? isScreen;

  /// The interval of this statistical period, in ms.
  ///
  /// This field is used to set the statistics period of the callback, the default setting is 2000ms.
  final int? statsInterval;

  /// Round-trip time delay in ms.
  final int? rtt;

  /// Cumulative duration of video stalls occurring after the remote user enters the room as a percentage of the total effective duration of the video.
  ///
  /// Effective video duration is the duration of video after the remote user enters the room and publishes the video stream, except for stopping sending the video stream and disabling the video module.
  final int? frozenRate;

  /// Encoding type of the video.
  final VideoCodecType? codecType;

  /// Numbering of video streams with different resolution.
  final int? videoIndex;

  /// Video downlink network jitter in ms.
  final int? jitter;

  /// Super-resolution mode.
  final VideoSuperResolutionMode? superResolutionMode;

  /// @nodoc
  const RemoteVideoStats({
    this.width,
    this.height,
    this.videoLossRate,
    this.receivedKBitrate,
    this.decoderOutputFrameRate,
    this.renderOutputFrameRate,
    this.stallCount,
    this.stallDuration,
    this.e2eDelay,
    this.isScreen,
    this.statsInterval,
    this.rtt,
    this.frozenRate,
    this.codecType,
    this.videoIndex,
    this.jitter,
    this.superResolutionMode,
  });

  /// @nodoc
  factory RemoteVideoStats.fromMap(Map<dynamic, dynamic> map) {
    return RemoteVideoStats(
      width: map['width'],
      height: map['height'],
      videoLossRate: map['videoLossRate'],
      receivedKBitrate: map['receivedKBitrate'],
      decoderOutputFrameRate: map['decoderOutputFrameRate'],
      renderOutputFrameRate: map['renderOutputFrameRate'],
      stallCount: map['stallCount'],
      stallDuration: map['stallDuration'],
      e2eDelay: map['e2eDelay'],
      isScreen: map['isScreen'],
      statsInterval: map['statsInterval'],
      rtt: map['rtt'],
      frozenRate: map['frozenRate'],
      codecType: (map['codecType'] as int).videoCodecType,
      videoIndex: map['videoIndex'],
      jitter: map['jitter'],
      superResolutionMode:
          (map['superResolutionMode'] as int?).videoSuperResolutionMode,
    );
  }
}

/// Local audio stream statistics called back every 2 seconds.
///
/// After a local user successfully publishes an audio stream, the SDK periodically notifies the user of the status of the published audio stream during this statistics cycle through [RTCRoomEventHandler.onLocalStreamStats].
class LocalAudioStats {
  /// Audio uplink packet loss rate in % for this statistical period, in the range `[0, 1]`.
  final double? audioLossRate;

  /// The audio transmission code rate in kbps for this statistical period.
  final double? sentKBitrate;

  /// The audio capture sample rate in Hz for this statistical period.
  final int? recordSampleRate;

  /// The interval of this statistical period, in ms.
  ///
  /// This field is used to set the statistics period of the callback, the default setting is 2000ms.
  final int? statsInterval;

  /// Round-trip time delay in ms.
  final int? rtt;

  /// Number of audio channels.
  final int? numChannels;

  /// The audio send sample rate in Hz for this statistical period.
  final int? sentSampleRate;

  /// Audio uplink network jitter in ms.
  final int? jitter;

  /// @nodoc
  const LocalAudioStats({
    this.audioLossRate,
    this.sentKBitrate,
    this.recordSampleRate,
    this.statsInterval,
    this.rtt,
    this.numChannels,
    this.sentSampleRate,
    this.jitter,
  });

  /// @nodoc
  factory LocalAudioStats.fromMap(Map<dynamic, dynamic> map) {
    return LocalAudioStats(
      audioLossRate: map['audioLossRate'],
      sentKBitrate: map['sentKBitrate'],
      recordSampleRate: map['recordSampleRate'],
      statsInterval: map['statsInterval'],
      rtt: map['rtt'],
      numChannels: map['numChannels'],
      sentSampleRate: map['sentSampleRate'],
      jitter: map['jitter'],
    );
  }
}

/// Remote audio stream statistics called back every 2 seconds.
///
/// After a local user successfully subscribed to a remote audio stream, the SDK periodically notifies the user of the status of the received audio stream during this statistics cycle through [RTCRoomEventHandler.onRemoteStreamStats].
class RemoteAudioStats {
  /// Audio downlink packet loss rate in % for this statistical period, in the range `[0, 1]`.
  final double? audioLossRate;

  /// The audio reception code rate in kbps for this statistical period.
  final double? receivedKBitrate;

  /// The number of stalls in this statistical cycle.
  final int? stallCount;

  /// The duration of the stall in this statistical period, in ms.
  final int? stallDuration;

  /// User experience level end-to-end latency.
  ///
  /// The delay time in ms from the start of encoding at the sender-side after capturing to the start of rendering at the receiver-side after decoding.
  final int? e2eDelay;

  /// The audio playback sample rate in Hz for the statistical period.
  final int? playoutSampleRate;

  /// The interval of this statistical period, in ms.
  final int? statsInterval;

  /// The round-trip time delay of the data transfer from the client to the server, in ms.
  final int? rtt;

  /// Sender-server-receiver data transmission time delay in ms.
  final int? totalRtt;

  /// The quality of the audio stream sent by the remote user.
  ///
  /// See [NetworkQuality] for the value meaning.
  final int? quality;

  /// The delay in ms due to the introduction of the jitter buffer mechanism.
  final int? jitterBufferDelay;

  /// Number of audio channels.
  final int? numChannels;

  /// Audio reception sample rate in Hz for this statistical cycle.
  final int? receivedSampleRate;

  /// Cumulative duration of audio stall occurring after the remote user joins the room as a percentage of the total effective duration of audio.
  ///
  /// Effective audio duration is the duration after the remote user enters the room and publishes the audio stream, except for stopping sending the audio stream and disabling the audio module.
  final int? frozenRate;

  /// Total number of audio packet loss compensation (PLC) sample points.
  final int? concealedSamples;

  /// Cumulative times of PLC
  final int? concealmentEvent;

  /// The audio decoding sample rate in Hz for this statistical period.
  final int? decSampleRate;

  /// Cumulative decoding time in seconds of the remote audio stream in this subscription.
  final int? decDuration;

  /// Audio downlink network jitter in ms.
  final int? jitter;

  /// @nodoc
  const RemoteAudioStats({
    this.audioLossRate,
    this.receivedKBitrate,
    this.stallCount,
    this.stallDuration,
    this.e2eDelay,
    this.playoutSampleRate,
    this.statsInterval,
    this.rtt,
    this.totalRtt,
    this.quality,
    this.jitterBufferDelay,
    this.numChannels,
    this.receivedSampleRate,
    this.frozenRate,
    this.concealedSamples,
    this.concealmentEvent,
    this.decSampleRate,
    this.decDuration,
    this.jitter,
  });

  /// @nodoc
  factory RemoteAudioStats.fromMap(Map<dynamic, dynamic> map) {
    return RemoteAudioStats(
      audioLossRate: map['audioLossRate'],
      receivedKBitrate: map['receivedKBitrate'],
      stallCount: map['stallCount'],
      stallDuration: map['stallDuration'],
      e2eDelay: map['e2eDelay'],
      playoutSampleRate: map['playoutSampleRate'],
      statsInterval: map['statsInterval'],
      rtt: map['rtt'],
      totalRtt: map['totalRtt'],
      quality: map['quality'],
      jitterBufferDelay: map['jitterBufferDelay'],
      numChannels: map['numChannels'],
      receivedSampleRate: map['receivedSampleRate'],
      frozenRate: map['frozenRate'],
      concealedSamples: map['concealedSamples'],
      concealmentEvent: map['concealmentEvent'],
      decSampleRate: map['decSampleRate'],
      decDuration: map['decDuration'],
      jitter: map['jitter'],
    );
  }
}

/// Local audio/video stream statistics and network status called back every 2s.
class LocalStreamStats {
  /// Statistics of audio stream published by local device.
  final LocalAudioStats? audioStats;

  /// Statistics of video stream published by local device.
  final LocalVideoStats? videoStats;

  /// Whether the media stream is from screen sharing.
  final bool? isScreen;

  /// @nodoc
  const LocalStreamStats({
    this.audioStats,
    this.videoStats,
    this.isScreen,
  });

  /// @nodoc
  factory LocalStreamStats.fromMap(Map<dynamic, dynamic> map) {
    return LocalStreamStats(
      audioStats: LocalAudioStats.fromMap(map['audioStats']),
      videoStats: LocalVideoStats.fromMap(map['videoStats']),
      isScreen: map['isScreen'],
    );
  }
}

/// Statistical information and network status of subscribed remote audio/video stream called back every 2s.
class RemoteStreamStats {
  /// ID of the user who published the audio
  final String? uid;

  /// Statistics of remote audio stream
  final RemoteAudioStats? audioStats;

  /// Statistics of remote video stream
  final RemoteVideoStats? videoStats;

  /// Whether the media stream is from screen sharing.
  final bool? isScreen;

  /// @nodoc
  const RemoteStreamStats({
    this.uid,
    this.audioStats,
    this.videoStats,
    this.isScreen,
  });

  /// @nodoc
  factory RemoteStreamStats.fromMap(Map<dynamic, dynamic> map) {
    return RemoteStreamStats(
      audioStats: RemoteAudioStats.fromMap(map['audioStats']),
      videoStats: RemoteVideoStats.fromMap(map['videoStats']),
      uid: map['uid'],
      isScreen: map['isScreen'],
    );
  }
}

/// Information about the remote stream
class RemoteStreamKey {
  /// The ID of the room from which the stream was published.
  final String roomId;

  /// The ID of the user who published the stream.
  final String uid;

  /// Stream type.
  final StreamIndex streamIndex;

  /// @nodoc
  const RemoteStreamKey({
    required this.roomId,
    required this.uid,
    required this.streamIndex,
  });

  /// @nodoc
  factory RemoteStreamKey.fromMap(Map<dynamic, dynamic> map) {
    return RemoteStreamKey(
      roomId: map['roomId'],
      uid: map['uid'],
      streamIndex: (map['streamIndex'] as int).streamIndex,
    );
  }

  /// @nodoc
  Map<String, dynamic> toMap() => {
        'roomId': roomId,
        'uid': uid,
        'streamIndex': streamIndex.index,
      };
}

/// Local recording configuration
class RecordingConfig {
  /// The absolute path to save the recording file. You need to specify a legal path with read and write permissions.
  String dirPath;

  /// Storage Format。
  RecordingFileType recordingFileType;

  /// @nodoc
  RecordingConfig({
    required this.dirPath,
    this.recordingFileType = RecordingFileType.mp4,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'dirPath': dirPath,
      'recordingFileType': recordingFileType.index,
    };
  }
}

/// Local recording progress
class RecordingProgress {
  /// The cumulative recording time of the current file, in milliseconds.
  final int? duration;

  /// The size of the currently recorded file in bytes.
  final int? fileSize;

  /// @nodoc
  const RecordingProgress({
    this.duration,
    this.fileSize,
  });

  /// @nodoc
  factory RecordingProgress.fromMap(Map<dynamic, dynamic> map) {
    return RecordingProgress(
      duration: map['duration'],
      fileSize: map['fileSize'],
    );
  }
}

/// Details of local recording
class RecordingInfo {
  /// The absolute path of the recorded file, including file name and file suffix.
  final String? filePath;

  /// Video encoding type of recorded files.
  final VideoCodecType? videoCodecType;

  /// The width of the recorded video, in pixels. Please ignore this field for audio-only recording.
  final int? width;

  /// The height of the recorded video, in pixels. Please ignore this field for audio-only recording.
  final int? height;

  /// @nodoc
  const RecordingInfo({
    this.filePath,
    this.videoCodecType,
    this.width,
    this.height,
  });

  /// @nodoc
  factory RecordingInfo.fromMap(Map<dynamic, dynamic> map) {
    return RecordingInfo(
      filePath: map['filePath'],
      videoCodecType: (map['videoCodecType'] as int).videoCodecType,
      width: map['width'],
      height: map['height'],
    );
  }
}

/// Configuration related to media streaming information synchronization
class StreamSyncInfoConfig {
  /// Stream type.
  StreamIndex streamIndex;

  /// The number of repetitions of the message sent, in the range `[0,25]`. It is recommended to be set to `[3,5]`.
  int repeatCount;

  /// Stream type for synchronization.
  SyncInfoStreamType streamType;

  /// @nodoc
  StreamSyncInfoConfig({
    this.streamIndex = StreamIndex.main,
    this.repeatCount = 0,
    this.streamType = SyncInfoStreamType.audio,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'streamIndex': streamIndex.index,
      'repeatCount': repeatCount,
      'streamType': streamType.index,
    };
  }
}

/// Information about the target room for media stream relaying across rooms.
class ForwardStreamInfo {
  /// Target room ID
  String roomId;

  /// Token generated using the target room RoomID and UserID.
  ///
  /// For testing, you can use the console to generate temporary Token, for official launch you need to use the key SDK to generate and issue Token on your server.<br>
  /// Invalid token will cause relaying failure.
  String token;

  /// @nodoc
  ForwardStreamInfo({
    required this.roomId,
    required this.token,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'roomId': roomId,
      'token': token,
    };
  }
}

/// Status and error messages for different target rooms during cross-room relaying of media streams
class ForwardStreamStateInfo {
  /// Target room ID.
  ///
  /// Empty string represents all target rooms.
  final String? roomId;

  /// Status of the target room.
  final ForwardStreamState? state;

  /// Error that occurs during relaying.
  final ForwardStreamError? error;

  /// @nodoc
  const ForwardStreamStateInfo({
    this.roomId,
    this.state,
    this.error,
  });

  /// @nodoc
  factory ForwardStreamStateInfo.fromMap(Map<dynamic, dynamic> map) {
    return ForwardStreamStateInfo(
      roomId: map['roomId'],
      state: (map['state'] as int).forwardStreamState,
      error: (map['error'] as int).forwardStreamError,
    );
  }
}

/// Information on events occurring in different target room during media stream relaying.
class ForwardStreamEventInfo {
  /// Target room ID.
  ///
  /// Empty string represents all target rooms.
  final String? roomId;

  /// Events occurring in this target room.
  final ForwardStreamEvent? event;

  /// @nodoc
  const ForwardStreamEventInfo({
    this.roomId,
    this.event,
  });

  /// @nodoc
  factory ForwardStreamEventInfo.fromMap(Map<dynamic, dynamic> map) {
    return ForwardStreamEventInfo(
      roomId: map['roomId'],
      event: (map['event'] as int).forwardStreamEvent,
    );
  }
}

/// Tx/Rx network quality related data
class NetworkQualityStats {
  /// User ID
  final String? uid;

  /// Packet loss rate, and the range is [0.0,1.0].
  /// + For a local user, it is the sent-packet loss ratio.
  /// + For a remote user, it is the loss ratio of all the packets received.
  final double? fractionLost;

  /// Round-trip time (RTT) in ms from client to server. Effective for the local user.
  final int? rtt;

  /// Average transmission rate(bps) of the media RTP packages in 2s.
  /// + For a local user, it is the packet-sending speed.
  /// + For a more user, it is the speed of receiving all the subscribed medias.
  final int? totalBandwidth;

  /// Tx network quality grade in a range of [0,5]. The higher the grade, the worse the network quality.
  final NetworkQuality? txQuality;

  /// Rx network quality grade in a range of [0,5]. The higher the grade, the worse the network quality.
  final NetworkQuality? rxQuality;

  /// @nodoc
  const NetworkQualityStats({
    this.uid,
    this.fractionLost,
    this.rtt,
    this.totalBandwidth,
    this.txQuality,
    this.rxQuality,
  });

  /// @nodoc
  factory NetworkQualityStats.fromMap(Map<dynamic, dynamic> map) {
    return NetworkQualityStats(
      uid: map['uid'],
      fractionLost: map['fractionLost'],
      rtt: map['rtt'],
      totalBandwidth: map['totalBandwidth'],
      txQuality: (map['txQuality'] as int).networkQuality,
      rxQuality: (map['rxQuality'] as int).networkQuality,
    );
  }
}

/// Network time info
class NetworkTimeInfo {
  /// Network time in ms.
  final int timestamp;

  /// @nodoc
  const NetworkTimeInfo({
    required this.timestamp,
  });

  /// @nodoc
  factory NetworkTimeInfo.fromMap(Map<dynamic, dynamic> map) {
    return NetworkTimeInfo(
      timestamp: map['timestamp'],
    );
  }
}

/// Reasons for stopping network detection before a call
enum NetworkDetectionStopReason {
  /// The user stops the detection.
  user,

  /// Detection has been on for more than 3 minutes and has stopped automatically.
  timeout,

  /// Detection network disconnected.
  ///
  /// When no reply is received for more than 12s, the SDK will disconnect from the network and will not attempt to reconnect.
  connectionLost,

  /// Detection stops because local user has already started to publish and receive streams.
  streaming,

  /// Internal exceptions.
  innerErr,
}

/// Type of network detected before a call.
enum NetworkDetectionLinkType {
  /// Uplink network
  up,

  /// Downlink network
  down,
}

/// Result of the detection inited before joining a room.
enum HardwareEchoDetectionResult {
  /// Detection is stopped by the call of [RTCVideo.stopHardwareEchoDetection] before the SDK reports the detection result.
  canceled,

  /// Unknown state
  ///
  /// Contact the technical support if you retry several times but keep failing.
  unknown,

  /// Excellent
  ///
  /// No echo issue is detected.
  normal,

  /// Echo
  ///
  /// Echo issue is detected. Recommend the user on the user interface to join the call with a headset.
  poor,
}

/// User priority configuration in the audio selection.
enum AudioSelectionPriority {
  /// Normal. Whether the stream can be subscribed is determined by the result of audio selection.
  normal,

  /// Hight priority, with which the stream can skip the audio selection and always be subscribable.
  high,
}

/// Extra information setting result.
enum SetRoomExtraInfoResult {
  /// Success.
  success,

  /// Failure. The use is not in the room.
  notJoinRoom,

  /// Failure. The key pointer is null.
  keyIsNull,

  /// Failure. The value pointer is null.
  valueIsNull,

  /// Failure. Unknown error.
  unknown,

  /// Failure. The key length is 0.
  keyIsEmpty,

  /// Failure. Excessively frequent setting. 10 times per second is preferable.
  tooOften,

  /// Failure. Invisible users are not allowed to call [RTCRoom.setRoomExtraInfo].
  silentUser,

  /// Failure. Key length exceeds 10 bytes.
  keyTooLong,

  /// Failure. Value length exceeds 128 bytes.
  valueTooLong,

  /// Failure. Server error.
  serverError,
}

/// The states of the subtitling task.
enum SubtitleState {
  /// Subtitles are on.
  started,

  /// Subtitles are off.
  stopped,

  /// Errors occurred concerning the subtitling task.
  error,
}

/// Subtitle modes.
enum SubtitleMode {
  /// The recognition mode, in which the speech of a users in the room will be recognized and converted into captions.
  recognition,

  /// The translation mode, in which the speech of a users in the room will be converted into captions and then translated.
  translation,
}

/// Error codes of the subtitling task.
enum SubtitleErrorCode {
  /// The client side failed to identity error codes sent by cloud media processing. Please contact the technical support.
  unknown,

  /// Subtitles are turned on.
  success,

  ///  Errors occurred concerning cloud media processing. Please contact the technical support.
  postProcessError,

  /// Failed to connect to the third-party service. Please contact the technical support.
  asrConnectionError,

  /// Errors occurred concerning the third-party service. Please contact the technical support.
  asrServiceError,

  /// Failed to call [RTCRoom.startSubtitle].
  beforeJoinRoom,

  ///  Subtitles are already on. There is no need to call [RTCRoom.startSubtitle] again.
  alreadyOn,

  /// The selected target language is not Unsupported.
  unsupportedLanguage,

  /// Cloud media processing is timeout. Please contact the technical support.
  postProcessTimeout,
}

/// Subtitle configurations.
class SubtitleConfig {
  /// Subtitle mode.
  ///
  /// You can choose between the recognition and translation modes based on your needs. In the recognition mode, the speech of the speaker in the room will be recognized and converted into captions. In the translation mode, the transcribed text will be translated.
  SubtitleMode mode;

  /// Target language.
  String targetLanguage;

  /// @nodoc
  SubtitleConfig({
    this.mode = SubtitleMode.recognition,
    this.targetLanguage = '',
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'mode': mode.index,
      'targetLanguage': targetLanguage,
    };
  }
}

/// Related information about subtitles.
class SubtitleMessage {
  /// The speaker's user ID.
  final String uid;

  /// Transcribed or translated texts of the speaker's speech, encoded in UTF-8 format.
  final String text;

  /// Incremental sequence numbers assigned to transcribed or translated texts of the speaker's speech. Complete and incomplete sentences are numbered individually.
  final int sequence;

  /// Whether transcribed texts are complete sentences. True means yes and False means no.
  final bool definite;

  /// @nodoc
  const SubtitleMessage({
    required this.uid,
    required this.text,
    required this.sequence,
    required this.definite,
  });

  /// @nodoc
  factory SubtitleMessage.fromMap(Map<dynamic, dynamic> map) {
    return SubtitleMessage(
      uid: map['uid'],
      text: map['text'],
      sequence: map['sequence'],
      definite: map['definite'],
    );
  }
}

/// User visibility state change error.
enum UserVisibilityChangeError {
  /// Success.
  ok,

  /// Unknown error.
  unknown,

  /// The number of visible users in the room reaches the upper limit.
  tooManyVisibleUser,
}

/// @nodoc('Not available')
/// Video noise reduction mode.
enum VideoDenoiseMode {
  /// The video noise reduction mode is off.
  off,

  /// Video noise reduction mode is turned on. The algorithm is configured automatically.
  auto,
}

/// @nodoc('Not available')
/// Super-resolution mode.
enum VideoSuperResolutionMode {
  /// Turn off super-resolution mode.
  off,

  /// Turn on super-resolution mode.
  on,
}
