// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'bytertc_audio_defines.dart';
import 'bytertc_video_api.dart';

/// Audio mixing configuration
class MediaPlayerConfig {
  /// Mixed audio playback types.
  final AudioMixingType type;

  /// Mix playback times
  /// + Play_count < = 0: Infinite loop
  /// + Play_count == 1: Play once (default)
  /// + Play_count > 1: Play play_count times
  final int playCount;

  /// The starting position in ms. 0 by default.
  final int startPos;

  /// Set the interval of the periodic callback [RTCMediaPlayerEventHandler.onMediaPlayerPlayingProgress] during audio mixing in ms.
  /// + interval > 0: The callback is enabled. The actual interval is `10*(mod(10)+1)`.
  /// + interval <= 0: The callback is disabled.
  final int callbackOnProgressInterval;

  /// Attach the process information of local audio file mixing to the captured audio data. Enable the function to enhance the synchronicity of the remote audio mixing.
  /// + The function is effective when mixing a single audio file.
  /// + Use `true` for enabling the function and `false` for disable the function. The default is `false`.
  final bool syncProgressToRecordFrame;

  /// Play the audio automatically. If not, call [RTCMediaPlayer.start] to play the audio.
  final bool autoPlay;

  /// @nodoc
  const MediaPlayerConfig({
    this.type = AudioMixingType.playoutAndPublish,
    this.playCount = 1,
    this.startPos = 0,
    this.callbackOnProgressInterval = 0,
    this.syncProgressToRecordFrame = false,
    this.autoPlay = true,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'type': type.index,
      'playCount': playCount,
      'startPos': startPos,
      'callbackOnProgressInterval': callbackOnProgressInterval,
      'syncProgressToRecordFrame': syncProgressToRecordFrame,
      'autoPlay': autoPlay,
    };
  }
}

/// [playerId]: The ID of [RTCMediaPlayer], set by calling [RTCVideo.getMediaPlayer].
///
/// [state]: Media player state.
///
/// [error]: Media player error.
typedef OnMediaPlayerStateChangedType = void Function(
    int playerId, PlayerState state, PlayerError error);

/// [playerId]: The ID of [RTCMediaPlayer], set by calling [RTCVideo.getMediaPlayer].
///
/// [progress]: Mixing progress in ms.
typedef OnMediaPlayerPlayingProgressType = void Function(
    int playerId, int progress);

/// Event handler for [RTCMediaPlayer]. You must call [RTCMediaPlayer.setEventHandler] to set the corresponding event handler to get the events.
class RTCMediaPlayerEventHandler {
  /// Callback for audio mixing status change.
  OnMediaPlayerStateChangedType? onMediaPlayerStateChanged;

  /// Periodic callback for audio mixing progress. The period is set by [RTCMediaPlayer.setProgressInterval].
  OnMediaPlayerPlayingProgressType? onMediaPlayerPlayingProgress;

  /// @nodoc
  RTCMediaPlayerEventHandler({
    this.onMediaPlayerStateChanged,
    this.onMediaPlayerPlayingProgress,
  });
}

/// Audio effect player.
///
/// Call [RTCMediaPlayer.setEventHandler] to set the callback handler to receive related callbacks.
///
/// Valid since v3.56.1
abstract class RTCMediaPlayer {
  /// Open the audio file.<br>
  /// You can only open one audio file with one player instance at the same time. For multiple audio files at the same time, create multiple player instances.
  ///
  /// [filePath]: Audio file paths.<br>
  /// URL of online file, URI of local file, full path to local file, or path to local file starting with `/assets/` are supported. For URL of online file, only the https protocol is supported.<br>
  /// Recommended sample rate for audio effect files: 8KHz、16KHz、22.05KHz、44.1KHz、48KHz.<br>
  /// Local audio effect file formats supported by different platforms:
  /// <table border>
  ///    <tr><th></th><th>mp3</th><th>mp4</th><th>aac</th><th>m4a</th><th>3gp</th><th>wav</th><th>ogg</th><th>ts</th><th>wma</th></tr>
  ///    <tr><td>Android</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td></td></tr>
  ///    <tr><td>iOS/macOS</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td></td><td></td></tr>
  ///    <tr><td>Windows</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td>Y</td><td>Y</td></tr>
  /// </table>
  /// Online audio effect file formats supported by different platforms:
  /// <table border>
  ///    <tr><th></th><th>mp3</th><th>mp4</th><th>aac</th><th>m4a</th><th>3gp</th><th>wav</th><th>ogg</th><th>ts</th><th>wma</th></tr>
  ///    <tr><td>Android</td><td>Y</td><td></td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td></td><td></td></tr>
  ///    <tr><td>iOS/macOS</td><td>Y</td><td></td><td>Y</td><td>Y</td><td></td><td>Y</td><td></td><td></td><td></td></tr>
  ///    <tr><td>Windows</td><td>Y</td><td></td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td></td><td>Y</td><td>Y</td></tr>
  /// </table>
  ///
  /// [config]: Player configuration.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  Future<int?> open({
    required String filePath,
    MediaPlayerConfig config = const MediaPlayerConfig(),
  });

  /// Start playing the audio. Call this API when you call [RTCMediaPlayer.open] and set `AutoPlay=False`.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// Call [RTCMediaPlayer.stop] to stop playing the audio file.
  Future<int?> start();

  /// After calling [RTCMediaPlayer.open] or [RTCMediaPlayer.start] to start audio mixing, call this method to stop audio mixing.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  Future<int?> stop();

  /// After calling [RTCMediaPlayer.open] or [RTCMediaPlayer.start] to start audio mixing, call this API to pause audio mixing.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// After calling this API to pause audio mixing, call [RTCMediaPlayer.resume] to resume audio mixing.
  Future<int?> pause();

  /// After calling [RTCMediaPlayer.pause] to pause audio mixing, call this API to resume audio mixing.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  Future<int?> resume();

  /// Adjusts the volume of the specified audio mixing.
  ///
  /// [volume]: The ratio of the volume to the original volume in % with overflow protection. The range is `[0, 400]` and the recommended range is `[0, 100]`.
  ///
  /// [type]: Specifies whether played at local devices or sent to remote devices.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// Call this API only when the audio is mixing.
  Future<int?> setVolume({
    required int volume,
    AudioMixingType type = AudioMixingType.playoutAndPublish,
  });

  /// Gets the current volume.
  ///
  /// Return value:
  /// + `>0`: The current volume.
  /// + `<0`: Failure.
  ///
  /// Call this API only when audio is mixing.
  Future<int?> getVolume([
    AudioMixingType type = AudioMixingType.playoutAndPublish,
  ]);

  /// Gets the duration of the media file.
  ///
  /// Return value:
  /// + `>0`: The duration of the media file in milliseconds.
  /// + `<0`: Failure.
  ///
  /// Call this API only when audio is mixing.
  Future<int?> getTotalDuration();

  /// Gets the actual playback duration of the mixed media file, in milliseconds.
  ///
  /// Return value:
  /// + `>0`: The actual playback time.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + The actual playback time refers to the playback time of the song without being affected by stop, jump, double speed, and freeze. For example, if the song stops playing for 30 seconds at 1:30 or skips to 2:00, and then continues to play normally for 2 minutes, the actual playing time is 3 minutes and 30 seconds.
  /// + Call this API only when audio is mixing.
  Future<int?> getPlaybackDuration();

  /// Gets the playback progress of the media file.
  ///
  /// Return value:
  /// + `>0`: The playback progress of media file in ms.
  /// + `<0`: Failure.
  ///
  /// Call this API only when audio is mixing.
  Future<int?> getPosition();

  /// Sets the pitch of the local audio file mixing. Usually used in karaoke scenes.
  ///
  /// [pitch]: The increase or decrease value compared with the original pitch of the music file. The range is `[-12, 12]`. The default value is 0. The pitch distance between two adjacent values is half a step. A positive value indicates a rising pitch, and a negative value indicates a falling pitch.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// Call this API only when audio is mixing.
  Future<int?> setAudioPitch(int pitch);

  /// Sets the starting playback position of the media file.
  ///
  /// [position]: The starting position of the media file in milliseconds.<br>
  /// You can get the total duration of the media file through [RTCMediaPlayer.getTotalDuration]. The value of position should be less than the total duration of the media file.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// When playing online files, calling this API may cause playback delay.
  Future<int?> setPosition(int position);

  /// Sets the channel mode of the mixing of the media file.
  ///
  /// [mode]: The mode of channel. The default channel mode is the same as the source file.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// Call this API only when audio is mixing.
  Future<int?> setAudioDualMonoMode(AudioMixingDualMonoMode mode);

  /// Gets the track count of the current media file.
  ///
  /// Return value:
  /// + `>= 0`: The track count of the current media file.
  /// + `<0`: Failure.
  ///
  /// Call this API only when audio is mixing.
  Future<int?> getAudioTrackCount();

  /// Specifies the playback track of the current media file.
  ///
  /// [index]: The specified playback audio track, starting from 0, and the range is `[0, getAudioTrackCount()-1]`.<br>
  /// The value must be less than the return value of [RTCMediaPlayer.getAudioTrackCount].
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// Call this API only when audio is mixing.
  Future<int?> selectAudioTrack(int index);

  /// Sets the playback speed of the audio file.
  ///
  /// [speed]: The ratio of the actual playback speed than that of the original speed of the audio file in %. The range is `[50,200]`. The default value is 100.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// Call this API only when audio is mixing.
  Future<int?> setPlaybackSpeed(int speed);

  /// Sets the interval of the periodic callback [RTCMediaPlayerEventHandler.onMediaPlayerPlayingProgress] during audio mixing.
  ///
  /// [interval]: Interval in ms.
  /// + interval > 0: The callback is enabled. The actual interval is `10*(mod(10)+1)`.
  /// + interval <= 0: The callback is disabled.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// Call this API only when audio is mixing.
  Future<int?> setProgressInterval(int interval);

  /// To call [RTCVideo.enableVocalInstrumentBalance] to adjust the volume of the mixed media file or the PCM audio data, you must pass in its original loudness through this API.
  ///
  /// [loudness]: Original loudness in lufs. The range is `[-70.0, 0.0]`.<br>
  /// When the value is less than -70.0lufs, it will be adjusted to -70.0lufs by default, and if it is more than 0.0lufs, the loudness will not be equalized. The default value is 1.0lufs, which means no processing.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// Call this API only when audio is mixing.
  Future<int?> setLoudness(double loudness);

  /// Sets the event handler.
  void setEventHandler(RTCMediaPlayerEventHandler? handler);
}
