// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import '../src/bytertc_room_event_impl.dart';
import 'bytertc_event_define.dart';
import 'bytertc_media_defines.dart';
import 'bytertc_room_api.dart';
import 'bytertc_rts_defines.dart';
import 'bytertc_video_api.dart';
import 'bytertc_video_defines.dart';

/// [roomId]: ID of the room where the status change occurred.
///
/// [uid]: ID of the user whose status has changed.
///
/// [state]: Room status code:
/// + 0: Success
/// + !0: Failure, see [ErrorCode] and [WarningCode] for details.
///
/// [extraInfo]: Extra information.
/// + `joinType`: `0` represents joining in the room for the first time, `1` represents a reconnection into the room.
/// + `elapsed`: The time interval (in ms) between the local user calling [RTCRoom.joinRoom] and successfully entering the room.
typedef OnRoomStateChangedType = void Function(
    String roomId, String uid, int state, String extraInfo);

/// [stats]: Statistics of this call.
typedef OnLeaveRoomType = void Function(RTCRoomStats stats);

/// [state]: A/V synchronization status.
typedef OnAVSyncStateChangeType = void Function(AVSyncState state);

/// [userInfo]: User information.
///
/// [elapsed]: The time in ms between the visible user calling [RTCRoom.joinRoom] to join the room and other users in the room receiving this event.
typedef OnUserJoinedType = void Function(UserInfo userInfo, int elapsed);

/// [uid]: ID of the remote user who left the room or switched to invisible.
///
/// [reason]: Reasons why remote users leave.
typedef OnUserLeaveType = void Function(String uid, UserOfflineReason reason);

/// [uid]: ID of the remote user who publishes the stream.
///
/// [type]: Media stream type.
typedef OnUserPublishStreamType = void Function(
    String uid, MediaStreamType type);

/// [uid]: ID of the user whose stream is removed.
///
/// [type]: Removed stream type.
///
/// [reason]: Reasons for removal of the remote stream.
typedef OnUserUnpublishStreamType = void Function(
    String uid, MediaStreamType type, StreamRemoveReason reason);

/// @nodoc('Useless')
typedef OnStreamSubscribedType = void Function(
    SubscribeState stateCode, String uid, SubscribeConfig info);

/// [msgid]: ID of this message.
///
/// [error]: Text or binary message sending results.
typedef OnRoomMessageSendResultType = void Function(
    int msgid, RoomMessageSendResult error);

/// [taskId]: The task ID of the API call.
///
/// [error]: Setting results and reasons.
typedef OnSetRoomExtraInfoResultType = void Function(
    int taskId, SetRoomExtraInfoResult error);

/// [key]: Key of the extra information.
///
/// [value]: Content of the extra information.
///
/// [lastUpdateUserId]: The ID of the last user who updated this information.
///
/// [lastUpdateTimeMs]: The Unix time in ms when this information was last updated.
typedef OnRoomExtraInfoUpdateType = void Function(
    String key, String value, String lastUpdateUserId, int lastUpdateTimeMs);

/// [currentUserVisibility]: Visibility of the current user.
/// + true: Visible. The user can publish media streams. The other users in the room get informed of the behaviors of the user, such as joining room, starting video capture, and leaving room.
/// + false: Invisible. The user cannot publish media streams. The other users in the room do not get informed of the behaviors of the user, such as joining room, starting video capture, or leaving room.
///
/// [errorCode]: Error code for setting user visibility.
typedef OnUserVisibilityChangedType = void Function(
    bool currentUserVisibility, UserVisibilityChangeError errorCode);

/// [uid]: ID of the user of whom the published video stream is banned/unbanned.
///
/// [banned]: Video stream sending status.
/// + true: Video stream is banned.
/// + false: Video stream is unbanned.
typedef OnVideoStreamBannedType = void Function(String uid, bool banned);

/// [uid]: ID of the user of whom the published audio stream is banned/unbanned.
///
/// [banned]: Audio stream sending status.
/// + true: Audio stream is banned.
/// + false: Audio stream is unbanned.
typedef OnAudioStreamBannedType = void Function(String uid, bool banned);

/// [stateInfos]: List of target room information.
typedef OnForwardStreamStateChangedType = void Function(
    List<ForwardStreamStateInfo> stateInfos);

/// [eventInfos]: List of target room events.
typedef OnForwardStreamEventType = void Function(
    List<ForwardStreamEventInfo> eventInfos);

/// [state]: The states of subtitles.
///
/// [errorCode]: Error codes of the subtitling task.
///
/// [errorMessage]: Detailed information on the third party services' errors.
typedef OnSubtitleStateChangedType = void Function(
    SubtitleState state, SubtitleErrorCode errorCode, String errorMessage);

/// [subtitles]: Subtitle messages.
typedef OnSubtitleMessageReceivedType = void Function(
    List<SubtitleMessage> subtitles);

/// Room event callbacks.
class RTCRoomEventHandler extends RTCRoomEventValue {
  /// Callback about room state changes.
  ///
  /// The user will receive this callback when they join the room, leave the room, or receive room relating warnings and errors.
  OnRoomStateChangedType? onRoomStateChanged;

  /// Callback about stream state changes.
  ///
  /// The user will receive this callback when they receive stream relating warnings and errors.
  OnRoomStateChangedType? onStreamStateChanged;

  /// Callback about successfully leaving the room.
  ///
  /// After the user calls [RTCRoom.leaveRoom], the SDK will stop publishing and subscribing to all streams and will notify the user of the success of leaving the room through this callback after releasing all audio and video resources associated with the call.
  ///
  /// Notes:
  /// + After leaving the room, if the App needs to use the system audio/video device, it is recommended to initialize the audio/video device after receiving this callback, otherwise the initialization may fail due to the SDK occupying the audio/video device.
  /// + If the [RTCVideo.destroy] is called immediately after the user leaves the room by calling the [RTCRoom.leaveRoom] method, this callback event will not be received if the RTC engine is destroyed.
  OnLeaveRoomType? onLeaveRoom;

  /// Stream publisher will receive this callback when the A/V synchronization state changes after [RTCRoom.setMultiDeviceAVSync] is called.
  OnAVSyncStateChangeType? onAVSyncStateChange;

  /// In-room call statistics callback.
  ///
  /// After the user enters the room and starts the call, they will receive this callback every 2s.
  @override
  abstract OnRoomStatsType? onRoomStats;

  /// Callback about the statistics of audio/video streams sent by the local device during the call and the network status, which is triggered every 2s.
  @override
  abstract OnLocalStreamStatsType? onLocalStreamStats;

  /// Callback about the statistics of the local device receiving the subscribed remote audio/video streams during the call and the network status, which is triggered every 2s.
  @override
  abstract OnRemoteStreamStatsType? onRemoteStreamStats;

  /// Callback about remote users joining the room.
  ///
  /// This callback is triggered in the following scenarios:
  /// + When the remote visible user calls [RTCRoom.joinRoom] to join the room, other users in the room will receive the callback.
  /// + when the remote visible user reconnects to the room after disconnecting from the network, other users in the room will receive the callback.
  /// + When the remote invisible user calls [RTCRoom.setUserVisibility] to switch themselves to visible, other users in the room will receive the callback.
  /// + New comers will receive this callback of exiting visible users entering the room.
  OnUserJoinedType? onUserJoined;

  /// Callback about remote users leaving the room.
  ///
  /// Other users in the room will receive the callback in the following scenarios:
  /// + When the remote visible user calls [RTCRoom.leaveRoom] to leave the room.
  /// + When the remote visible user calls [RTCRoom.setUserVisibility] to switch themselves to invisible
  /// + When the remote user is disconnected and never restored.
  OnUserLeaveType? onUserLeave;

  /// The callback will be triggered 30 seconds before the Token expires.
  ///
  /// In this case, you should call [RTCRoom.updateToken] to update the Token, otherwise the user would be removed from the room and not be able to continue the call.
  EmptyCallbackType? onTokenWillExpire;

  /// Callback triggered 30s before the publishing privilege of the Token expires. <br>
  /// After receiving this callback, you must call [RTCRoom.updateToken] to update the publishing privilege Token.
  ///
  /// After a user's publishing privilege expires:
  /// + When attempting to publish a stream, the user will receive [onStreamStateChanged] with the error code "-noPublishPermission" indicating no permission to publish streams.
  /// + The published streams of the user will be removed, and he/she will receive [onStreamStateChanged] with the error code "-noPublishPermission" indicating no permission to publish streams. Remote users in the room will receive [onUserUnpublishStream]/[onUserUnpublishScreen] with the reason "publishPrivilegeExpired" indicating that the publishing privilege of the remote user has expired.
  EmptyCallbackType? onPublishPrivilegeTokenWillExpire;

  /// Callback triggered 30s before the subscribing privilege of the Token expires.
  ///
  /// After receiving this callback, you must call [RTCRoom.updateToken] to update the subscribing privilege Token.
  /// After a user's subscribing privilege expires, the user will fail to subscribe to new streams, or the subscribed streams will be removed, and he/she will receive [onStreamStateChanged] with error code "-1003" indicating no permission to subscribe to streams.
  EmptyCallbackType? onSubscribePrivilegeTokenWillExpire;

  /// Callback about new media streams captured by camera/microphone in the room.
  ///
  /// The local user will receive this callback after a remote user successfully published media streams captured by camera/microphone in the room with [RTCRoom.publishStream].<br>
  /// Then the local user can choose whether or not to call [RTCRoom.subscribeStream] to subscribe to this stream.
  OnUserPublishStreamType? onUserPublishStream;

  /// Callback about the removal of remote media stream captured by camera/microphone.
  ///
  /// After receiving this callback, the local user can choose whether to call [RTCRoom.unsubscribeStream] to unsubscribe from the streams or not.
  OnUserUnpublishStreamType? onUserUnpublishStream;

  /// Callback about new screen sharing media streams from remote users in the room.
  ///
  /// The local user will receive this callback after a remote user successfully publishes screen sharing streams in the room with [RTCRoom.publishScreen].
  /// Then the local user can choose whether to call [RTCRoom.subscribeScreen] to subscribe to the streams or not.
  OnUserPublishStreamType? onUserPublishScreen;

  /// Callback about the removal of screen sharing media streams from remote users in the room.
  ///
  /// After receiving this callback, the local can choose whether to call [RTCRoom.unsubscribeScreen] to unsubscribe from the streams or not.
  OnUserUnpublishStreamType? onUserUnpublishScreen;

  /// @nodoc('Useless')
  OnStreamSubscribedType? onStreamSubscribed;

  /// This callback is triggered after the local user receives a in-room broadcast text message.
  OnMessageReceivedType? onRoomMessageReceived;

  /// This callback is triggered after the local user receives a in-room broadcast binary message.
  OnBinaryMessageReceivedType? onRoomBinaryMessageReceived;

  /// This callback is triggered after the local user receives a text message from the other users in the room.
  OnMessageReceivedType? onUserMessageReceived;

  /// This callback is triggered after the local user receives a binary message from the other users in the room.
  OnBinaryMessageReceivedType? onUserBinaryMessageReceived;

  /// This callback is triggered after the local user sends a message to another user in the room.
  OnMessageSendResultType? onUserMessageSendResult;

  /// This callback is triggered after the local user sends a message to all the other users in the room.
  OnRoomMessageSendResultType? onRoomMessageSendResult;

  /// Valid since v3.56.1
  ///
  /// Callback on the result of calling [RTCRoom.setRoomExtraInfo] to set extra information about the room.
  OnSetRoomExtraInfoResultType? onSetRoomExtraInfoResult;

  /// Valid since v3.56.1
  ///
  /// Callback used to receive the extra information set by the other users in the same room with [RTCRoom.setRoomExtraInfo].
  ///
  /// Users who join the room later will be notified of all extra information in the room set prior to entering.
  OnRoomExtraInfoUpdateType? onRoomExtraInfoUpdate;

  /// Valid since v3.56.1
  ///
  /// Callback for user to set user visibility by calling [RTCRoom.setUserVisibility].
  OnUserVisibilityChangedType? onUserVisibilityChanged;

  /// This callback is triggered by calling the OpenAPI BanUserStream/UnbanUserStream to disable/enable the specified user publishing video streams in the specified room.
  ///
  /// Notes:
  /// + When the specified user in the room is banned/unbanned from publishing video streams, all users in the room will receive the callback.
  /// + The banned/unbanned status will not change until the room is destroyed.
  /// + Only users being banned/unbanned will receive this callback after the conference mode is turned on in the console.
  OnVideoStreamBannedType? onVideoStreamBanned;

  /// This callback is triggered by calling the OpenAPI BanUserStream/UnbanUserStream to disable/enable the specified user publishing audio streams in the specified room.
  ///
  /// Notes:
  /// + When the specified user in the room is banned/unbanned from publishing audio streams, all users in the room will receive the callback.
  /// + The banned/unbanned status will not change until the room is destroyed.
  /// + Only users being banned/unbanned will receive this callback after the conference mode is turned on in the console.
  OnAudioStreamBannedType? onAudioStreamBanned;

  /// Callback about the state and errors during relaying media streams across rooms.
  OnForwardStreamStateChangedType? onForwardStreamStateChanged;

  /// Callback about events during relaying media streams across rooms.
  OnForwardStreamEventType? onForwardStreamEvent;

  /// After the user joins the room, this callback will report the user's network quality information every 2 seconds.
  ///
  /// See [In-call Stats](https://docs.byteplus.com/byteplus-rtc/docs/106866) for more details.
  @override
  abstract OnNetworkQualityType? onNetworkQuality;

  /// @nodoc('Not available')
  /// Callback on subtitle states. <br>
  /// After calling [RTCRoom.startSubtitle] or [RTCRoom.stopSubtitle], you will receive this callback which informs you of the states and error codes of the subtitling task, as well as detailed information on the third party services' errors.
  OnSubtitleStateChangedType? onSubtitleStateChanged;

  /// @nodoc('Not available')
  /// Callback on subtitle messages. <br>
  /// After calling [RTCRoom.startSubtitle], you will receive the subtitles and other relevant information through this callback.
  OnSubtitleMessageReceivedType? onSubtitleMessageReceived;

  /// @nodoc
  RTCRoomEventHandler({
    this.onRoomStateChanged,
    this.onStreamStateChanged,
    this.onLeaveRoom,
    this.onAVSyncStateChange,
    OnRoomStatsType? onRoomStats,
    OnLocalStreamStatsType? onLocalStreamStats,
    OnRemoteStreamStatsType? onRemoteStreamStats,
    this.onUserJoined,
    this.onUserLeave,
    this.onTokenWillExpire,
    this.onPublishPrivilegeTokenWillExpire,
    this.onSubscribePrivilegeTokenWillExpire,
    this.onUserPublishStream,
    this.onUserUnpublishStream,
    this.onUserPublishScreen,
    this.onUserUnpublishScreen,
    this.onStreamSubscribed,
    this.onRoomMessageReceived,
    this.onRoomBinaryMessageReceived,
    this.onUserMessageReceived,
    this.onUserBinaryMessageReceived,
    this.onUserMessageSendResult,
    this.onRoomMessageSendResult,
    this.onSetRoomExtraInfoResult,
    this.onRoomExtraInfoUpdate,
    this.onUserVisibilityChanged,
    this.onVideoStreamBanned,
    this.onAudioStreamBanned,
    this.onForwardStreamStateChanged,
    this.onForwardStreamEvent,
    OnNetworkQualityType? onNetworkQuality,
    this.onSubtitleStateChanged,
    this.onSubtitleMessageReceived,
  }) : super(
          onRoomStats: onRoomStats,
          onLocalStreamStats: onLocalStreamStats,
          onRemoteStreamStats: onRemoteStreamStats,
          onNetworkQuality: onNetworkQuality,
        );
}
