// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import '../src/base/bytertc_enum_convert.dart';
import 'bytertc_room_api.dart';
import 'bytertc_video_api.dart';
import 'bytertc_video_event_handler.dart';

/// The reason why the remote user leaves the room.
enum UserOfflineReason {
  /// The remote user calls [RTCRoom.leaveRoom].
  quit,

  /// The remote user is disconnected.
  dropped,

  /// The remote user switches to invisible.
  switchToInvisible,

  /// The remote user is kicked out of the room due to an OpenAPI call.
  kickedByAdmin,
}

/// Status of the SDK connection to the signalling server
enum RTCConnectionState {
  /// Disconnected for 12s, SDK will try to reconnect automatically.
  disconnected,

  /// The first request to connect to the server. Connecting.
  connecting,

  /// The first connection was successful.
  connected,

  /// Triggered in the following scenarios:
  /// + Failed to connect within 10 seconds on first connection.
  /// + Disconnected for 10 seconds after a successful connection. Automatic reconnection in progress.
  reconnecting,

  /// Successful reconnection.
  reconnected,

  /// In disconnected status for more than 10s without a successful reconnection. SDK will still continue to try to reconnect.
  lost,

  /// Connection failure caused by abnormal state of the server. SDK will not automatically reconnect, please re-enter the room or contact technical support.
  failed,
}

/// Network connection type.
enum NetworkType {
  /// Unknown type
  unknown,

  /// The network is disconnected.
  disconnected,

  /// The network connection type is LAN.
  lan,

  /// The network connection type is Wi-Fi (including hotspots).
  wifi,

  /// 2G mobile network.
  mobile2G,

  /// 3G mobile network.
  mobile3G,

  /// 4G mobile network.
  mobile4G,

  /// 5G mobile network.
  mobile5G,
}

/// Media streaming network quality
enum NetworkQuality {
  /// Unknown quality
  unKnown,

  /// Excellent
  excellent,

  /// Good
  good,

  /// The network quality is poor, but does not affect communication.
  poor,

  /// The network quality is bad, users can barely communicate but not smoothly.
  bad,

  /// The quality of the network is very bad and communication is basically impossible.
  veryBad,

  /// The network is down. It may be down due to no answer within 12s, airplane mode on, disconnected cable, etc.
  ///
  /// Refer to [Get connection state](https://docs.byteplus.com/byteplus-rtc/docs/95376) for more details about getting connection state.
  down,
}

/// Login result
///
/// The result of calling [RTCVideo.login] will be notified to the user through [RTCVideoEventHandler.onLoginResult].
enum LoginErrorCode {
  /// Login successful
  success,

  /// The token is invalid or expired. The user is required to re-acquire the token.
  invalidToken,

  /// An unknown error occurred when calling [RTCVideo.login], which caused the login to fail. The user is required to log in again.
  loginFailed,

  /// Invalid user ID passed in through [RTCVideo.login].
  invalidUserId,

  /// Server error during login
  codeServerError,
}

/// Reliability and orderliness of the message
enum MessageConfig {
  /// Reliable and ordered low-latency message
  reliableOrdered,

  /// Ordered ultra-low latency message
  unreliableOrdered,

  /// Unordered ultra-low latency message
  unreliableUnordered,
}

/// Message delivery result
enum UserMessageSendResult {
  /// The message has been sent successfully.
  success,

  /// Failed, message delivery timeout.
  timeout,

  /// Failed. Channel disconnected.
  broken,

  /// Failed. Recipient not found.
  noReceiver,

  /// Failed to get cascade path.
  noRelayPath,

  /// Failure. Exceeds QPS limit.
  exceedQPS,

  /// Failure. The app server failed to receive the message sent from the client.
  ///
  /// Triggered by calling [RTCVideo.sendServerMessage] or [RTCVideo.sendServerBinaryMessage], and carried by [RTCVideoEventHandler.onServerMessageSendResult] callback.
  e2bsSendFailed,

  /// Failure. The app server received the message sent by the client, but failed to response.
  ///
  /// Triggered by calling [RTCVideo.sendServerMessage] or [RTCVideo.sendServerBinaryMessage], and carried by [RTCVideoEventHandler.onServerMessageSendResult] callback.
  e2bsReturnFailed,

  /// The message sender is not in the room.
  notJoin,

  /// Connection is not initialized.
  init,

  /// No data transmission channel connection available.
  noConnection,

  /// Message exceeds maximum length (64KB).
  exceedMaxLength,

  /// The ID of the recipient is empty.
  emptyUser,

  /// The message sender did not log in.
  notLogin,

  /// No parameters were set before sending the message to the server.
  serverParamsNotSet,

  /// Undefined error.
  unknown,
}

/// User online status
enum UserOnlineStatus {
  /// The peer user is offline, either because [RTCVideo.login] has not been called or [RTCVideo.logout] has been called.
  offline,

  /// The peer user is online
  online,

  /// Unable to get the peer user's online status.
  ///
  /// Returned when a cascade error occurs and the online state of the peer user is abnormal.
  unreachable,
}

/// Result of broadcasting messages in the room
enum RoomMessageSendResult {
  /// The message has been sent successfully.
  success,

  /// Failure. Sending timeout.
  timeout,

  /// Failure. Channel disconnected.
  networkDisconnected,

  /// Failure. Exceeds QPS limit.
  exceedQPS,

  /// Failed. The message sender is not in the room.
  notJoin,

  /// Failed. Connection is not initialized.
  init,

  /// Failed. No data transmission channel connection available.
  noConnection,

  /// Failed. Message exceeds maximum length (64KB).
  exceedMaxLength,

  /// Failed. Undefined error.
  unknown,
}

/// Information on CPU and memory used by App.
class SysStats {
  /// Number of CPU cores on current system.
  final int? cpuCores;

  /// The CPU usage of the current application, the value range is [0,1].
  final double? cpuAppUsage;

  /// The current system CPU usage, the value range is [0,1].
  final double? cpuTotalUsage;

  /// Memory usage of the current app in MB.
  final double? memoryUsage;

  /// Memory size of the device in MB.
  final int? fullMemory;

  /// System memory used in MB.
  final int? totalMemoryUsage;

  /// Current free system memory in MB
  final int? freeMemory;

  /// Current application memory usage (%)
  final double? memoryRatio;

  /// System memory usage (%)
  final double? totalMemoryRatio;

  /// @nodoc
  const SysStats({
    this.cpuCores,
    this.cpuAppUsage,
    this.cpuTotalUsage,
    this.memoryUsage,
    this.fullMemory,
    this.totalMemoryUsage,
    this.freeMemory,
    this.memoryRatio,
    this.totalMemoryRatio,
  });

  /// @nodoc
  factory SysStats.fromMap(Map<dynamic, dynamic> map) {
    return SysStats(
      cpuCores: map['cpuCores'],
      cpuAppUsage: map['cpuAppUsage'],
      cpuTotalUsage: map['cpuTotalUsage'],
      memoryUsage: map['memoryUsage'],
      fullMemory: map['fullMemory'],
      totalMemoryUsage: map['totalMemoryUsage'],
      freeMemory: map['freeMemory'],
      memoryRatio: map['memoryRatio'],
      totalMemoryRatio: map['totalMemoryRatio'],
    );
  }
}

/// Cloud proxy information
class CloudProxyInfo {
  /// Server IP
  String cloudProxyIp;

  /// Server port
  int cloudProxyPort;

  /// @nodoc
  CloudProxyInfo({
    required this.cloudProxyIp,
    required this.cloudProxyPort,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'cloudProxyIp': cloudProxyIp,
      'cloudProxyPort': cloudProxyPort,
    };
  }
}

/// Types of local proxies.
enum LocalProxyType {
  /// Socks5 proxy. If you chose Socks5 as the local proxy, you need to make sure all requirements listed in the Socks5 document are satisfied.
  socks5,

  /// Http tunnel proxy.
  httpTunnel,
}

/// Detailed information of local proxy configurations.
class LocalProxyConfiguration {
  /// Types of local proxies.
  LocalProxyType localProxyType;

  /// Local proxy IP.
  String localProxyIp;

  /// Local proxy port.
  int localProxyPort;

  /// The username of the local proxy.
  String localProxyUsername;

  /// The password of the local proxy.
  String localProxyPassword;

  /// @nodoc
  LocalProxyConfiguration({
    this.localProxyType = LocalProxyType.socks5,
    required this.localProxyIp,
    required this.localProxyPort,
    this.localProxyUsername = '',
    this.localProxyPassword = '',
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'localProxyType': localProxyType.value,
      'localProxyIp': localProxyIp,
      'localProxyPort': localProxyPort,
      'localProxyUsername': localProxyUsername,
      'localProxyPassword': localProxyPassword,
    };
  }
}

/// The states of local proxy connection.
enum LocalProxyState {
  /// proxy server is connected.
  inited,

  /// The local proxy is connected.
  connected,

  /// Errors occurred when connecting to the local proxy.
  error,
}

/// The errors of local proxy connection.
enum LocalProxyError {
  /// There are no errors in local proxies.
  ok,

  /// The connection to Socks5 proxy failed because the proxy server replies wrong version numbers which don't accord with the Socks5 document. Please check the proxy server.
  socks5VersionError,

  /// The connection to Socks5 proxy failed because the format of the proxy's replies doesn't accord with the Socks5 document. Please check the proxy server.
  socks5FormatError,

  /// The connection to Socks5 proxy failed because the proxy replies wrong information which doesn't accord with the Socks5 document. Please check the proxy server.
  socks5InvalidValue,

  /// The connection to Socks5 proxy failed because the username and password of the local proxy are not provided. Please call [RTCVideo.setLocalProxy] and enter your username and password.
  socks5UserPassNotGiven,

  /// The connection to Socks5 proxy failed because TCP is closed. Please check the proxy server and your network connection status.
  socks5TcpClosed,

  /// Errors in Http tunnel proxy. Please check Http tunnel proxy and your network connection status.
  httpTunnelFailed,
}

/// Local log parameters.
enum LocalLogLevel {
  /// Info.
  info,

  /// Warning (default).
  warning,

  /// Error.
  error,

  /// Turn off logging.
  none,
}

/// Local log parameters.
class RTCLogConfig {
  /// Local log directory.
  final String logPath;

  /// The logging level. The default is "Warning".
  final LocalLogLevel logLevel;

  /// The maximum cache space available for logs in MB. the range is 1 to 100 MB, and the default value is 10 MB.
  final int logFileSize;

  /// (Optional) Local log file name prefix. It should follow the regular expression pattern of `[a-zA-Z0-9_@-.]{1,128}`.
  ///
  /// The final file name will be the prefix followed by "_" and the file creation time, and "_rtclog.log" at the end. For example, `logPrefix_2023-05-25_172324_rtclog.log`.
  final String logFilenamePrefix;

  /// @nodoc
  const RTCLogConfig({
    required this.logPath,
    this.logLevel = LocalLogLevel.warning,
    this.logFileSize = 10,
    this.logFilenamePrefix = '',
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'logPath': logPath,
      'logLevel': logLevel.index,
      'logFileSize': logFileSize,
      'logFilenamePrefix': logFilenamePrefix,
    };
  }
}

/// The reason why the user log out.
enum LogoutReason {
  /// The user choose to logout by calling [RTCVideo.logout] to log out or terminating the engine to log out.
  logout,

  /// The current user is kicked out as another user logs in with the same UserId.
  duplicateLogin,
}
