// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import '../src/base/bytertc_enum_convert.dart';
import 'bytertc_audio_defines.dart';
import 'bytertc_media_player_api.dart';

/// Karaoke scoring mode.
enum MulDimSingScoringMode {
  /// The score is provided based on the pitch.
  note,
}

/// Real-time scoring data.
class SingScoringRealtimeInfo {
  /// Current playback position.
  final int currentPosition;

  /// The singer's pitch.
  final int userPitch;

  /// Standard pitch.
  final int standardPitch;

  /// Lyric index.
  final int sentenceIndex;

  /// The score for the previous lyric.
  final int sentenceScore;

  /// The total score for the singer's current performance.
  final int totalScore;

  /// The average score for the user's current performance.
  final int averageScore;

  /// @nodoc
  const SingScoringRealtimeInfo({
    required this.currentPosition,
    required this.userPitch,
    required this.standardPitch,
    required this.sentenceIndex,
    required this.sentenceScore,
    required this.totalScore,
    required this.averageScore,
  });

  /// @nodoc
  factory SingScoringRealtimeInfo.fromMap(Map<dynamic, dynamic> map) {
    return SingScoringRealtimeInfo(
      currentPosition: map['currentPosition'],
      userPitch: map['userPitch'],
      standardPitch: map['standardPitch'],
      sentenceIndex: map['sentenceIndex'],
      sentenceScore: map['sentenceScore'],
      totalScore: map['totalScore'],
      averageScore: map['averageScore'],
    );
  }
}

/// Standard Pitch Data.
class StandardPitchInfo {
  /// Starting time in ms.
  final int startTime;

  /// Duration in ms.
  final int duration;

  /// Pitch.
  final int pitch;

  /// @nodoc
  const StandardPitchInfo({
    required this.startTime,
    required this.duration,
    required this.pitch,
  });

  /// @nodoc
  factory StandardPitchInfo.fromMap(Map<dynamic, dynamic> map) {
    return StandardPitchInfo(
      startTime: map['startTime'],
      duration: map['duration'],
      pitch: map['pitch'],
    );
  }
}

/// Karaoke scoring configuration.
class SingScoringConfig {
  /// Sample rate.
  ///
  /// Only 44,100 Hz and 48,000 Hz are supported.
  AudioSampleRate sampleRate;

  /// Scoring mode.
  MulDimSingScoringMode mode;

  /// The file path of the lyrics.
  ///
  /// The scoring feature only supports KRC lyrics file.
  String lyricsFilepath;

  /// The path of the midi file.
  String midiFilepath;

  /// @nodoc
  SingScoringConfig({
    required this.sampleRate,
    this.mode = MulDimSingScoringMode.note,
    required this.lyricsFilepath,
    required this.midiFilepath,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'sampleRate': sampleRate.value,
      'mode': mode.index,
      'lyricsFilepath': lyricsFilepath,
      'midiFilepath': midiFilepath,
    };
  }
}

/// [info]: Real-time scoring data.
typedef OnCurrentScoringInfoType = void Function(SingScoringRealtimeInfo? info);

/// Karaoke scoring event handler.
class RTCSingScoringEventHandler {
  /// The callback for real-time scoring data.
  ///
  /// This callback is triggered after [RTCSingScoringManager.startSingScoring] is called.
  OnCurrentScoringInfoType? onCurrentScoringInfo;

  /// @nodoc
  RTCSingScoringEventHandler({
    this.onCurrentScoringInfo,
  });
}

/// Karaoke scoring management interface.
abstract class RTCSingScoringManager {
  /// Initializes karaoke scoring feature.
  ///
  /// [singScoringAppKey]: The key for karaoke scoring, used to authenticate whether the karaoke scoring is enabled.<br>
  /// [singScoringToken]: The key for karaoke scoring, used to authenticate whether the karaoke scoring is enabled.
  ///
  /// Authentication is done offline, bind the AppKey and Token according to the package name (bundleID). Please contact technical support to apply for the key.
  ///
  /// [handler]: Karaoke scoring event handler.
  ///
  /// Return value:
  /// + 0: Success.
  /// + -1: Failure.
  /// + -2: Karaoke scoring module not integrated.
  Future<int?> initSingScoring({
    required String singScoringAppKey,
    required String singScoringToken,
    RTCSingScoringEventHandler? handler,
  });

  /// Sets the configuration of karaoke scoring.
  ///
  /// [config]: The parameters of karaoke scoring.
  ///
  /// Return value:
  /// + 0: Success.
  /// + -1: Failure.
  /// + -2: Karaoke scoring module not integrated.
  Future<int?> setSingScoringConfig(SingScoringConfig config);

  /// Gets standard pitch.
  ///
  /// [midiFilepath]: Midi file path of the song.
  ///
  /// Return value: Standard pitch data array.
  ///
  /// Please make sure the same midi file path is passed in this API and [RTCSingScoringManager.setSingScoringConfig].
  Future<List<StandardPitchInfo>?> getStandardPitchInfo(String midiFilepath);

  /// Starts karaoke scoring.
  ///
  /// [position]: You can get the playback position where you start karaoke scoring. Unit: ms.
  ///
  /// [scoringInfoInterval]: Time interval between two real-time callbacks. Unit: ms; Default interval: 50 ms. Minimum interval: 20 ms.
  ///
  /// Return value:
  /// + 0: Success.
  /// + -1: Failure.
  /// + -2: Karaoke scoring module not integrated.
  ///
  /// Notes
  /// + You can call this API after calling [RTCSingScoringManager.initSingScoring] to initialize karaoke scoring.
  /// + After this interface is called, you will receive the scoring result [RTCSingScoringEventHandler.onCurrentScoringInfo] at set interval.
  /// + If you call the [RTCMediaPlayer.start] to play an audio file, call this interface after you receive [RTCMediaPlayerEventHandler.onMediaPlayerStateChanged] (playing).
  Future<int?> startSingScoring({
    int position,
    int scoringInfoInterval,
  });

  /// Stops karaoke scoring.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  Future<int?> stopSingScoring();

  /// Gets the score for the previous lyric.
  ///
  /// You can call this API after [RTCSingScoringManager.startSingScoring] is called.
  ///
  /// Return value:
  /// + `<0`: Failed to get the score for the previous lyric.
  /// + `>=0`: The score for the previous lyric.
  Future<int?> getLastSentenceScore();

  /// Gets the total score for the user's current performance.
  ///
  /// You can call this API after [RTCSingScoringManager.startSingScoring] is called
  ///
  /// Return value:
  /// + `<0`: Failed to get the total score.
  /// + `>=0`: The current total score.
  Future<int?> getTotalScore();

  /// Gets the average score for the user's current performance.
  ///
  /// Return value:
  /// + `<0`: Failed to get the average score.
  /// + `>=0`: The average score.
  Future<int?> getAverageScore();
}
