// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'dart:async';
import 'dart:collection';
import 'dart:typed_data';
import 'dart:ui';

import 'package:async/async.dart';

import '../src/bytertc_video_impl.dart';
import 'bytertc_asr_engine.dart';
import 'bytertc_audio_defines.dart';
import 'bytertc_audio_effect_player_api.dart';
import 'bytertc_audio_mixing_api.dart';
import 'bytertc_cdn_stream.dart';
import 'bytertc_face_detection.dart';
import 'bytertc_ktv_manager_api.dart';
import 'bytertc_live_transcoding.dart';
import 'bytertc_media_defines.dart';
import 'bytertc_media_player_api.dart';
import 'bytertc_room_api.dart';
import 'bytertc_room_event_handler.dart';
import 'bytertc_rts_defines.dart';
import 'bytertc_sing_scoring_manager_api.dart';
import 'bytertc_video_defines.dart';
import 'bytertc_video_effect_api.dart';
import 'bytertc_video_event_handler.dart';

/// Used to initialize the RTCVideo.
class RTCVideoContext {
  /// A unique identifier for each App, randomly generated by the RTC console.
  ///
  /// Engine instances created with different AppIds cannot communicate with each other.
  String appId;

  /// Delegate sent from SDK to App.
  RTCVideoEventHandler? eventHandler;

  /// Reserved parameters. Please contact technical support fellow if needed.
  Map<String, dynamic>? parameters;

  /// @nodoc
  RTCVideoContext(
    this.appId, {
    this.eventHandler,
    this.parameters,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    HashMap<String, dynamic> dic = HashMap();
    dic['appId'] = appId;
    if (parameters != null) {
      dic['parameters'] = parameters;
    }
    return dic;
  }
}

/// The engine instance.
abstract class RTCVideo {
  /// Creates an engine instance.
  ///
  /// This is the very first API that you must call if you want to use all the RTC capabilities. <br>
  /// If there is no engine instance in current process, calling this API will create one. If an engine instance has been created, calling this API again will have the created engine instance returned.
  static Future<RTCVideo?> createRTCVideo(RTCVideoContext context) =>
      RTCVideoImpl.createRTCVideo(context);

  /// Gets the current SDK version.
  static Future<String?> getSDKVersion() => RTCVideoImpl.getSDKVersion();

  /// Valid since v3.56.1
  ///
  /// Configures the local log parameters of RTC SDK, including the logging level, directory, and the maximum cache space that can be used.
  ///
  /// [logConfig]: Local log parameters.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1: Failure. This API must be called before creating engine.
  /// + –2: Failure. Invalid parameters.
  ///
  /// This API must be called before [RTCVideo.createRTCVideo].
  static Future<int?> setLogConfig(RTCLogConfig logConfig) =>
      RTCVideoImpl.setLogConfig(logConfig);

  /// Gets the description of error codes and warning codes in the SDK.
  static Future<String?> getErrorDescription(int code) =>
      RTCVideoImpl.getErrorDescription(code);

  /// Destroys the engine instance created by [RTCVideo.createRTCVideo] and release all related resources.
  ///
  /// Notes:
  /// + Call this API after all business scenarios related to the RTCVideo instance that need to be destroyed have been completed.
  /// + Calling this API will destroy all memory associated with this RTCVideo instance and stops any interaction with the media server.
  Future<void> destroy();

  /// Sets the event callback class.
  void setRTCVideoEventHandler(RTCVideoEventHandler handler);

  /// Starts internal audio capturing. Off by default.
  ///
  /// Internal capture refers to audio capture using the built-in capture module of the SDK.<br>
  /// After this API is called, the local user will receive [RTCVideoEventHandler.onAudioDeviceStateChanged].<br>
  /// If this API is called by a visible user in the current room, the other users in the room will receive [RTCVideoEventHandler.onUserStartAudioCapture].
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Calling this API without obtaining permission to use the microphone of the current device will trigger [RTCVideoEventHandler.onWarning].
  /// + Call [RTCVideo.stopAudioCapture] to stop the internal audio capturing, otherwise it will not stop until the engine is destroyed.
  /// + Since different hardware devices have different initialization response times, frequent calls to this method and [RTCVideo.stopAudioCapture] may cause short silence during the call. Therefore to temporarily mute or unmute microphone, we recommend you use [RTCRoom.publishStream]/[RTCRoom.unpublishStream].
  /// + You can call this API no matter you need to publish audio or not, but you can only publish audio after calling it.
  Future<int?> startAudioCapture();

  /// Stops internal audio capturing.
  ///
  /// After this API is called, the local user will receive [RTCVideoEventHandler.onAudioDeviceStateChanged].<br>
  /// If this API is called by a visible user in the current room, the other users in the room will receive [RTCVideoEventHandler.onUserStopAudioCapture].
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Call [RTCVideo.startAudioCapture] to start internal capturing.
  /// + If you don't call this API after the internal capture is started, it will not stop until the engine is destroyed.
  Future<int?> stopAudioCapture();

  /// Sets the audio scenarios.
  ///
  /// You should choose the right type of audio scenario according to your need.<br>
  /// When an audio scenario is selected, the RTC automatically applies call volume/media volume based on the client audio routing and publishing subscription status.<br>
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + You can call this API anytime before or after entering the room.
  /// + Call volume is more suitable for calls, meetings and other scenarios that demand information accuracy. Call volume will activate the system hardware signal processor, making the sound clearer. The volume cannot be reduced to 0.
  /// + Media volume is more suitable for entertainment scenarios, which require musical expression. The volume can be reduced to 0.
  Future<int?> setAudioScenario(AudioScenario audioScenario);

  /// Sets the sound quality.
  ///
  /// You can call this API when the audio parameters in the selected [RoomProfile] do not meet the needs of your scene.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes: You can call this API anytime before or after entering the room.
  Future<int?> setAudioProfile(AudioProfileType audioProfile);

  /// Sets the Active Noise Cancellation(ANC) mode during audio and video communications.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// You can call this API before or after entering a room. When you repeatedly call it, only the last call takes effect.
  Future<int?> setAnsMode(AnsMode ansMode);

  /// Sets the type of voice change effect.
  ///
  /// See [ReturnStatus] for return values.
  ///
  /// + You can call this API anytime before or after entering the room.
  /// + This method only works for mono audio.
  /// + The version of the RTC SDK you are using must provide sound effect function
  /// + This method is mutually exclusive with [RTCVideo.setVoiceReverbType], the effects set later will override the effects set first.
  Future<int?> setVoiceChangerType(VoiceChangerType voiceChanger);

  /// Sets the type of reverb effect.
  ///
  /// See [ReturnStatus] for return values.
  ///
  /// + You can call this API anytime before or after entering the room.
  /// + This method only works for mono audio.
  /// + The version of the RTC SDK you are using must provide sound effect function
  /// + This method is mutually exclusive with [RTCVideo.setVoiceChangerType], the effects set later will override the effects set first.<br>
  /// To use this feature, contact us for technical support.
  Future<int?> setVoiceReverbType(VoiceReverbType voiceReverb);

  /// Set the equalization effect for the local captured audio.<br>
  /// The audio includes both internal captured audio and external captured voice, but not the mixing audio file.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// According to the Nyquist sampling rate, the audio acquisition rate must be greater than twice the set center frequency. Otherwise, the setting will not be effective.
  Future<int?> setLocalVoiceEqualization(VoiceEqualizationConfig config);

  /// Set the reverb effect for the local captured audio.<br>
  /// The audio includes both internal captured audio and external captured voice, but not the mixing audio file.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// Call [RTCVideo.enableLocalVoiceReverb] to enable the reverb effect.
  Future<int?> setLocalVoiceReverbParam(VoiceReverbConfig config);

  /// Enable the reverb effect for the local captured voice.
  ///
  /// Call [RTCVideo.setLocalVoiceReverbParam] to set the reverb effect.
  Future<int?> enableLocalVoiceReverb(bool enable);

  /// Adjusts the audio capture volume.
  ///
  /// [volume]: Ratio(%) of capture volume to original volume with a range of `[0, 400]`.<br>
  /// This changes the volume property of the audio data other than the hardware volume.<br>
  /// For better call quality, it is recommended to set the volume value to `[0,100]`.
  /// + 0: Mute.
  /// + 100: Original volume.
  /// + 400: Four times the original volume with signal-clipping protection.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + You can call this method to adjust both the audio stream captured by the microphone, and the audio stream from the screen sharing.
  /// + You can call this method both before and after starting audio capturing.
  Future<int?> setCaptureVolume({
    StreamIndex index = StreamIndex.main,
    required int volume,
  });

  /// Adjusts the playback volume of the mixed remote audio.
  ///
  /// [volume]: Ratio(%) of capture volume to original volume with a range of `[0, 400]`.<br>
  /// This changes the volume property of the audio data other than the hardware volume.<br>
  /// For better call quality, it is recommended to set the volume value to `[0,100]`.
  /// + 0: Mute.
  /// + 100: Original volume.
  /// + 400: Four times the original volume with signal-clipping protection.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// You can call this API both before and during playing the audio.
  /// Suppose a remote user A is always within the range of the target user whose playback volume will be adjusted, if you use both this method and [RTCVideo.setRemoteAudioPlaybackVolume]/[RTCRoom.setRemoteRoomAudioPlaybackVolume], the volume that the local user hears from user A is the overlay of both settings.
  Future<int?> setPlaybackVolume(int volume);

  /// Enable audio information prompts.
  ///
  /// After that, you will receive [RTCVideoEventHandler.onLocalAudioPropertiesReport], [RTCVideoEventHandler.onRemoteAudioPropertiesReport], and [RTCVideoEventHandler.onActiveSpeaker].
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> enableAudioPropertiesReport(AudioPropertiesConfig config);

  /// Adjusts the audio playback volume from the remote user.
  ///
  /// [roomId]: ID of the room where the specified remote user is in.
  ///
  /// [uid]: ID of the user who publishes the audio.
  ///
  /// [volume]: Ratio(%) of capture volume to original volume with a range of `[0, 400]`.<br>
  /// This changes the volume property of the audio data other than the hardware volume.<br>
  /// For better call quality, it is recommended to set the volume value to `[0,100]`.
  /// + 0: Mute.
  /// + 100: Original volume.
  /// + 400: Four times the original volume with signal-clipping protection.
  ///
  /// See [ReturnStatus] for return values.
  ///
  /// Suppose a remote user A is always within the range of the target user whose playback volume will be adjusted:
  /// + If you use both this method and [RTCRoom.setRemoteRoomAudioPlaybackVolume], the volume that the local user hears from user A is the volume set by the method called later.
  /// + If you use both this method and [RTCVideo.setPlaybackVolume], the volume that the local user hears from user A is the overlay of both settings.
  Future<int?> setRemoteAudioPlaybackVolume({
    required String roomId,
    required String uid,
    required int volume,
  });

  /// Turns on/off the earphone monitor function.
  ///
  /// [mode]: Whether to enable in-ear monitoring.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + In-ear monitoring is effective for audios captured by the RTC SDK.
  /// + We recommend that you use wired earbuds/headphones for a low-latency experience.
  /// + The RTC SDK supports both the hardware-level in-ear monitoring and the SDK-level in-ear monitoring. Most hardware-level in-ear monitors enjoy lower latency and better audio quality. If your App is in the special list of the smartphone manufacturer， RTC SDK uses hardware-level ear monitoring by default.
  Future<int?> setEarMonitorMode(EarMonitorMode mode);

  /// Sets the volume of in-ear monitoring.
  ///
  /// [volume]: Ratio(%) of in-ear monitoring volume to original volume with a range of `[0, 100]`.<br>
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes: Call [RTCVideo.setEarMonitorMode] to enable in-ear monitoring function before calling this API.
  Future<int?> setEarMonitorVolume(int volume);

  /// On iOS, you can change the Bluetooth profile when the media volume is set in all scenarios.
  ///
  /// [mode]: The Bluetooth profiles.
  ///
  /// You will receive [RTCVideoEventHandler.onAudioDeviceWarning] in the following scenarios:
  /// + You cannot change the Bluetooth profile to HFP.
  /// + The media volume is not set in all scenarios. We suggest that you call [setAudioScenario] to set the media volume scenario before calling this API.
  Future<int?> setBluetoothMode(BluetoothMode mode);

  /// Changes local voice to a different key, mostly used in Karaoke scenarios.
  ///
  /// You can adjust the pitch of local voice such as ascending or descending with this method.
  ///
  /// [pitch] The value that is higher or lower than the original local voice within a range of `[-12, 12]`. The default value is 0, i.e. no adjustment is made.  <br>
  /// The difference in pitch between two adjacent values within the value range is a semitone, with positive values indicating an ascending tone and negative values indicating a descending tone, and the larger the absolute value set, the more the pitch is raised or lowered.  <br>
  /// Out of the value range, the setting fails and triggers [RTCVideoEventHandler.onWarning].
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> setLocalVoicePitch(int pitch);

  /// Enables/disables the loudness equalization function.
  ///
  /// If you call this API with the parameter set to True, the loudness of user's voice will be adjusted to -16lufs.<br>
  /// If then you also call [RTCMediaPlayer.setLoudness] and import the original loudness of the audio data used in audio mixing, the loudness will be adjusted to -20lufs when the audio data starts to play.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes: You must call this API before calling [RTCMediaPlayer.start] to start playing the audio.
  Future<int?> enableVocalInstrumentBalance(bool enable);

  /// Enables/disables the playback ducking function. This function is usually used in scenarios where short videos or music will be played simultaneously during RTC calls.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// With the function on, if remote voice is detected, the local media volume will be lowered to ensure the clarity of the remote voice. If remote voice disappears, the local media volume restores.
  Future<int?> enablePlaybackDucking(bool enable);

  /// @nodoc('Not available')
  /// Sets the super resolution mode for remote video stream.
  ///
  /// [streamKey]: Remote stream information that specifies the source and type of the video stream.
  ///
  /// [mode]: Super resolution mode.
  ///
  /// Return value:
  /// + 0: Success. It does not indicate the actual status of the super resolution mode, you should refer to [RTCVideoEventHandler.onRemoteVideoSuperResolutionModeChanged] callback.
  /// + -1: Failure. Native library is not loaded.
  /// + -2: Failure. Invalid parameter.
  /// + -9: Failure. Screen stream is not supported.
  ///
  /// Notes:
  /// + This feature is only supported on the ARM architecture.
  /// + Call this API after joining room.
  /// + The original resolution of the remote video stream should not exceed 640 × 360 pixels.
  /// + You can only turn on super-resolution mode for one remote stream.
  Future<int?> setRemoteVideoSuperResolution({
    required RemoteStreamKey streamKey,
    required VideoSuperResolutionMode mode,
  });

  /// @nodoc('Not available')
  /// Sets the video noise reduction mode.
  ///
  /// [mode] Video noise reduction mode.
  ///
  /// Return value:
  /// + 0: Success. Please refer to [RTCVideoEventHandler.onVideoDenoiseModeChanged] callback for the actual state of video noise reduction mode.
  /// + < 0: Failure.
  ///
  /// This feature is only supported on the ARM architecture.
  Future<int?> setVideoDenoiser(VideoDenoiseMode mode);

  /// Enables/Disables the mode of publishing multiple video streams with different encoding configuration. Off by default.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + You should call this method before entering the room or after entering the room but before publishing the stream.
  /// + After setting this API to "True", you can call [RTCVideo.setVideoEncoderConfig] to set encoding configuration for each stream.
  /// + If this function is off, or if this function is on but you don't set the configuration of any stream, only one stream will be sent with a resolution of 640px × 360px and a frame rate of 15fps.
  Future<int?> enableSimulcastMode(bool enable);

  /// Video publisher calls this API to set the parameters of the maximum resolution video stream that is expected to be published, including resolution, frame rate, bitrate, scale mode, and fallback strategy in poor network conditions.
  ///
  /// [maxSolution]: The maximum video encoding parameter.
  ///
  /// Notes:
  /// + You can use [RTCVideo.enableSimulcastMode] simultaneously to publish streams with different resolutions. Specifically, if you want to publish multiple streams with different resolutions, you need to call this method and enable the simulcast mode with [RTCVideo.enableSimulcastMode] before publishing your streams. The SDK will intelligently adjust the number of streams to be published (up to 4) and their parameters based on the settings of the subscribing end. The resolution set by calling this method will be the maximum resolution among the streams. For specific rules, please refer to [Simulcasting](https://docs.byteplus.com/en/byteplus-rtc/docs/70139).
  /// + Without calling this API, SDK will only publish one stream for you with a resolution of 640px × 360px and a frame rate of 15fps.
  /// + This API applies to the video stream captured by the camera, see [RTCVideo.setScreenVideoEncoderConfig] for setting parameters for screen sharing video stream.
  /// + See [Simulcasting](https://docs.byteplus.com/byteplus-rtc/docs/70139) for more details.
  Future<int?> setMaxVideoEncoderConfig(VideoEncoderConfig maxSolution);

  /// Video publisher calls this API to set the configurations of each stream published, including resolution, frame rate, bitrate, scale mode, and fallback strategy in poor network conditions.
  ///
  /// [channelSolutions]: List of configurations for multiple streams. You can set parameters for up to 3 streams, SDK will always take the value of the first 3 streams when parameters for more streams are set.<br>
  /// The resolution you set is the maximum resolution of each stream, and must be arranged in descending order. The frame rates must be arranged in descending order.
  ///
  /// Notes:
  /// + If you call this API after enabling the mode of publishing multiple streams with [RTCVideo.enableSimulcastMode], SDK will publish streams as you set, otherwise the SDK by default publishes the first stream that you have set.
  /// + If you want to publish multiple streams with different resolutions, you need to call this method and [RTCVideo.enableSimulcastMode].
  /// + Without calling this API to set parameters for multiple video streams, the SDK only publishes one video stream with a resolution of 640px × 360px and a frame rate of 15fps.
  /// + After setting parameters for multiple video streams, SDK will automatically match the streams to be published based on the desired subscription parameters set by subscribers.
  /// + This API applies the video stream captured by the camera, see [RTCVideo.setScreenVideoEncoderConfig] for setting parameters for screen sharing video stream.
  /// + See [Simulcasting](https://docs.byteplus.com/byteplus-rtc/docs/70139) for more details.
  Future<int?> setVideoEncoderConfig(List<VideoEncoderConfig> channelSolutions);

  /// Video publisher calls this API to set the expected configurations for the screen sharing video stream, including resolution, frame rate, bitrate, scale mode, and fallback strategy in poor network conditions.
  ///
  /// [screenSolution]: The encoding configuration for shared-screen streams.
  ///
  /// Notes:
  /// + We recommend that you set the encoding configuration before video capture. Otherwise, the streams will be captured based on the default configuration(1080p@15fps).
  /// + On iOS, call this API before publishing screen sharing stream with [RTCVideo.startScreenCapture].
  Future<int?> setScreenVideoEncoderConfig(
      ScreenVideoEncoderConfig screenSolution);

  /// Sets the video capture parameters for internal capturing of the RTC SDK.
  ///
  /// Notes:
  /// + This API can be called after the engine is created and takes effect immediately after the call. It is recommended to call this API before calling [RTCVideo.startVideoCapture].
  /// + It is recommended to use the same video capture parameters for different engines on the same device.
  /// + Internal capturing defaults to Auto mode.
  Future<int?> setVideoCaptureConfig(VideoCaptureConfig config);

  /// Remove the local video canvas.
  Future<int?> removeLocalVideo([StreamIndex streamType = StreamIndex.main]);

  /// Remove the remote video canvas.
  Future<int?> removeRemoteVideo({
    required String roomId,
    required String uid,
    StreamIndex streamType = StreamIndex.main,
  });

  /// Enables internal video capturing immediately. Off by default.
  ///
  /// Internal capture refers to video capture using the built-in capture module of the SDK.<br>
  /// After this API is called, the local user will receive [RTCVideoEventHandler.onVideoDeviceStateChanged].<br>
  /// If this API is called by a visible user in the current room, the other users in the room will receive [RTCVideoEventHandler.onUserStartVideoCapture].
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Calling this API without obtaining permission to use the camera of the current device will trigger [RTCVideoEventHandler.onVideoDeviceStateChanged].
  /// + Call [RTCVideo.stopVideoCapture] to stop the internal video capturing, otherwise it will not stop until the engine is destroyed.
  /// + You can call this API no matter you need to publish audio or not, but you can only publish audio after calling it.
  /// + Android needs to introduce Kotlin in Gradle. iOS needs to request camera permission from the user in the app before it can start capturing.
  Future<int?> startVideoCapture();

  /// Stops internal video capturing immediately.
  ///
  /// After this API is called, the local user will receive [RTCVideoEventHandler.onVideoDeviceStateChanged].<br>
  /// If this API is called by a visible user in the current room, the other users in the room will receive [RTCVideoEventHandler.onUserStopVideoCapture].
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Call [RTCVideo.startVideoCapture] to start internal video capturing.
  /// + If you don't call this API after the internal capture is started, it will not stop until the engine is destroyed.
  Future<int?> stopVideoCapture();

  /// Sets the mirror mode for the captured video stream.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Switching video streams does not affect the settings of the mirror type.
  /// + This API is not applicable to screen-sharing streams.
  /// + Before you call this API, the initial states of each video stream are as follows:
  /// <table border>
  /// <tr><th></th><th>Front-facing camera</th><th>Rear-facing camera</th><th>Built-in camera</th>
  /// <tr><td>Mobile device</td><td>The preview is mirrored. The published video stream is not mirrored.</td><td>The preview and the published video stream are not mirrored.</td><td>/</td>
  /// <tr><td>PC</td><td>/</td><td>/</td><td>The preview is mirrored. The published video stream is not mirrored.</td>
  /// </table>
  Future<int?> setLocalVideoMirrorType(MirrorType mirrorType);

  /// Sets the orientation of the video capture. By default, the App direction is used as the orientation reference.
  ///
  /// During rendering, the receiving client rotates the video in the same way as the sending client does.
  ///
  /// [rotationMode]: Rotation reference can be the orientation of the App or gravity.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + The orientation setting is not effective to screen-sharing stream.
  /// + If the video capture is on, the setting will be effective once you call this API. If the video capture is off, the setting will be effective on when capture starts.
  /// + See [Video Rotation](https://docs.byteplus.com/byteplus-rtc/docs/106458) for more details.
  Future<int?> setVideoRotationMode(VideoRotationMode rotationMode);

  /// Sets the orientation of the video frame before custom video processing and encoding. The default value is `Adaptive`.
  ///
  /// You should set the orientation to `Portrait` when using video effects or custom processing.<br>
  /// You should set the orientation to `Portrait` or `Landscape` when pushing a single stream to the CDN.<br>
  /// For details of different modes, see [Video Orientation](https://docs.byteplus.com/byteplus-rtc/docs/128787).
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes: We recommend you set the orientation before joining room. The updates of encoding configurations and the orientation are asynchronous, therefore can cause a brief malfunction in preview if you change the orientation after joining room.
  Future<int?> setVideoOrientation(VideoOrientation orientation);

  /// Switches mobile front/rear camera.
  ///
  /// After the API is called, the local user will receive [RTCVideoEventHandler.onVideoDeviceStateChanged].
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + The front camera is used by default before calling this method.
  /// + If you are using the camera for video capture, the switching operation takes effect immediately. If the camera is not off, the camera you set will be opened when internal capturing is enabled.
  Future<int?> switchCamera(CameraId cameraId);

  /// Gets video effect API class.
  RTCVideoEffect get videoEffectInterface;

  /// Checks video effect license.
  ///
  /// [licenseFile]: Absolute path of the license file.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use RTCVideoEffect.initCVResource instead.')
  Future<int?> checkVideoEffectLicense(String licenseFile);

  /// Creates/destroys video effects engine.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// Notes:
  /// + Call this API after calling [RTCVideo.checkVideoEffectLicense] and [RTCVideo.setVideoEffectAlgoModelPath].
  /// + After calling this method, you must call [RTCVideo.setVideoEffectNodes] to actually turn on video effects.
  /// + In a common scenario, the special effects engine will be destroyed with the destruction of the RTC engine. When you have high performance requirements, you can call false in this method to destroy the special effects engine separately when the special effects related functions are not used.
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use RTCVideoEffect.enableVideoEffect and RTCVideoEffect.disableVideoEffect instead.')
  Future<int?> enableVideoEffect(bool enable);

  /// Sets the video effects algorithm model path.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use RTCVideoEffect.initCVResource instead.')
  Future<int?> setVideoEffectAlgoModelPath(String modelPath);

  /// Sets video effects resource package.
  ///
  /// [effectNodes]: An array of package paths.<br>
  /// Set this parameter to null to remove the current video effect.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// Notes: Call this API after calling [RTCVideo.enableVideoEffect].
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use RTCVideoEffect.setEffectNodes instead.')
  Future<int?> setVideoEffectNodes(List<String>? effectNodes);

  /// Sets effect intensity.
  ///
  /// [effectNode]: Package path.
  ///
  /// [key]: Name of the resource key to be set, see [Functions of Resource Keys](https://docs.byteplus.com/effects/docs/functions-of-resource-keys-v421-and-later) for details.
  ///
  /// [value]: The intensity value with a range of `[0,1]`. Setting beyond this range is invalid.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// This API is only applicable to special effects resources that contain the above three parameters at the same time. For special effects that do not have strength parameters, such as most stickers, this interface call is invalid.
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use RTCVideoEffect.updateEffectNode instead.')
  Future<int?> updateVideoEffectNode({
    required String effectNode,
    required String key,
    required double value,
  });

  /// Sets color filter.
  ///
  /// [resFile]: Absolute path of the color filter resource package.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use RTCVideoEffect.setColorFilter instead.')
  Future<int?> setVideoEffectColorFilter(String? resFile);

  /// Sets the color filter intensity.
  ///
  /// [intensity]: Filter intensity with a range of `[0,1]`.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use RTCVideoEffect.setColorFilterIntensity instead.')
  Future<int?> setVideoEffectColorFilterIntensity(double intensity);

  /// Sets the original background to a specified image or a solid color.
  ///
  ///  To disable this effect, set the modelPath parameter to null.
  ///
  /// [modelPath]: The path of virtual background effects.
  ///
  /// [source]: The local path of background images.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// Call the following APIs in sequence before calling this API:
  /// 1. [RTCVideo.checkVideoEffectLicense]
  /// 2. [RTCVideo.setVideoEffectAlgoModelPath]
  /// 3. [RTCVideo.enableVideoEffect]
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use RTCVideoEffect.enableVirtualBackground and RTCVideoEffect.disableVirtualBackground instead.')
  Future<int?> setBackgroundSticker({
    String? modelPath,
    VirtualBackgroundSource? source,
  });

  /// Registers the observer for the result of face detection.
  ///
  /// After the observer is registered, the local user will receive [RTCFaceDetectionObserver.onFaceDetectResult] periodically.
  ///
  /// The [interval] (in ms) must be greater than 0. The actual time interval of receiving callbacks is between `interval` and the sum of `interval` and the time slot of a captured video frame.
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use RTCVideoEffect.enableFaceDetection and RTCVideoEffect.disableFaceDetection instead.')
  Future<int?> registerFaceDetectionObserver({
    RTCFaceDetectionObserver? observer,
    int interval = 0,
  });

  /// Enables/Disables basic beauty effects.
  ///
  /// [enable]: Whether to enable basic beauty effects.
  /// + true: Enables basic beauty effects.
  /// + false: (Default) Disables basic beauty effects.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + –1000: The Effect SDK is not integrated.
  /// + –1001: This API is not available for your current RTC SDK.
  /// + –1002: This API is not available for your current Effect SDK. You can upgrade your Effect SDK to v4.4.2+.
  /// + –1003: Contact our technical support team for further instructions.
  /// + –1004: Downloading related resources. The beauty effects will take effect after downloading.
  /// + <0: Failure. Effect SDK internal error. For specific error code, see [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// Notes:
  /// + You cannot use the basic beauty effects and the advanced effect features at the same time. See [how to use advanced effect features](https://docs.byteplus.com/byteplus-rtc/docs/114717) for more information.
  /// + You need to integrate Effect SDK before calling this API. Effect SDK v4.4.2+ is recommended.
  /// + Call [RTCVideo.setBeautyIntensity] to set the beauty effect intensity. If you do not set the intensity before calling this API, the default intensity will be enabled. The default values for the intensity of each beauty mode are as follows: 0.7 for brightening, 0.8 for smoothing, 0.5 for sharpening, and 0.7 for clarity.
  /// + This API is not applicable to screen capturing.
  Future<int?> enableEffectBeauty(bool enable);

  /// Sets the beauty effect intensity.
  ///
  /// The [intensity] must be within the range of `[0,1]`.<br>
  /// When you set it to 0, the beauty effect will be turned off. The default value is 0.5.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// Notes
  /// + If you call this API before calling [RTCVideo.enableEffectBeauty], the default settings of beauty effect intensity will adjust accordingly.
  /// + After the engine is destroyed, the beauty effect settings will be invalid.
  Future<int?> setBeautyIntensity({
    required EffectBeautyMode beautyMode,
    required double intensity,
  });

  /// Change the optical zoom magnification.
  ///
  /// [zoom]: Zoom magnification of the currently used camera (front/postcondition). The value range is [1, < Maximum Zoom Multiplier >].<br>
  /// the maximum value that can be set is obtained through [RTCVideo.getCameraZoomMaxRatio].
  ///
  /// Notes:
  /// + The result of the setting will expire after calling [RTCVideo.stopVideoCapture] to turn off the internal capturing.
  /// + Call [setVideoDigitalZoomConfig] to set digital zoom. Call [setVideoDigitalZoomControl]to perform digital zoom.
  Future<int?> setCameraZoomRatio(double zoom);

  /// Gets the maximum zoom magnification of the currently used camera (front/rear).
  Future<double?> getCameraZoomMaxRatio();

  /// Detects whether the currently used camera supports zoom (digital/optical zoom).
  Future<bool?> isCameraZoomSupported();

  /// Detects whether the currently used camera supports flash.
  Future<bool?> isCameraTorchSupported();

  /// Turns on/off the flash of the currently used camera.
  ///
  /// The result of the setting will expire after calling [RTCVideo.stopVideoCapture] to turn off the internal capturing.
  Future<int?> setCameraTorch(TorchState torchState);

  /// Checks if manual focus is available for the currently used camera.
  Future<bool?> isCameraFocusPositionSupported();

  /// Sets the manual focus position for the currently used camera.
  ///
  /// [position]: The position of the focus point. Setting the upper-left corner of the canvas as the origin, the `x` in `position` means the x-coordinate of the focus point in range of `[0, 1]`, and the `y` in `position` means the y-coordinate of the focus point in range of `[0, 1]`.
  ///
  /// Notes:
  /// + When you set the focus point at the center of the canvas, the focus point setting will be canceled.
  /// + The camera focus point setting will be invalid after calling [RTCVideo.stopVideoCapture] to stop internal capturing.
  Future<int?> setCameraFocusPosition(Offset position);

  /// Checks if manual exposure setting is available for the currently used camera.
  Future<bool?> isCameraExposurePositionSupported();

  /// Sets the manual exposure position for the currently used camera.
  ///
  /// [position]: The position of the exposure point. Setting the upper-left corner of the canvas as the origin, the `x` in `position` means the x-coordinate of the exposure point in range of `[0, 1]`, and the `y` in `position` means the y-coordinate of the exposure point in range of `[0, 1]`.
  ///
  /// Notes:
  /// + When you set the exposure point at the center of the canvas, the exposure point setting will be canceled.
  /// + The camera exposure point setting will be invalid after calling [RTCVideo.stopVideoCapture] to stop internal capturing.
  Future<int?> setCameraExposurePosition(Offset position);

  /// Sets the exposure compensation for the currently used camera.
  ///
  /// [val]: Exposure compensation in range of `[-1, 1]`. Default to 0, which means no exposure compensation.
  ///
  /// The camera exposure compensation setting will be invalid after calling [RTCVideo.stopVideoCapture] to stop internal capturing.
  Future<int?> setCameraExposureCompensation(double val);

  /// Valid since v3.56.1
  ///
  /// Enable or disable face auto exposure mode during internal video capture. This mode fixes the problem that the face is too dark under strong backlight; but it will also cause the problem of too bright/too dark in the area outside the ROI region.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure.
  ///
  /// You must call this API before calling [RTCVideo.startVideoCapture] to enable video capture to make the setting valid.
  Future<int?> enableCameraAutoExposureFaceMode(bool enable);

  /// Valid since v3.56.1
  ///
  /// Sets the minimum frame rate of of the dynamic framerate mode during internal video capture.
  ///
  /// [framerate]: The minimum value in fps. The default value is 7.<br>
  /// The maximum value of the dynamic framerate mode is set by calling [RTCVideo.setVideoCaptureConfig]. When minimum value exceeds the maximum value, the frame rate is set to a fixed value as the maximum value; otherwise, dynamic framerate mode is enabled.
  ///
  /// Return value:
  /// + 0: Success.
  /// + !0: Failure.
  ///
  /// Notes
  /// + You must call this API before calling [RTCVideo.startVideoCapture] to enable video capture to make the setting valid.
  /// + If the maximum frame rate changes due to performance degradation, static adaptation, etc., the set minimum frame rate value will be re-compared with the new maximum value. Changes in comparison results may cause switch between fixed and dynamic frame rate modes.
  Future<int?> setCameraAdaptiveMinimumFrameRate(int framerate);

  /// Sends SEI data.
  ///
  /// In a video call scenario, SEI is sent with the video frame, while in a voice call scenario, SDK will automatically publish a black frame with a resolution of 16px × 16px to carry SEI data.
  ///
  /// [streamIndex] specifies the type of media stream that carries SEI data.<br>
  /// In a voice call, you should set this parameter to `main`, otherwise the SEI data is discarded and cannot be sent to the remote user.
  ///
  /// The length of [message] must not exceed 4KB.
  ///
  /// [repeatCount]: Number of times a message is sent repeatedly. The value range is `[0, max{29, %{video frame rate}-1}]`. Recommended range: `[2,4]`.<br>
  /// After calling this API, the SEI data will be added to a consecutive `repeatCount+1` number of video frames starting from the current frame.
  ///
  /// [mode]: SEI sending mode.
  ///
  /// Returned value:
  /// + `>=0`: The number of SEIs to be added to the video frame.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + We recommend the number of SEI messages per second should not exceed the current video frame rate. In a voice call, the blank-frame rate is 15 fps.
  /// + In a voice call, this API can be called to send SEI data only in internal capture mode.
  /// + Each video frame carries only the SEI data received within 2s before and after. In a voice call scenario, if no SEI data is sent within 1min after calling this API, SDK will automatically cancel publishing black frames.
  /// + After the message is sent successfully, the remote user who subscribed the video stream will receive [RTCVideoEventHandler.onSEIMessageReceived].
  /// + When you switch from a voice call to a video call, SEI data will automatically start to be sent with normally captured video frames instead of black frames.
  Future<int?> sendSEIMessage({
    StreamIndex streamIndex = StreamIndex.main,
    required Uint8List message,
    required int repeatCount,
    SEICountPerFrame mode = SEICountPerFrame.single,
  });

  /// Sets the step size for each digital zooming control to the local videos.
  ///
  /// [type]: Identifying which type the `size` is referring to. Required parameter.
  ///
  /// [size]: Reserved to three decimal places. It defaults to `0`. Required parameter.<br>
  /// The meaning and range vary from different `type`. If the scale or moving distance exceeds the range, the limit is taken as the result.
  /// + `focusOffset`: Increment or decrease to the scaling factor. Range: `[0, 7]`. For example, when it is set to 0.5 and [setVideoDigitalZoomControl] is called to zoom in, the scale will increase `0.5`. The scale ranges `[1，8]` and defaults to `1`, which means an original size.
  /// + `moveOffset`: Ratio of the distance to the border of video images. It ranges `[0, 0.5]` and defaults to `0`, which means no offset. When you call [setVideoDigitalZoomControl] and choose `cameraMoveLeft`, the moving distance is size x original width. While for the `cameraMoveUp`, the moving distance is size x original height. Suppose that a video spans 1080 px and the `size` is set to `0.5` so that the distance would be 0.5 x 1080 px = 540 px.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Only one size can be set for a single call. You must call this API to pass values respectively if you intend to set multiple `size`s.
  /// + As the default `size` is `0`, you must call this API before performing any digital zoom control by calling [setVideoDigitalZoomControl] or [startVideoDigitalZoomControl].
  Future<int?> setVideoDigitalZoomConfig({
    required ZoomConfigType type,
    double size,
  });

  /// Digital zoom or move the local video image once. This action affects both the video preview locally and the stream published.
  ///
  /// Valid since v3.51.1
  ///
  /// [direction]: Action of the digital zoom control
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + As the default offset is `0`, you must call [setVideoDigitalZoomConfig].
  /// + You can only move video images after they are magnified via this API or [startVideoDigitalZoomControl].
  /// + When you request an out-of-range scale or movement, SDK will execute it with the limits. For example, when the image has been moved to the border, the image cannot be zoomed out, or has been magnified to 8x.
  /// + Call [startVideoDigitalZoomControl] to have a continuous and repeatedly digital zoom control.
  /// + Refer to [setCameraZoomRatio] if you intend to have an optical zoom control to the camera.
  Future<int?> setVideoDigitalZoomControl(ZoomDirectionType direction);

  /// Continuous and repeatedly digital zoom control. This action effect both the video preview locally and the stream published.
  ///
  /// Valid since v3.51.1
  ///
  /// [direction]: Action of the digital zoom control.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + As the default offset is `0`, you must call [setVideoDigitalZoomConfig] before this API.
  /// + You can only move video images after they are magnified via this API or [setVideoDigitalZoomControl].
  /// + The control process stops when the scale reaches the limit, or the images have been moved to the border. if the next action exceeds the scale or movement range, SDK will execute it with the limits.
  /// + Call [stopVideoDigitalZoomControl] to stop the ongoing zoom control.
  /// + Call [setVideoDigitalZoomControl] to have a one-time digital zoom control.
  /// + Refer to [setCameraZoomRatio] if you intend to have an optical zoom control to the camera.
  Future<int?> startVideoDigitalZoomControl(ZoomDirectionType direction);

  /// Stops the ongoing digital zoom control instantly.
  ///
  /// Valid since v3.51.1
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Call [startVideoDigitalZoomControl] to start digital zooming.
  Future<int?> stopVideoDigitalZoomControl();

  /// Designate the current audio playback device.
  ///
  /// The default device is set through [RTCVideo.setDefaultAudioRoute].<br>
  /// When the audio playback route changes, the local user will receive [RTCVideoEventHandler.onAudioRouteChanged].
  ///
  /// You can not set [audioRoute] to `Unknown`.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + This API is only supported in the `AUDIO_SCENARIO_COMMUNICATION` audio scenario. Call [setAudioScenario] to switch between different audio scenarios.
  /// + You can implement most scenarios by calling [setDefaultAudioRoute] and the default audio route switching strategy of the RTC SDK. For details about the strategy, see [Set the Audio Route](https://docs.byteplus.com/byteplus-rtc/docs/117836). You should use this API in a few exceptional scenarios like manually switching audio route with external audio device connected.
  /// + For the volume type in different audio scenarios, see [AudioScenario].
  Future<int?> setAudioRoute(AudioRoute audioRoute);

  /// Sets the speaker or earpiece as the default audio playback device.
  ///
  /// Returned value:
  /// + `0`: Success. The setting takes effect immediately. However, the audio route will not switch to the default device until all the audio peripheral devices are disconnected.
  /// + `<0`: Failure. It fails when the device designated is neither a speaker nor an earpiece.
  ///
  /// Notes: For the default audio route switching strategy of the RTC SDK, see [Set the Audio Route](https://docs.byteplus.com/byteplus-rtc/docs/117836).
  Future<int?> setDefaultAudioRoute(AudioRoute audioRoute);

  /// Gets the information of currently used playback route.
  Future<AudioRoute> getAudioRoute();

  /// Enables the audio process mode for external sound card.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + When you use external sound card for audio capture, enable this mode for better audio quality.
  /// + When using the mode, you can only use earphones. If you need to use internal or external speaker, disable this mode.
  Future<int?> enableExternalSoundCard(bool enable);

  /// Creates a new task of pushing media streams to CDN and sets the relevant configurations.
  ///
  /// When pushing more than one live streams in the same task, SDK will first mix those streams into one single stream and then push it to CDN.
  ///
  /// [taskId]: Task ID.<br>
  /// You may want to push more than one mixed stream to CDN from the same room. When you do that, use different ID for corresponding tasks; if you will start only one task, use an empty string.
  ///
  /// [transcoding]: Configurations to be set when pushing streams to CDN.
  ///
  /// [observer]: Register this observer to receive callbacks from the SDK.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Before calling this API，you need to enable Push to CDN feature on the [console](https://console.byteplus.com/rtc/workplaceRTC).
  /// + After this API is called, the local user will receive [RTCLiveTranscodingObserver.onStreamMixingEvent] about the result and errors during the pushing process.
  /// + Call [RTCVideo.stopLiveTranscoding]  to stop pushing streams to CDN.
  @Deprecated('Deprecated since v3.54.1, use startPushMixedStreamToCDN instead')
  Future<int?> startLiveTranscoding({
    required String taskId,
    required LiveTranscoding transcoding,
    required RTCLiveTranscodingObserver observer,
  });

  /// Stops pushing streams to CDN.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  @Deprecated('Deprecated since v3.54.1, use stopPushStreamToCDN instead')
  Future<int?> stopLiveTranscoding(String taskId);

  /// Updates parameters needed when pushing media streams to CDN.
  ///
  /// [transcoding]: Configurations to be set when pushing streams to CDN. You can update any property for the task unless it is specified as unavailable for updates.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  @Deprecated(
      'Deprecated since v3.54.1, use updatePushMixedStreamToCDN instead')
  Future<int?> updateLiveTranscoding({
    required String taskId,
    required LiveTranscoding transcoding,
  });

  /// Valid since v3.56.1
  ///
  /// Creates a new task of pushing mixed media streams to CDN and sets the relevant configurations.<br>
  /// When pushing more than one live streams in the same task, SDK will first mix those streams into one single stream and then push it to CDN.
  ///
  /// [taskId]: Task ID. The length should not exceed 126 bytes.<br>
  /// You may want to push more than one mixed stream to CDN from the same room. When you do that, use different ID for corresponding tasks; if you will start only one task, use an empty string.
  ///
  /// [mixedConfig]: Configurations to be set when pushing streams to CDN.
  ///
  /// [observer]: Register this observer to receive callbacks from the SDK.
  ///
  /// Return value:
  /// + 0: Success.
  /// + !0: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Before calling this API，you need to enable Push to CDN on the [console](https://console.byteplus.com/rtc/workplaceRTC).
  /// + After calling this API, you will be informed of the result and errors during the pushing process through [RTCMixedStreamObserver.onMixingEvent].
  /// + Call [RTCVideo.stopPushStreamToCDN] to stop pushing mixed streams to CDN.
  Future<int?> startPushMixedStreamToCDN({
    required String taskId,
    required MixedStreamConfig mixedConfig,
    RTCMixedStreamObserver? observer,
  });

  /// Valid since v3.56.1
  ///
  /// Updates parameters needed when pushing mixed media streams to CDN. You will be informed of the change through [RTCMixedStreamObserver.onMixingEvent].<br>
  /// After calling [RTCVideo.startPushMixedStreamToCDN] to enable the function of pushing streams to CDN, you can call this API to update the relevant configurations.
  ///
  /// [taskId]:Task ID specifying of which pushing task you want to update the parameters.
  ///
  /// [mixedConfig]Configurations that you want to update. You can update any property for the task unless it is specified as unavailable for updates.<br>
  /// If you left some properties blank, you can expect these properties to be set to their default values.
  ///
  /// Return value:
  /// + 0: Success.
  /// + !0: Failure. See [ReturnStatus] for the reason.
  Future<int?> updatePushMixedStreamToCDN({
    required String taskId,
    required MixedStreamConfig mixedConfig,
  });

  /// This function is currently not available. Creates a new task of pushing a single media stream to CDN.
  ///
  /// [taskId]: Task ID.<br>
  /// You may want to start more than one task to push streams to CDN. When you do that, use different IDs for corresponding tasks; if you will start only one task, use an empty string.
  ///
  /// [param]: Configurations for pushing a single stream to CDN.
  ///
  /// [observer]: Register this observer to receive callbacks from the SDK.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + After this API is called, the local user will receive [RTCPushSingleStreamToCDNObserver.onStreamPushEvent] about the result and errors during the pushing process.
  /// + Call [RTCVideo.stopPushStreamToCDN] to stop the task.
  Future<int?> startPushSingleStreamToCDN({
    required String taskId,
    required PushSingleStreamParam param,
    required RTCPushSingleStreamToCDNObserver observer,
  });

  /// Stops the task of pushing a single media stream to CDN.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> stopPushStreamToCDN(String taskId);

  /// @nodoc('Not available')
  /// Starts publishing a public media stream.
  ///
  /// A public stream refers to a media stream that does not belong to any room or any user. Users within the same `appID` can call [RTCVideo.startPlayPublicStream] to subscribe to the public stream regardless the user has joined which room or has not joined any room.
  ///
  /// [publicStreamId]: ID of the public stream.
  ///
  /// [publicStreamParam]: Configurations about the public stream.<br>
  /// A public stream can include a bundle of media streams and appears as the designated layout.<br>
  /// If the specified media stream has not yet been published, the public stream will be updated in real time after the specified stream starts to be published.
  ///
  /// Returned value:
  /// + 0: Success, and [RTCVideoEventHandler.onPushPublicStreamResult] will be triggered.
  /// + !0: Failure because of invalid parameter or empty parameters.
  ///
  /// Notes:
  /// + Call [RTCVideo.updatePublicStreamParam] to update the properties of the public stream which is published by the same user. Calling this API with the same stream ID repeatedly by the same user can not update the existing public stream.
  /// + If Users with different userID call this API with the same stream ID, the public stream will be updated with the parameters passed in the latest call.
  /// + To publish multiple public streams, call this API with different stream ID.
  /// + Call [RTCVideo.stopPushPublicStream] to stop publishing the public stream.
  /// + Please contact technical support to enable this function before calling the public stream related APIs.
  Future<int?> startPushPublicStream({
    required String publicStreamId,
    required PublicStreaming publicStreamParam,
  });

  /// @nodoc('Not available')
  /// Stops publishing the public stream.
  Future<int?> stopPushPublicStream(String publicStreamId);

  /// @nodoc('Not available')
  /// Updates the configurations of the public stream.
  Future<int?> updatePublicStreamParam({
    required String publicStreamId,
    required PublicStreaming publicStreamParam,
  });

  /// @nodoc('Not available')
  /// Subscribes to specified public stream.
  ///
  /// This API can be called to get and play the specified public stream regardless of whether the user is in the room or not.<br>
  /// If the stream has not been published then, the local user will receive the public stream once it starts publishing.
  ///
  /// Returned value:
  /// + 0: Success, and [RTCVideoEventHandler.onPlayPublicStreamResult] will be triggered.
  /// + !0: Failure because of invalid parameter or empty parameters.
  ///
  /// Notes:
  /// + We recommend you bind a view for the public stream before calling this API.
  /// + After calling this API, the local user will receive [RTCVideoEventHandler.onFirstPublicStreamVideoFrameDecoded] and [RTCVideoEventHandler.onFirstPublicStreamAudioFrame] about first video and audio frame decoding for public streams.
  /// + If the public stream contains SEI information, the local user will be informed by [RTCVideoEventHandler.onPublicStreamSEIMessageReceived].
  /// + Call [RTCVideo.stopPlayPublicStream] to unsubscribe from the public stream.
  Future<int?> startPlayPublicStream(String publicStreamId);

  /// @nodoc('Not available')
  /// Stops subscribing from the public stream.
  Future<int?> stopPlayPublicStream(String publicStreamId);

  /// @nodoc('Not available')
  /// Unsets the public stream video canvas.
  Future<int?> removePublicStreamVideo(String publicStreamId);

  /// @nodoc('Not available')
  /// Sets the audio playback volume of the public stream.
  ///
  /// Valid since v3.51.1
  ///
  /// [publicStreamId]: ID of the public stream.
  ///
  /// [volume]: Ratio(%) of the audio playback volume to the original volume, in the range `[0, 400]`, with overflow protection. The default volume is 100. <br>
  /// To ensure the audio quality, the recommended range is `[0,  100]`.
  ///
  /// Return value:
  /// + 0: Success. <br>
  /// + -2: Wrong parameter.
  Future<int?> setPublicStreamAudioPlaybackVolume({
    required String publicStreamId,
    required int volume,
  });

  /// Sets the business ID.
  ///
  /// You can distinguish different business scenarios (roles/strategies, etc.) by [businessId].<br>
  /// You can customize your businessId to serve as a sub AppId, which can share and refine the function of the AppId, but it does not need authentication.
  ///
  /// Returned value:
  /// + 0: Success
  /// + <0: Failure
  /// + -6001: The user is already in the room.
  /// + -6002: Invalid parameter. Legal characters include all lowercase letters, uppercase letters, numbers, and four other symbols, including '.', '-','_', and '@'.
  ///
  /// Notes: Call this API before calling [RTCRoom.joinRoom].
  Future<int?> setBusinessId(String businessId);

  /// Reports user feedback to RTC.
  ///
  /// [types]: A collection of pre-defined question types with multiple choices.
  /// [info]: Specific descriptions of issues other than pre-defined questions.
  ///
  /// Returned value:
  /// + 0: Success
  /// + -1: Failure, the user has not joined a room yet.
  /// + -2: Failure, data analysis error.
  /// + -3: Failure, missing fields.
  ///
  /// Notes:
  /// If the user is in the room when reporting feedback, then the problem is located in the room(s) the user is currently in.<br>
  /// If the user is not in the room, then the problem is located in the room the engine previously exited.
  Future<int?> feedback({
    required List<ProblemFeedbackOption> types,
    ProblemFeedbackInfo? info,
  });

  /// Sets the fallback option for published audio & video streams.
  ///
  /// You can call this API to set whether to automatically lower the resolution you set of the published streams under limited network conditions.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + This API only works after you call [RTCVideo.enableSimulcastMode] to enable the mode of publishing multiple streams.
  /// + You must call this API before the user enters the room.
  /// + After calling this API, if there is a performance fallback or recovery due to device performance or network issues, the local end will receive early warnings through the [RTCVideoEventHandler.onPerformanceAlarms] callback to adjust the capture device.
  /// + When the fallback option is set, the client subscribed to the audio/video stream will receive [RTCVideoEventHandler.onSimulcastSubscribeFallback] when the audio/video stream published by the local user falls back or resumes from the fallback.
  /// + You can alternatively set fallback options in th console, which is of higher priority.
  Future<int?> setPublishFallbackOption(PublishFallbackOption option);

  /// Sets the fallback option for subscribed RTC streams..
  ///
  /// You can call this API to set whether to lower the resolution of currently subscribed stream under limited network conditions.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + You must call this API before the user enters the room.
  /// + When the fallback option is set, the local user will receive [RTCVideoEventHandler.onSimulcastSubscribeFallback] when the subscribed audio/video stream falls back or resumes from a fallback.
  /// + You can alternatively set fallback options in th console, which is of higher priority.
  Future<int?> setSubscribeFallbackOption(SubscribeFallbackOption option);

  /// Set the priority of the specified remote user in the fallback.
  ///
  /// If the subscribe stream fallback option is enabled, the quality of streams received from high-priority users will be prioritized under limit network conditions.
  ///
  /// Notes:
  /// + Call this API together with [RTCVideo.setSubscribeFallbackOption].
  /// + This method can be used before and after entering the room, and the priority of the remote user can be modified.
  Future<int?> setRemoteUserPriority({
    required String roomId,
    required String uid,
    required RemoteUserPriority priority,
  });

  /// Set the built-in encryption method.
  ///
  /// [key]: Encryption key, limited to 36 bits in length, beyond which it will be truncated.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes: This API can be called repeatedly before or after the user enters the room with the last called parameter as the effective parameter.
  Future<int?> setEncryptInfo({
    required EncryptType aesType,
    required String key,
  });

  /// Creates a room instance.
  ///
  /// This API only returns a room instance. You still need to call [RTCRoom.joinRoom] to actually create/join the room.<br>
  /// A new [RTCRoom] instance is created every time this API is called.<br>
  /// Call [RTCRoom.joinRoom] in each RTCRoom instance separately to join multiple rooms at the same time.<br>
  /// In multi-room mode, users can subscribe to the audio and video streams of each room at the same time.
  ///
  /// [roomId]: An non-empty string within 128 bytes. Each room created by calling this API requires a unique roomId, so that the room can be distinguished from each other.<br>
  /// The following characters can be used:
  /// + 26 uppercase letters A~Z.
  /// + 26 lowercase letters a~z.
  /// + 10 numbers: 0~9.
  /// + "_", "@" or "-".<br>
  /// In multi-room mode, you need to use different roomId to create multiple rooms, otherwise the newly created room instance will replace the old one.
  ///
  /// Notes:
  /// + If the room that you wish to join already exists, you still need to call this API first to create the RTCRoom instance, and then call [RTCRoom.joinRoom].
  /// + To publish streams to the multiple rooms, call [RTCRoom.startForwardStreamToRooms] instead of enabling multi-room mode.
  Future<RTCRoom?> createRTCRoom(String roomId);

  /// Starts capturing screen sharing streams with the internal capturing module of th RTC SDK.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Screen sharing related APIs are only applicable to iOS 12 and above.
  /// + For iOS, if you start the Extension from the iOS control center, this API do not need to be called.
  /// + After the streams are captured, you need to call [RTCRoom.publishScreen] to publish the streams.
  /// + After this API is successfully called, the local user will receive [RTCVideoEventHandler.onVideoDeviceStateChanged] and [RTCVideoEventHandler.onAudioDeviceStateChanged].
  /// + Call [RTCVideo.stopScreenCapture] to stop capturing screen sharing streams.
  Future<int?> startScreenCapture(ScreenMediaType type);

  /// Updates the media type of the screen capture.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> updateScreenCapture(ScreenMediaType type);

  /// Stops capturing screen sharing streams.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> stopScreenCapture();

  /// Sends custom messages to screen sharing Extension. For iOS only.
  ///
  /// Call this API after calling [RTCVideo.startScreenCapture]
  Future<int?> sendScreenCaptureExtensionMessage(Uint8List message);

  /// Sets runtime parameters.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> setRuntimeParameters(Map<String, dynamic> params);

  /// @nodoc('Not available')
  /// Enables automatic speech recognition service.
  ///
  /// This method converts the recognized user's voice into text and calls it back to the user through [RTCASREngineEventHandler.onMessage].
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> startASR({
    required RTCASRConfig asrConfig,
    required RTCASREngineEventHandler handler,
  });

  /// @nodoc('Not available')
  /// Disables automatic speech recognition service.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> stopASR();

  /// Records the audio & video data during the call to a local file.
  ///
  /// Returned value:
  /// + 0: Normal.
  /// + -1: Parameter exception.
  /// + -2: The current version of the SDK does not support this feature, please contact technical support.
  ///
  /// Notes:
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onRecordingStateUpdate].
  /// + If the recording is normal, the system will report the recording progress through [RTCVideoEventHandler.onRecordingProgressUpdate] every second.
  Future<int?> startFileRecording({
    StreamIndex streamIndex = StreamIndex.main,
    required RecordingConfig config,
    required RecordingType recordingType,
  });

  /// Stops local recording.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + After calling [RTCVideo.startFileRecording], you must call this API to stop local recording.
  /// + The recording result will be called back through [RTCVideoEventHandler.onRecordingStateUpdate].
  Future<int?> stopFileRecording([StreamIndex streamIndex = StreamIndex.main]);

  /// Starts recording audio communication, and generates the local file.
  ///
  /// If you call this API before or after joining the room without internal audio capture, then the recording task can still begin but the data will not be recorded in the local files. Only when you call [startAudioCapture] to enable internal audio capture, the data will be recorded in the local files.
  ///
  /// Return value:
  /// + 0: Success.
  /// + -2: Invalid parameters.
  /// + -3: Not valid in this SDK. Please contact the technical support.
  ///
  /// Notes:
  /// + All audio effects are valid in the file. Mixed audio file is not included in the file.
  /// + You can call this API before and after joining the room. If this API is called before you join the room, you need to call [stopAudioRecording] to stop recording. If this API is called after you join the room, the recording task ends automatically. If you join multiple rooms, audio from all rooms are recorded in one file.
  /// + After calling the API, you'll receive [RTCVideoEventHandler.onAudioRecordingStateUpdate].
  Future<int?> startAudioRecording(AudioRecordingConfig config);

  /// Stops audio recording.
  ///
  /// Return value:
  /// + 0: Success <br>
  /// + -3: Not valid in this SDK. Please contact the technical support.
  ///
  /// After calling [startAudioRecording] to start audio recording, you must call this API to stop audio recording.
  Future<int?> stopAudioRecording();

  /// Creates the audio mixing manager.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCVideo.getAudioEffectPlayer and RTCVideo.getMediaPlayer instead')
  RTCAudioMixingManager get audioMixingManager;

  /// Valid since v3.56.1
  ///
  /// Creates an instance for audio effect player.
  ///
  /// Return value: An audio effect player instance.
  FutureOr<RTCAudioEffectPlayer?> getAudioEffectPlayer();

  /// Valid since v3.56.1
  ///
  /// Creates a media player instance.
  ///
  /// [playerId]: Media player id. The range is `[0, 3]`. You can create up to 4 instances at the same time. If it exceeds the range, nullptr will be returned.
  ///
  /// Return value: A media player instance.
  FutureOr<RTCMediaPlayer?> getMediaPlayer(int playerId);

  /// Logs in to send out-of-room messages or send messages to the business server.
  ///
  /// The user must carry a [token] for authentication while logging in.<br>
  /// This Token is different from that required by calling [RTCRoom.joinRoom].<br>
  /// You can assign any value even null to `roomId` to generate a login token. During developing and testing, you can use temporary tokens generated on the console. Deploy the token generating application on your server.
  ///
  /// [uid] is unique within one appId.
  ///
  /// Returned value:
  /// + `0`: Success.
  /// + `-1`: Failure due to invalid parameter.
  /// + `-2`: Failure. The user is already logged in.
  /// + `-3`: Failure. There is no created engine instance. Call [RTCVideo.createRTCVideo] before calling this method.
  ///
  /// Notes:
  /// + To log out, call  [RTCVideo.logout].
  /// + After this API is successfully called, the local user will receive [RTCVideoEventHandler.onLoginResult].
  Future<int?> login({
    required String token,
    required String uid,
  });

  /// Logs out.
  ///
  /// After calling this API to logout, it is not possible to call methods or receive callbacks related to out-of-room messages and end-to-server messages.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// After this API is successfully called, the local user will receive [RTCVideoEventHandler.onLogout].
  Future<int?> logout();

  /// Updates the Token used by the user to log in.
  ///
  /// The Token has a certain valid period. When the Token expires, you need to call this method to update the login Token.<br>
  /// If [RTCVideo.login] is called with an expired Token, the login fails and the local user will receive [RTCVideoEventHandler.onLoginResult] about corresponding error code. You need to reacquire a Token and update it with this method.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// If the token is invalid and the login fails, call this method to update the token, and the SDK will automatically log in again. The user does not need to call [RTCVideo.login].
  Future<int?> updateLoginToken(String token);

  /// Sets application server parameters.
  ///
  /// Before calling [RTCVideo.sendServerMessage] or [RTCVideo.sendServerBinaryMessage] to send messages to the application server, call this API to set a valid signature and application server address.
  ///
  /// [signature]: Dynamic signature. The App server may use the signature to verify the source of messages.<br>
  /// You need to define the signature yourself. It can be any non-empty string. It is recommended to encode information such as UID into the signature.<br>
  /// The signature will be sent to the address set through the "url" parameter in the form of a POST request.
  ///
  /// [url]: Address of the application server.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// + Call this API after calling [RTCVideo.login] to log in.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onServerParamsSetResult].
  Future<int?> setServerParams({
    required String signature,
    required String url,
  });

  /// Queries the login status of the specified remote user or local user before sending the message.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// + Call this API after calling [RTCVideo.login] to log in.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onGetPeerOnlineStatus].
  /// + Before sending out-of-room messages, users can find out whether the user on the other end is logged in or not through this API, so that they can decide whether to send messages or not. You can also check your own login status through this API.
  Future<int?> getPeerOnlineStatus(String peerUid);

  /// Sends a text message to a specified user outside the room.
  ///
  /// The length of [message] must not exceed 64KB.
  ///
  /// + Call this API after calling [RTCVideo.login] to log in.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onUserMessageSendResultOutsideRoom] about the sending result.
  /// + If the text message is sent successfully, the user specified by [uid] will receive [RTCVideoEventHandler.onUserMessageReceivedOutsideRoom].
  Future<int?> sendUserMessageOutsideRoom({
    required String uid,
    required String message,
    required MessageConfig config,
  });

  /// Sends a binary message to a specified user outside the room.
  ///
  /// The length of [message] must not exceed 46KB.
  ///
  /// + Call this API after calling [RTCVideo.login] to log in.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onUserMessageSendResultOutsideRoom] about the sending result.
  /// + If the binary message is sent successfully, the user specified by [uid] will receive [RTCVideoEventHandler.onUserBinaryMessageReceivedOutsideRoom]。
  Future<int?> sendUserBinaryMessageOutsideRoom({
    required String uid,
    required Uint8List message,
    required MessageConfig config,
  });

  /// Client sends a text message to the business server.
  ///
  /// The length of [message] must not exceed 64KB.
  ///
  /// + Call this API after calling [RTCVideo.login] to log in and [RTCVideo.setServerParams] to set up the business server.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onServerMessageSendResult].
  Future<int?> sendServerMessage(String message);

  /// Client sends a binary message to the business server.
  ///
  /// The length of [message] must not exceed 46KB.
  ///
  /// + Call this API after calling  [RTCVideo.login] to log in and [RTCVideo.setServerParams] to set up the business server.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onServerMessageSendResult].
  Future<int?> sendServerBinaryMessage(Uint8List message);

  /// Starts pre-call network detection.
  ///
  /// [isTestUplink]: Whether to detect uplink bandwidth.
  ///
  /// [expectedUplinkBitrate]: Expected uplink bandwidth in kbps within a range of `{0, [100-10000]}`. `0` indicates that the maximum code rate is specified by the SDK.
  ///
  /// [isTestDownlink]: Whether to detect downlink bandwidth.
  ///
  /// [expectedDownlinkBitrate]: Expected downlink bandwidth in kbps within a range of `{0, [100-10000]}`. `0` indicates that the maximum code rate is specified by the SDK.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + After a successful call to this API, the [RTCVideoEventHandler.onNetworkDetectionResult] callback will be received once in 3s and every 2s thereafter to notify the detection result.
  /// + If detection is stopped, the detection will be notified once by [RTCVideoEventHandler.onNetworkDetectionStopped].
  Future<int?> startNetworkDetection({
    required bool isTestUplink,
    required int expectedUplinkBitrate,
    required bool isTestDownlink,
    required int expectedDownlinkBitrate,
  });

  /// Stops pre-call network detection.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Calling this API will trigger [RTCVideoEventHandler.onNetworkDetectionStopped] notifying the stop of the detection.
  Future<int?> stopNetworkDetection();

  /// Sets whether to mix screen audio stream and the audio stream collected by the microphone in screen sharing.
  ///
  /// Specify the mixing method via [index]:
  /// + `main`: Mixing the screen audio stream with the audio stream captured by the microphone.
  /// + `screen`: Default setting. Split the screen audio stream and the audio stream captured by the microphone into two audio streams.
  ///
  /// Call this API before [RTCRoom.publishScreen], otherwise [RTCVideoEventHandler.onWarning] will be triggered.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> setScreenAudioStreamIndex(StreamIndex index);

  /// Sends audio stream synchronization information.
  ///
  /// This method sends the message to the remote end via an audio stream and synchronizes it with the audio stream.
  ///
  /// The [data] length must be `[1, 16]` bytes.
  ///
  /// Returned value:
  /// + `>=0`:  Message sent successfully. Returns the number of successful sends
  /// + `-1`: Message sending failed. The message length is greater than 255 bytes.
  /// + `-2`: Message sending failed. The message content is empty.
  /// + `-3`: Message sending failed. This screen stream was not published when the message was synchronized through the screen stream.
  /// + `-4`: Message sending failed. This audio stream is not yet published when you synchronize messages with an audio stream captured by a microphone or custom device, as described in [ErrorCode].
  ///
  /// + Regarding the frequency, we recommend no more than 50 calls per second.
  /// + After a successful call to this API, the remote user will receive [RTCVideoEventHandler.onStreamSyncInfoReceived].
  Future<int?> sendStreamSyncInfo({
    required Uint8List data,
    required StreamSyncInfoConfig config,
  });

  /// Controls whether to play local audio streams.
  ///
  /// This method does not affect local audio playback devices.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use RTCVideo.setPlaybackVolume instead.')
  Future<int?> muteAudioPlayback(bool muteState);

  /// Starts a call test.
  ///
  /// Before entering the room, you can call this API to test whether your local audio/video equipment as well as the upstream and downstream networks are working correctly.  <br>
  /// Once the test starts, SDK will record your sound or video. If you receive the playback within the delay range you set, the test is considered normal.
  ///
  /// [delayTime]: Delayed audio/video playback time specifying how long you expect to receive the playback after starting the. The range of the value is `[2,10]` in seconds and the default value is 2.
  ///
  /// API call result:
  /// + 0: Success.
  /// + -1: Failure, testing in progress.
  /// + -2: Failure, the user us already in the room.
  /// + -3: Failure, neither video nor audio is captured.
  /// + -4: Failure, parameter exception.
  /// + -5: Failure, the roomID is already used.
  ///
  /// Notes:
  /// + Once you start the test, you can either call [RTCVideo.stopEchoTest] or wait until the test stops automatically after 60s, to start the next test or enter the room.
  /// + All APIs related to device control and stream control called before this API are invalidated during the test and are restored after the test.
  /// + All the calls to APIs related to device control, stream control, and room entry called during the test do not take effect, and will triggered [RTCVideoEventHandler.onWarning].
  /// + The result of the audio/video loop detection is notified via the [RTCVideoEventHandler.onEchoTestResult] callback.
  Future<int?> startEchoTest({
    required EchoTestConfig config,
    required int delayTime,
  });

  /// Stops the current call test.
  ///
  /// Notes: After stopping the test with this API, all the system devices and streams are restored to the state they were in before the test.
  Future<int?> stopEchoTest();

  /// Adds watermark to designated video stream.
  ///
  /// [imagePath]: File path of the watermark image. You can use the absolute path or the Asset path(/assets/xx.png).<br>
  /// The path should be less than 512 bytes. <br>
  /// The watermark image should be in PNG or JPG format.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Call [RTCVideo.clearVideoWatermark] to remove the watermark on the designated video stream.
  /// + You can only add one watermark to one video stream. The newly added watermark replaces the previous one. You can call this API multiple times to add watermarks to different streams.
  /// + You can call this API before and after joining room.
  /// + If you mirror the preview, or the preview and the published stream, the watermark will also be mirrored locally, but the published watermark will not be mirrored.
  /// + hen you enable simulcast mode, the watermark will be added to all video streams, and it will scale down to smaller encoding configurations accordingly.
  Future<int?> setVideoWatermark({
    StreamIndex streamIndex = StreamIndex.main,
    required String imagePath,
    required WatermarkConfig watermarkConfig,
  });

  /// Removes video watermark from designated video stream.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> clearVideoWatermark(
      [StreamIndex streamIndex = StreamIndex.main]);

  /// Takes a snapshot of the local video.
  ///
  /// [streamIndex]: Stream type.
  ///
  /// [filePath]: Save path for the snapshot.
  ///
  /// If the API takes effect, you will receive a [LocalSnapshot] object, otherwise you will receive the following error codes:
  /// + -1: Failure. Fails to generate the image.
  /// + -2: Failure. The stream is invalid.
  /// + -3: Failure. Snapshot timeout, default 1s.
  /// + -100: API call exception.
  /// + -101: No taskId returned.
  /// + -102: File write failed.
  /// + -103: Wrong image format.
  ///
  /// Note:
  /// + The snapshot is taken with all video effects on, like rotation, and mirroring.
  /// + You can take the snapshot either using SDK internal video capture or customized capture.
  CancelableOperation<LocalSnapshot> takeLocalSnapshot(
    StreamIndex streamIndex,
    String filePath,
  );

  /// Takes a snapshot of the remote video.
  ///
  /// [streamKey]: Information about the remote stream.
  ///
  /// [filePath]: Save path for the snapshot.
  ///
  /// If the API takes effect, you will receive a [RemoteSnapshot] object, otherwise you will receive the following error codes:
  /// + -1: Failure. Fails to generate the image.
  /// + -2: Failure. The stream is invalid.
  /// + -3: Failure. snapshot timeout, default 1s.
  /// + -100: API call exception.
  /// + -101: No taskId returned.
  /// + -102: File write failed.
  /// + -103: Wrong image format.
  CancelableOperation<RemoteSnapshot> takeRemoteSnapshot(
    RemoteStreamKey streamKey,
    String filePath,
  );

  /// Starts cloud proxy.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + Call this API before joining the room.
  /// + Pre-call network detection should be after the cloud proxy is turned on.
  /// + [RTCVideoEventHandler.onCloudProxyConnected] will be triggered after starting cloud proxy and successfully connecting the cloud proxy server.
  /// + Call [RTCVideo.stopCloudProxy] to stop cloud proxy.
  Future<int?> startCloudProxy(List<CloudProxyInfo> cloudProxiesInfo);

  /// Stops cloud proxy.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  Future<int?> stopCloudProxy();

  /// Creates a karaoke scoring API manager.
  FutureOr<RTCSingScoringManager?> getSingScoringManager();

  /// Sets an alternative image when the local internal video capture is not enabled.
  ///
  /// Call this API repeatedly can update the image.<br>
  /// You can set the path to null or open the camera to stop publishing the image.
  ///
  /// [filePath]: The path of the static image.  <br>
  /// You can use the absolute path (file://xxx) or the asset directory path (/assets/xx.png). The maximum size for the path is 512 bytes.  <br>
  /// You can upload a .JPG, .JPEG, .PNG, or .BMP file.  <br>
  /// When the aspect ratio of the image is inconsistent with the video encoder configuration, the image will be proportionally resized, with the remaining pixels rendered black. The framerate and the bitrate are consistent with the video encoder configuration.
  ///
  /// Return value:
  /// + 0: Success.
  /// + -1: Failure.
  ///
  /// Notes:
  /// + You cannot locally preview the image.
  /// + You can call this API before and after joining an RTC room. In the multi-room mode, the image can be only displayed in the room you publish the stream.
  /// + You cannot apply effects like filters and mirroring to the image, while you can watermark the image.
  /// + The image is not effective for a screen-sharing stream.
  /// + When you enable the simulcast mode, the image will be added to all video streams, and it will be proportionally scaled down to smaller encoding configurations.
  Future<int?> setDummyCaptureImagePath(String filePath);

  /// Obtains the synchronization network time information.
  ///
  /// Notes:
  /// + When you call this API for the first time, you starts synchronizing the network time information and receive the return value `0`. After the synchronization finishes, you will receive [RTCVideoEventHandler.onNetworkTimeSynchronized]. After that, calling this API will get you the correct network time.
  /// + Under chorus scenario, participants shall start audio mixing at the same network time.
  Future<NetworkTimeInfo?> getNetworkTimeInfo();

  /// On the listener side, sets all subscribed audio streams precisely timely aligned.
  ///
  /// [streamKey]: The remote audio stream used as the benchmark during time alignment.<br>
  /// We recommend you use the audio stream from the lead singer. <br>
  /// You must call this API after receiving [RTCRoomEventHandler.onUserPublishStream].
  ///
  /// [mode]: Whether to enable the alignment. Disabled by default.
  ///
  /// Return value:
  /// + `0`: Success.
  /// + `<0`: Failure. See [ReturnStatus] for the reason.
  ///
  /// Notes:
  /// + You must use the function when all participants set [RoomProfile] to `chorus` when joining the room.
  /// + All remote participants must call [RTCMediaPlayer.start] to play background music and set `syncProgressToRecordFrame` of [AudioMixingConfig] to `true`.
  /// + If the subscribed audio stream is delayed too much, it may not be precisely aligned.
  /// + The chorus participants must not enable the alignment. If you wish to change the role from listener to participant, you should disable the alignment.
  Future<int?> setAudioAlignmentProperty({
    required RemoteStreamKey streamKey,
    required AudioAlignmentMode mode,
  });

  /// @nodoc('For internal use')
  Future<int?> invokeExperimentalAPI(String param);

  /// Creates KTV manager.
  FutureOr<RTCKTVManager?> getKTVManager();

  /// Starts echo detection before joining a room.
  ///
  /// [testAudioFilePath]: Absolute path of the music file for the detection, which is expected to encode with UTF-8. The following files are supported: mp3, aac, m4a, 3gp, wav. <br>
  /// We recommend you assign a music file whose duration is between 10 to 20 seconds.<br>
  /// Do not pass a Silent file.
  ///
  /// Return value:
  /// + 0: Success.
  /// + -1: Failure due to the ongoing process of the previous detection. Call [stopHardwareEchoDetection] to stop it before calling this API again.
  /// + -2: Failure due to an invalid file path or file format.
  ///
  /// Notes:
  /// + You can use this feature only when [RoomProfile] is set to `meeting` or `meetingroom`.
  /// + Before calling this API, ask the user for the permissions to access the local audio devices.
  /// + Before calling this api, make sure the audio devices are activate and keep the capture volume and the playback volume within a reasonable range.
  /// + The detection result is passed as the argument of [RTCVideoEventHandler.onHardwareEchoDetectionResult].
  /// + During the detection, the SDK is not able to response to the other testing API such as [startEchoTest].
  /// + Call [stopHardwareEchoDetection] to stop the detection and release the audio devices.
  Future<int?> startHardwareEchoDetection(String testAudioFilePath);

  /// Stops the echo detection before joining a room.
  ///
  /// Return value:
  /// + 0: Success.
  /// + -1: Failure.
  ///
  /// Notes:
  /// + Refer to [startHardwareEchoDetection] for information on how to start a echo detection.
  /// + We recommend calling this API to stop the detection once getting the detection result from [RTCVideoEventHandler.onHardwareEchoDetectionResult].
  /// + You must stop the echo detection to release the audio devices before the user joins a room. Otherwise, the detection may interfere with the call.
  Future<int?> stopHardwareEchoDetection();

  /// Valid since v3.56.1
  ///
  /// Enables cellular network assisted communication to improve call quality.
  ///
  /// Return value:
  /// + 0: Success.
  /// + !0: Failure. See [ReturnStatus] for the reason.
  ///
  /// The function is off by default.
  Future<int?> setCellularEnhancement(MediaTypeEnhancementConfig config);

  /// Valid since v3.56.1
  ///
  /// Sets local proxy.
  ///
  /// [configurations]: Local proxy configurations.<br>
  /// You can set both Http tunnel and Socks5 as your local proxies, or only set one of them based on your needs. If you set both Http tunnel and Socks5 as your local proxies, then media traffic and signaling are routed through Socks5 proxy and Http requests through Http tunnel proxy. If you set either Http tunnel or Socks5 as your local proxy, then media traffic, signaling and Http requests are all routed through the proxy you chose.<br>
  /// If you want to remove the existing local proxy configurations, you can call this API with the parameter set to null.
  ///
  /// Notes:
  /// + You must call this API before joining the room.
  /// + After calling this API, you will receive [RTCVideoEventHandler.onLocalProxyStateChanged] callback informing you of the states of local proxy connection.
  Future<int?> setLocalProxy(List<LocalProxyConfiguration>? configurations);
}
