// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import '../src/base/bytertc_enum_convert.dart';
import 'bytertc_media_defines.dart';
import 'bytertc_video_api.dart';

/// Camera on a mobile device.
enum CameraId {
  /// Front-facing camera.
  front,

  /// Rear-facing camera (default setting).
  back,

  /// @nodoc('Not available')
  /// External camera.
  external
}

/// Video rendering scale mode.
enum VideoRenderMode {
  /// Fill and Crop.
  ///
  /// The video frame is scaled with fixed aspect ratio, until it completely fills the canvas. The region of the video exceeding the canvas will be cropped.
  hidden,

  /// Fit.
  ///
  /// The video frame is scaled with fixed aspect ratio, until it fits just within the canvas. Other part of the canvas is filled with the background color.
  fit,

  /// Fill the canvas.
  ///
  /// The video frame is scaled to fill the canvas. During the process, the aspect ratio may change.
  adaptive,
}

/// Mirror type.
enum MirrorType {
  /// Neither preview nor the published video stream is mirrored.
  none,

  /// The preview is mirrored. The published video stream is not mirrored.
  render,

  /// The preview and the published video stream are mirrored.
  renderAndEncoder,
}

/// Basic beauty effect.
enum EffectBeautyMode {
  /// Brightening.
  white,

  /// Smoothing.
  smooth,

  /// Sharpening.
  sharpen,

  /// Clarity.
  ///
  /// Valid since v3.56.1
  ///
  /// You need to integrate Effects SDK v4.4.2+ as a prerequisite for using this parameter.
  clear,
}

/// Video frame rotation angle.
enum VideoRotation {
  /// Video does not rotate (default setting).
  rotation0,

  /// Video rotates 90 degrees clockwise
  rotation90,

  /// Video rotates 180 degrees clockwise
  rotation180,

  /// Video rotates 270 degrees clockwise
  rotation270
}

/// Video frame scale mode.
enum ScaleMode {
  /// Automatic mode. The default value is `stretch`.
  auto,

  /// Stretch the video frame until the video frame and the window have the same resolution. The video frame's aspect ratio can be changed as it is automatically stretched to fill the window, but the whole image is visible.
  stretch,

  /// Fit the window with cropping。
  ///
  /// Scale the video frame uniformly until the window is filled. If the video frame's aspect ratio is different from that of the window, the extra part of the video frame will be cropped. <br>
  /// After the scaling process is completed, the width or height of the video frame will be consistent with that of the window, and the other dimension will be greater than or equal to that of the window.
  fitWithCropping,

  /// Fit the window with filling.
  ///
  /// Scale the video frame uniformly until its width or height reaches the boundary of the window. If the video frame's aspect ratio is different from that of the window, the area that is not filled will be black. <br>
  /// After the scaling process is completed, the width or height of the video frame will be consistent with that of the window, and the other dimension will be less than or equal to that of the window.
  fitWithFilling
}

/// Coding strategy preference.
enum VideoEncoderPreference {
  /// No preference.
  disable,

  /// Frame rate first.
  maintainFrameRate,

  /// Quality first.
  maintainQuality,

  /// Balances quality and frame rate.
  balance,
}

/// The encoding modes for shared-screen streams.
enum ScreenVideoEncoderPreference {
  /// The high frame rate mode, which ensure the highest frame rate possible under challenging network conditions.
  ///
  /// This mode is designed to share audiovisual content, including games and videos.
  maintainFrameRate,

  /// Default mode. The high-resolution mode, which ensure the highest resolution possible under challenging network conditions.
  ///
  /// This mode is designed to share micro-detailed content, including slides, documents, images, illustrations, or graphics.
  maintainQuality,
}

/// Type of media stream supposed to be paused/resume.
enum PauseResumeControlMediaType {
  /// Controls audio only.
  audio,

  /// Controls video only.
  video,

  /// Controls both audio and video.
  videoAndAudio,
}

/// Stream switching information when a local user-subscribed remote stream triggers a fallback policy.
class RemoteStreamSwitch {
  /// User ID of the publisher of the subscribed media stream.
  final String? uid;

  /// Whether it is a screen-sharing stream.
  final bool? isScreen;

  /// Index corresponding to the resolution of the subscribed video stream before the stream switch.
  final int? beforeVideoIndex;

  /// Index corresponding to the resolution of the subscribed video stream after the stream switch.
  final int? afterVideoIndex;

  /// Is there a video stream before switching.
  final bool? beforeEnable;

  /// Is there a video stream after switching.
  final bool? afterEnable;

  /// Reason of the fallback or recover of the stream.
  final FallbackOrRecoverReason? reason;

  /// @nodoc
  const RemoteStreamSwitch({
    this.uid,
    this.isScreen,
    this.beforeVideoIndex,
    this.afterVideoIndex,
    this.beforeEnable,
    this.afterEnable,
    this.reason,
  });

  /// @nodoc
  factory RemoteStreamSwitch.fromMap(Map<dynamic, dynamic> map) {
    return RemoteStreamSwitch(
      uid: map['uid'],
      isScreen: map['isScreen'],
      beforeVideoIndex: map['beforeVideoIndex'],
      afterVideoIndex: map['afterVideoIndex'],
      beforeEnable: map['beforeEnable'],
      afterEnable: map['afterEnable'],
      reason: (map['reason'] as int).fallbackOrRecoverReason,
    );
  }
}

/// Stream mixing content type.
enum TranscoderContentControlType {
  /// The mixed stream contains audio and video.
  hasAudioAndVideo,

  /// The mixed stream only contains audio.
  hasAudioOnly,

  /// The mixed stream only contains video.
  hasVideoOnly,
}

/// Performance fallback related data.
class SourceWantedData {
  /// If the performance fallback on the publisher side is not enabled, this value represents the recommended video input width.<br>
  /// Otherwise, this value represents the maximum width of the current push stream.
  final int? width;

  /// If the performance fallback on the publisher side is not enabled, this value represents the recommended video input height.<br>
  /// Otherwise, this value represents the maximum height of the current push stream.
  final int? height;

  /// If the performance fallback on the publisher side is not enabled, this value represents the recommended video input frame rate in fps.<br>
  /// Otherwise, this value represents the maximum frame rate of the current push stream.
  final int? frameRate;

  /// @nodoc
  const SourceWantedData({
    this.width,
    this.height,
    this.frameRate,
  });

  /// @nodoc
  factory SourceWantedData.fromMap(Map<dynamic, dynamic> map) {
    return SourceWantedData(
      width: map['width'],
      height: map['height'],
      frameRate: map['frameRate'],
    );
  }
}

/// Subscription related configuration.
class SubscribeConfig {
  /// Whether the stream is from screen sharing.
  final bool? isScreen;

  /// Whether video stream is included.
  final bool? subVideo;

  /// Whether audio stream is included.
  final bool? subAudio;

  /// The label of subscribed video stream with different resolution.
  ///
  /// Users can call [RTCVideo.setVideoEncoderConfig] to publish multiple video streams of different resolutions. Therefore, when subscribing to a remote stream, you need to specify the specific resolution of the subscription. This parameter is used to specify which resolution you want to subscribe, the default value is 0.
  final int? videoIndex;

  /// Width of the video stream in px.
  final int? subWidth;

  /// Height of the video stream in px.
  final int? subHeight;

  /// @nodoc('Not available')
  final int? subVideoIndex;

  /// Time domain hierarchy of the subscribed video stream, default value is 0.
  ///
  /// Not available for Android.
  final int? svcLayer;

  /// Expected maximum frame rate of the subscribed stream in px. The default value is 0, values greater than 10 are valid.
  ///
  /// If the frame rate of the stream published is lower than the value you set, or if your subscribed stream falls back under limited network conditions, the frame rate you set will drop accordingly. <br>
  /// Only valid if the stream is coded with SVC technique.
  final int? frameRate;

  /// @nodoc
  const SubscribeConfig({
    this.isScreen,
    this.subVideo,
    this.subAudio,
    this.videoIndex,
    this.subWidth,
    this.subHeight,
    this.subVideoIndex,
    this.svcLayer,
    this.frameRate,
  });

  /// @nodoc
  factory SubscribeConfig.fromMap(Map<dynamic, dynamic> map) {
    return SubscribeConfig(
      isScreen: map['isScreen'],
      subVideo: map['subVideo'],
      subAudio: map['subAudio'],
      videoIndex: map['videoIndex'],
      subWidth: map['subWidth'],
      subHeight: map['subHeight'],
      subVideoIndex: map['subVideoIndex'],
      svcLayer: map['svcLayer'],
      frameRate: map['frameRate'],
    );
  }
}

/// Virtual background types.
enum VirtualBackgroundSourceType {
  /// Replace the background with a solid color.
  color,

  /// Replace the background with the specified image.
  image,
}

/// Virtual background source.
class VirtualBackgroundSource {
  /// Virtual background types.
  VirtualBackgroundSourceType sourceType;

  /// The solid color of the background. The format is 0xAARRGGBB.
  int sourceColor;

  /// The absolute path of the specified image.
  ///
  /// + You can use the absolute path of the local file (file://xxx) and the Asset path (asset://xxx).
  /// + You can upload a .JPG, .PNG, or .JPEG file.
  /// + The image with a resolution higher than 1080p(Full HD) will be rescaled proportionally to fit in the video.
  /// + If the image's aspect ratio matches the video's, the image will be rescaled proportionally to fit in the video.  <br>
  /// + If the image’s aspect ratio doesn't match the video's, the shortest side (either height or width) of the image will be stretched proportionally to match the video. Then the image will be cropped to fill in the video.  <br>
  /// + The transparent area in the image will be filled with white color.
  String sourcePath;

  /// @nodoc
  VirtualBackgroundSource({
    required this.sourceType,
    this.sourceColor = 0,
    this.sourcePath = '',
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'sourceType': sourceType.index,
      'sourceColor': sourceColor,
      'sourcePath': sourcePath,
    };
  }
}

/// Video frame information。
class VideoFrameInfo {
  /// Width in px.
  final int? width;

  /// Height in px.
  final int? height;

  /// Video frame clockwise rotation angle.
  final VideoRotation? rotation;

  /// @nodoc
  const VideoFrameInfo({
    this.width,
    this.height,
    this.rotation,
  });

  /// @nodoc
  factory VideoFrameInfo.fromMap(Map<dynamic, dynamic> map) {
    return VideoFrameInfo(
      width: map['width'],
      height: map['height'],
      rotation: (map['rotation'] as int).videoRotation,
    );
  }
}

/// Video capture preference.
enum VideoCapturePreference {
  /// Automatically set capture parameters (default setting).
  ///
  /// SDK determines the best video capture parameters referring to the camera output parameters and the encoder configuration.
  auto,

  /// Manually set capture parameters, including resolution and frame rate.
  manual,

  /// The capture parameters are consistent with the encoding parameters.
  autoPerformance,
}

/// Video capture configuration.
class VideoCaptureConfig {
  /// Video capture preference.
  VideoCapturePreference capturePreference;

  /// Width in px.
  int width = 0;

  /// Height in px.
  int height = 0;

  /// Video capture frame rate in fps.
  int frameRate = 0;

  /// @nodoc
  VideoCaptureConfig({
    this.capturePreference = VideoCapturePreference.auto,
    this.width = 0,
    this.height = 0,
    this.frameRate = 0,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'capturePreference': capturePreference.index,
      'width': width,
      'height': height,
      'frameRate': frameRate,
    };
  }
}

/// Current audio device type.
enum VideoDeviceType {
  /// Unknown device type.
  unknown,

  /// Audio rendering device type.
  renderDevice,

  /// Audio capturing device type.
  captureDevice,

  /// Screen stream video device.
  screenCaptureDevice,
}

/// Video orientation.
enum VideoRotationMode {
  /// Follow the orientation of the App.
  followApp,

  /// Follow the gravity.
  followGSensor,
}

/// Orientation of the video frame.
enum VideoOrientation {
  /// The original orientation of the captured video frame. Default setting.
  adaptive,

  /// Portrait mode. Adjust the captured video frame to vertical layout in RTC SDK.
  portrait,

  /// Landscape mode. Adjust the captured video frame to horizontal layout in RTC SDK.
  landscape,
}

/// Camera flash status.
enum TorchState {
  /// Off.
  off,

  /// On.
  on,
}

/// Media stream type.
enum MediaStreamType {
  /// Controls audio only.
  audio,

  /// Controls video only.
  video,

  /// Controls both audio and video.
  both,
}

/// Description of video stream parameters.
class VideoEncoderConfig {
  /// Video width in px.
  int width;

  /// Height width in px.
  int height;

  /// Video frame rate in fps.
  int frameRate;

  /// Maximum bit rate in kbps.
  ///
  /// If you set this value to -1, SDK will automatically calculate the applicable bit rate based on the input resolution and frame rate.<br>
  /// No stream will be encoded and published if you set this value to 0.<br>
  /// In 344 and later versions, the default value is -1 for internal capturing, but no default value in versions earlier than 344.
  int maxBitrate;

  /// Minimum video encoding bitrate in kbps, which defaults to `0`. The encoding bitrate will not be lower than the `minBitrate`.
  ///
  /// The value ranges within `[0, maxBitrate)`. When `maxBitrate` < `minBitrate`, the bitrate is self-adapted.
  ///
  /// In the following circumstance, the assignment to this variable has no effect:
  /// + When `maxBitrate` = `0`, the video encoding is disabled.
  /// + When `maxBitrate` < `0`, the bitrate is self-adapted.
  int minBitrate;

  /// Coding policy preference.
  VideoEncoderPreference encoderPreference;

  /// @nodoc
  VideoEncoderConfig({
    required this.width,
    required this.height,
    required this.frameRate,
    this.maxBitrate = -1,
    this.minBitrate = 0,
    this.encoderPreference = VideoEncoderPreference.maintainFrameRate,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'width': width,
      'height': height,
      'frameRate': frameRate,
      'maxBitrate': maxBitrate,
      'minBitrate': minBitrate,
      'encoderPreference': encoderPreference.index,
    };
  }
}

/// The encoding configuration for shared-screen streams. See [Setting Video Encoder Configuration](https://docs.byteplus.com/byteplus-rtc/docs/70122).
class ScreenVideoEncoderConfig {
  /// Video width in px.
  int width;

  /// Video height in px.
  int height;

  /// Video frame rate in fps.
  int frameRate;

  /// Maximum bit rate in kbps. Optional for internal capturing while mandatory for custom capturing.
  ///
  /// The default value is -1, which means RTC will automatically recommend the bitrate based on the input resolution and frame rate.<br>
  /// If you set this value to 0, the streams will not be encoded and published.<br>
  /// No stream will be encoded and published if you set this parameter to 0.
  int maxBitrate;

  /// The minimum bitrate in kbps. Optional for internal capture while mandatory for custom capture.
  ///
  /// The minimum bitrate must be set lower than the maximum bitrate. Otherwise, the streams will not be encoded and published.
  int minBitrate;

  /// The encoding modes for shared-screen streams.
  ScreenVideoEncoderPreference encoderPreference;

  /// @nodoc
  ScreenVideoEncoderConfig({
    required this.width,
    required this.height,
    required this.frameRate,
    this.maxBitrate = -1,
    this.minBitrate = 0,
    this.encoderPreference = ScreenVideoEncoderPreference.maintainFrameRate,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'width': width,
      'height': height,
      'frameRate': frameRate,
      'maxBitrate': maxBitrate,
      'minBitrate': minBitrate,
      'encoderPreference': encoderPreference.value,
    };
  }
}

/// Information on remote video frame.
class RemoteVideoConfig {
  /// Expected maximum frame rate of the subscribed stream in px. The default value is 0, which represents full-frame-rate, values greater than 0 are valid.
  ///
  /// If the frame rate of the stream published is lower than the value you set, or if your subscribed stream falls back under limited network conditions, the frame rate you set will drop accordingly.<br>
  /// Only valid if the stream is coded with SVC technique.
  int frameRate;

  /// Video width in px.
  int width;

  /// Video height in px.
  int height;

  /// @nodoc
  RemoteVideoConfig({
    this.frameRate = 0,
    this.width = 0,
    this.height = 0,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'frameRate': frameRate,
      'width': width,
      'height': height,
    };
  }
}

/// Room configuration.
class RoomConfig {
  /// Room profile.
  ///
  /// The default is `communication`.<br>
  /// The setting cannot be changed after joining the room.
  RoomProfile profile;

  /// Whether to publish media streams automatically. The default is automatic publishing.
  ///
  /// Invisible user cannot publish media streams regardless of the value of `isAutoPublish`.
  /// Only one of the rooms the user joined can be set to auto-publish. If no settings are made in each room, the stream is automatically published in the first room joined by default.
  bool isAutoPublish;

  /// Whether to automatically subscribe to the audio stream. The default is automatic subscription.
  bool isAutoSubscribeAudio;

  /// Whether to automatically subscribe to the main video stream. The default is automatic subscription.
  bool isAutoSubscribeVideo;

  /// Expected configuration of remote video stream.
  RemoteVideoConfig? remoteVideoConfig;

  /// @nodoc
  RoomConfig({
    this.profile = RoomProfile.communication,
    this.isAutoPublish = true,
    this.isAutoSubscribeAudio = true,
    this.isAutoSubscribeVideo = true,
    this.remoteVideoConfig,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'profile': profile.index,
      'isAutoPublish': isAutoPublish,
      'isAutoSubscribeAudio': isAutoSubscribeAudio,
      'isAutoSubscribeVideo': isAutoSubscribeVideo,
      'remoteVideoConfig': remoteVideoConfig != null
          ? remoteVideoConfig!.toMap()
          : RemoteVideoConfig().toMap(),
    };
  }
}

/// Locally recorded media type.
enum RecordingType {
  /// Audio only
  audioOnly,

  /// Video only
  videoOnly,

  /// Record audio and video simultaneously
  videoAndAudio,
}

/// Screen media type.
enum ScreenMediaType {
  /// Capture video only.
  videoOnly,

  /// Capture audio only.
  audioOnly,

  /// Capture both audio and video.
  videoAndAudio,
}

/// Watermark's scaled coordinates and size, relative to the video stream.
class Watermark {
  /// The watermark's horizontal offset from the upper left corner of the video stream to the video stream's width in range of `[0,1)`.
  final double x;

  /// The watermark's vertical offset from the upper left corner of the video stream to the video stream's height in range of `[0,1)`.
  final double y;

  /// The watermark's width to the video stream's width in range of `[0,1)`.
  final double width;

  /// The watermark height to the video stream's height in range of `[0,1)`.
  final double height;

  /// @nodoc
  const Watermark({
    this.x = 0,
    this.y = 0,
    this.width = 0,
    this.height = 0,
  });

  /// @nodoc
  const Watermark.none() : this();

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'x': x,
      'y': y,
      'width': width,
      'height': height,
    };
  }
}

/// Watermark configurations.
class WatermarkConfig {
  /// Whether the watermark is visible in preview. The default value is `true`.
  bool visibleInPreview;

  /// Watermark's coordinates and size in landscape mode.
  Watermark positionInLandscapeMode;

  /// Watermark's coordinates and size in portrait mode.
  Watermark positionInPortraitMode;

  /// @nodoc
  WatermarkConfig({
    this.visibleInPreview = true,
    this.positionInLandscapeMode = const Watermark.none(),
    this.positionInPortraitMode = const Watermark.none(),
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'visibleInPreview': visibleInPreview,
      'positionInLandscapeMode': positionInLandscapeMode.toMap(),
      'positionInPortraitMode': positionInPortraitMode.toMap(),
    };
  }
}

/// Crop configuration of the public stream.
class SourceCrop {
  /// The normalized scale of the upper-left corner of the cropped video frame relative to the overall frame before cropping, in the range `[0.0, 1.0)`.
  double locationX;

  /// The normalized scale of the upper-left corner of the cropped video frame relative to the overall frame before cropping, in the range `[0.0, 1.0)`.
  double locationY;

  /// The normalized ratio of the width of the video frame obtained after cropping to the overall frame before cropping, in the range `(0.0, 1.0]`.
  double widthProportion;

  /// The normalized ratio of the height of the obtained video frame after cropping to the overall frame before cropping, in the range `(0.0, 1.0]`.
  double heightProportion;

  /// @nodoc
  SourceCrop({
    this.locationX = 0,
    this.locationY = 0,
    this.widthProportion = 1,
    this.heightProportion = 1,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'locationX': locationX,
      'locationY': locationY,
      'widthProportion': widthProportion,
      'heightProportion': heightProportion,
    };
  }
}

/// Layout of each stream to be mixed into the public stream.
class PublicStreamingRegion {
  /// Required. ID of the user who publishes the video stream.
  String uid;

  /// Required. ID of the room from which the stream is pushed.
  String roomId;

  /// Background image address.
  String alternateImage;

  /// Required. The normalized horizontal coordinate value of the top left end vertex of the user's view, ranging within `[0.0, 1.0).
  double x;

  /// Required. The normalized vertical coordinate value of the top left end vertex of the user's view, ranging within `[0.0, 1.0).
  double y;

  /// Required. The normalized width of the user's view, ranging within `(0.0, 1.0]`.
  double w;

  /// Required. The normalized height of the user's view, ranging within `(0.0, 1.0]`.
  double h;

  /// The layer order of the video in the stacks when Regions overlap with each other.
  ///
  /// `[0,100]`, 0 for the bottom, by default. 100 for the top.
  int zorder;

  /// Required. Transparency in a range of `[0.0, 1.0]`.
  double alpha;

  /// Required. Public media stream type.
  StreamIndex streamType;

  /// Required. Public media stream content.
  TranscoderContentControlType mediaType;

  /// Required. Video render mode.
  VideoRenderMode renderMode;

  /// Relative position and size of the user video layout.
  SourceCrop sourceCrop;

  /// @nodoc
  PublicStreamingRegion({
    required this.uid,
    required this.roomId,
    this.alternateImage = '',
    required this.x,
    required this.y,
    required this.w,
    required this.h,
    this.zorder = 0,
    this.alpha = 1.0,
    this.streamType = StreamIndex.main,
    this.mediaType = TranscoderContentControlType.hasAudioAndVideo,
    this.renderMode = VideoRenderMode.hidden,
    required this.sourceCrop,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'uid': uid,
      'roomId': roomId,
      'alternateImage': alternateImage,
      'x': x,
      'y': y,
      'w': w,
      'h': h,
      'zorder': zorder,
      'alpha': alpha,
      'streamType': streamType.index,
      'mediaType': mediaType.index,
      'renderMode': renderMode.value,
      'sourceCrop': sourceCrop.toMap()
    };
  }
}

/// Public stream layout information.
class PublicStreamingLayout {
  /// Layout of each stream to be mixed into the public stream.
  List<PublicStreamingRegion> regions;

  /// The interpolation mode:
  /// + `0`: Insert with the last frame.
  /// + `1`: Insert with the background image. RTC will insert frames of black if no background image has been set.
  int interpolationMode;

  /// Public stream layout mode. `2`: Custom mode.
  int layoutMode;

  /// The address of the background image of the canvas.
  String backgroundImage;

  /// Background color of the canvas in RGB hex value. For example, #FFB6C1 represents light pink. The default value is #000000 (black).
  String backgroundColor;

  /// @nodoc
  PublicStreamingLayout({
    required this.regions,
    this.interpolationMode = 0,
    this.layoutMode = 2,
    this.backgroundImage = '',
    this.backgroundColor = '#000000',
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'regions': regions.map((e) => e.toMap()).toList(),
      'interpolationMode': interpolationMode,
      'layoutMode': layoutMode,
      'backgroundImage': backgroundImage,
      'backgroundColor': backgroundColor,
    };
  }
}

/// Video parameters for public streams.
class PublicStreamingVideoConfig {
  /// Required. Frame rate in fps within a range of `[1, 60]`.
  int fps;

  /// Required. Bitrate in bps within a range of `[1,10000000]`.
  int kBitrate;

  /// Required. Width in px within a range of `[16, 1920]`. It defaults to 360px and must be even.
  int width;

  /// Required. Width in px within a range of `[16, 1280]`. It defaults to 640px and must be even.
  int height;

  /// @nodoc
  PublicStreamingVideoConfig({
    this.fps = 30,
    this.kBitrate = 500,
    this.width = 360,
    this.height = 640,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'fps': fps,
      'kBitrate': kBitrate,
      'width': width,
      'height': height,
    };
  }
}

/// Audio parameters for public streams.
class PublicStreamingAudioConfig {
  /// Required. Bitrate. Positive integer in kbps. Options: 16, 32, or 64.
  int kBitrate;

  /// Required. Sample rate in Hz. Options: 16000, 32000, 44100, or 48000.
  int sampleRate;

  /// Required. Number of the audio channels:
  /// + `1`: Mono channel by default.
  /// + `2`: Dual channel.
  int channels;

  /// @nodoc
  PublicStreamingAudioConfig({
    this.kBitrate = 16,
    this.sampleRate = 44100,
    this.channels = 1,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'kBitrate': kBitrate,
      'sampleRate': sampleRate,
      'channels': channels,
    };
  }
}

/// Public stream configuration.
class PublicStreaming {
  /// @nodoc('Not available')
  /// ID of the room from which the public stream is pushed.
  String roomId;

  /// Required. Video encoder configuration.
  PublicStreamingVideoConfig video;

  /// @nodoc('Not available')
  /// Required. Audio encoder configuration.
  PublicStreamingAudioConfig audio;

  /// Required. Public stream layout.
  PublicStreamingLayout layout;

  /// @nodoc
  PublicStreaming({
    required this.roomId,
    required this.video,
    required this.audio,
    required this.layout,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'roomId': roomId,
      'video': video.toMap(),
      'audio': audio.toMap(),
      'layout': layout.toMap(),
    };
  }
}

/// Result of the local snapshot.
class LocalSnapshot {
  /// Snapshot task ID.
  int taskId;

  /// Stream type.
  final StreamIndex streamIndex;

  /// Screenshot file save path.
  final String filePath;

  /// Image width.
  int width;

  /// Image height.
  int height;

  /// @nodoc
  LocalSnapshot({
    this.taskId = 0,
    required this.streamIndex,
    required this.filePath,
    this.width = 0,
    this.height = 0,
  });
}

/// Result of the remote snapshot.
class RemoteSnapshot {
  /// Snapshot task ID.
  int taskId;

  /// Stream information.
  final RemoteStreamKey streamKey;

  /// Screenshot file save path.
  final String filePath;

  /// Image width.
  int width;

  /// Image height.
  int height;

  /// @nodoc
  RemoteSnapshot({
    this.taskId = 0,
    required this.streamKey,
    required this.filePath,
    this.width = 0,
    this.height = 0,
  });
}

/// Digital Zoom type。
enum ZoomConfigType {
  /// Sets ZoomIn/ZoomOut config
  focusOffset,

  /// Sets move config
  moveOffset,
}

/// Action of the digital zoom control.
enum ZoomDirectionType {
  /// Move to the left.
  cameraMoveLeft,

  /// Move to the right.
  cameraMoveRight,

  /// Move upwards.
  cameraMoveUp,

  /// Move downwards.
  cameraMoveDown,

  /// Zoom out.
  cameraZoomOut,

  /// Zoom in.
  cameraZoomIn,

  /// Reset digital zoom.
  cameraReset,
}

/// Media type for cellular assisted Enhancement.
class MediaTypeEnhancementConfig {
  /// Apply to signaling or not. Not by default.
  bool enhanceSignaling;

  /// Apply to audio stream (Screen-sharing audio not included) or not. Not by default.
  bool enhanceAudio;

  /// Apply to video (screen-sharing video not included) or not. Not by default.
  bool enhanceVideo;

  /// Apply to screen sharing audio or not. Not by default.
  bool enhanceScreenAudio;

  /// Apply to screen sharing video or not. Not by default.
  bool enhanceScreenVideo;

  /// @nodoc
  MediaTypeEnhancementConfig({
    this.enhanceSignaling = false,
    this.enhanceAudio = false,
    this.enhanceVideo = false,
    this.enhanceScreenAudio = false,
    this.enhanceScreenVideo = false,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'enhanceSignaling': enhanceSignaling,
      'enhanceAudio': enhanceAudio,
      'enhanceVideo': enhanceVideo,
      'enhanceScreenAudio': enhanceScreenAudio,
      'enhanceScreenVideo': enhanceScreenVideo,
    };
  }
}

/// @nodoc('Not available')
/// The reason for the change in super resolution mode.
enum VideoSuperResolutionModeChangedReason {
  /// Successfully turned off the super resolution mode by calling [RTCVideo.setRemoteVideoSuperResolution].
  apiOff,

  /// Successfully turned on the super resolution mode by calling [RTCVideo.setRemoteVideoSuperResolution].
  apiOn,

  /// Failed to turn on super-resolution mode. The original resolution of the remote video stream should not exceed 640 × 360 pixels.
  resolutionExceed,

  /// Failed to turn on super-resolution mode. You can only turn on super-resolution mode for one stream.
  overUse,

  /// Incompatible device for super resolution.
  deviceNotSupport,

  /// The super-resolution mode is turned off because of lacking device capabilities.
  dynamicClose,

  /// The super-resolution mode is turned off for other reasons.
  otherSettingDisabled,

  /// The super-resolution mode is turned on for other reasons.
  otherSettingEnabled,

  /// The super-resolution mode is not compiled in the SDK.
  noComponent,

  /// The remote stream does not exist. Reasons include invalid room ID, user ID, or the stream is not published.
  streamNotExist,
}

/// @nodoc('Not available')
/// The reasons for the change in the video noise reduction mode.
enum VideoDenoiseModeChangedReason {
  /// Video noise reduction mode changed due to unknown reasons.
  unknown,

  /// Successfully turned off video noise reduction by calling [RTCVideo.setVideoDenoiser].
  apiOff,

  /// Successfully turned on the video noise reduction mode by calling [RTCVideo.setVideoDenoiser].
  apiOn,

  /// Video noise reduction disabled by configuration.
  configDisabled,

  /// Video noise reduction enabled by configuration.
  configEnabled,

  /// Video noise reduction turned off due to internal exceptions.
  internalException,

  /// Video noise reduction turned off due to hardware performance stress.
  dynamicClose,

  /// Device hardware capacity is sufficient, video noise reduction turned on.
  dynamicOpen,

  /// Video noise reduction mode changed due to video resolution. High resolution leads to high hardware utilization. Reduce the resolution to recover video noise reduction.
  resolution,
}
