// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'bytertc_face_detection.dart';
import 'bytertc_video_defines.dart';

/// Advanced video effects.
abstract class RTCVideoEffect {
  /// Checks video effect license, sets the video effect resource model path, and initializes video effect.
  ///
  /// [licenseFile]: The absolute path of the license file for authorization.
  ///
  /// [modelPath]: The absolute path of the Effects SDK's models file.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  Future<int?> initCVResource(String licenseFile, String modelPath);

  /// Enables video effects including beauty and color filters.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1001: This API is unavailable for your Effects SDK.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  ///
  /// Notes:
  /// + You must call [initCVResource] before calling this API.
  /// + This API does not turn on video effects directly, you must call [setEffectNodes] or [setColorFilter] next.
  /// + Call [disableVideoEffect] to turn off video effects.
  Future<int?> enableVideoEffect();

  /// Disables video effects.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1001: This API is unavailable for your Effects SDK.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  ///
  /// Call [enableVideoEffect] to enable video effects.
  Future<int?> disableVideoEffect();

  /// Sets the video effects material package.
  ///
  /// [effectNodes]: Array of effect material package paths. <br>
  /// To remove the current video effect, set it to null.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1001: This API is unavailable for your Effects SDK.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  ///
  /// You must call [enableVideoEffect] before calling this API.
  Future<int?> setEffectNodes(List<String>? effectNodes);

  /// Sets the intensity of video effects.
  ///
  /// [effectNode]: The absolute path of the effects resource package, see [Resource Package Structure](https://docs.byteplus.com/effects/docs/resource-package-structure-v421-and-later).
  ///
  /// [key]: The name of the material key to be set, see [Functions of Resource Keys](https://docs.byteplus.com/effects/docs/functions-of-resource-keys-v421-and-later) for the value.
  ///
  /// [value]: The intensity value that needs to be set, the value range `[0,1]`, and the setting is invalid when it exceeds the range.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1001: This API is unavailable for your Effects SDK.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  Future<int?> updateEffectNode({
    required String effectNode,
    required String key,
    required double value,
  });

  /// Sets the color filter.
  ///
  /// [resFile]: Filter effects package absolute path.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1001: This API is unavailable for your Effects SDK.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  ///
  /// Call [setColorFilterIntensity] to set the intensity of the color filter enabled. Set the intensity to 0 to turn off color filter.
  Future<int?> setColorFilter(String? resFile);

  /// Sets the intensity of the color filter enabled.
  ///
  /// [intensity]: Filter intensity. The value range `[0,1]` is set to be invalid when the range is exceeded.<br>
  /// Set the intensity to 0 to turn off color filter.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1001: This API is unavailable for your Effects SDK.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  Future<int?> setColorFilterIntensity(double intensity);

  /// Sets the original background to a specified image or a solid color.
  ///
  /// [modelPath]: The absolute path of virtual background effects.
  ///
  /// [source]: Virtual background source.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1001: This API is unavailable for your Effects SDK.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  ///
  /// Notes:
  /// + You must call [initCVResource] before calling this API.
  /// + Call [disableVirtualBackground] to turn off the virtual background.
  Future<int?> enableVirtualBackground({
    required String modelPath,
    required VirtualBackgroundSource source,
  });

  /// Turns off the virtual background.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1001: This API is unavailable for your Effects SDK.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  ///
  /// After calling [enableVirtualBackground] to enable the virtual background function, you can call this API to turn it off.
  Future<int?> disableVirtualBackground();

  /// Starts face detection and registers the observer for the result.<br>
  /// With this observer, you will receive [RTCFaceDetectionObserver.onFaceDetectResult] periodically.
  ///
  /// [interval]: The minimum time interval between two callbacks in milliseconds. The value should be greater than 0. The actual time interval is between interval_ms and interval_ms+the time slot of a captured video frame.
  ///
  /// [modelPath]: The absolute path of the face detection algorithm file. Typically it is the tt_face_vXXX.model file in the ttfacemodel folder.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1001: This API is unavailable for your Effects SDK.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + -1004: Initializing. This function will be available when the initialization is completed.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  Future<int?> enableFaceDetection({
    required RTCFaceDetectionObserver observer,
    required String modelPath,
    int interval = 0,
  });

  /// Stops face detection.
  ///
  /// Return value:
  /// + 0: Success.
  /// + –1000: The Effects SDK is not integrated.
  /// + –1001: This API is unavailable for your Effects SDK.
  /// + –1002: Your Effects SDK's version is incompatible.
  /// + < 0: Other error. See [error code table](https://docs.byteplus.com/effects/docs/error-code-table) for specific instructions.
  Future<int?> disableFaceDetection();
}
