// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

// ignore_for_file: public_member_api_docs
import '../api/bytertc_ktv_defines.dart';
import '../api/bytertc_ktv_manager_api.dart';
import '../api/bytertc_ktv_player_api.dart';
import 'base/bytertc_enum_convert.dart';

extension RTCKTVPlayerEventProcessor on RTCKTVPlayerEventHandler {
  void process(String methodName, Map<dynamic, dynamic> dic) {
    switch (methodName) {
      case 'onPlayProgress':
        final data = OnPlayProgressData.fromMap(dic);
        onPlayProgress?.call(data.musicId, data.progress);
        break;
      case 'onPlayStateChanged':
        final data = OnPlayStateChangedData.fromMap(dic);
        onPlayStateChanged?.call(data.musicId, data.playState, data.error);
        break;
      default:
        break;
    }
  }
}

class OnPlayProgressData {
  final String musicId;
  final int progress;

  const OnPlayProgressData({required this.musicId, required this.progress});

  factory OnPlayProgressData.fromMap(Map<dynamic, dynamic> map) {
    return OnPlayProgressData(
        musicId: map['musicId'], progress: map['progress']);
  }
}

class OnPlayStateChangedData {
  final String musicId;
  final PlayState playState;
  final KTVPlayerErrorCode error;

  const OnPlayStateChangedData({
    required this.musicId,
    required this.playState,
    required this.error,
  });

  factory OnPlayStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnPlayStateChangedData(
      musicId: map['musicId'],
      playState: (map['playState'] as int?).ktvPlayState,
      error: (map['error'] as int?).ktvPlayerError,
    );
  }
}

extension RTCKTVManagerEventProcessor on RTCKTVManagerEventHandler {
  void process(String methodName, Map<dynamic, dynamic> dic) {
    switch (methodName) {
      case 'onMusicListResult':
        final data = OnMusicListResultData.fromMap(dic);
        onMusicListResult?.call(data.musics, data.totalSize, data.error);
        break;
      case 'onSearchMusicResult':
        final data = OnMusicListResultData.fromMap(dic);
        onSearchMusicResult?.call(data.musics, data.totalSize, data.error);
        break;
      case 'onHotMusicResult':
        final data = OnHotMusicResultData.fromMap(dic);
        onHotMusicResult?.call(data.hotMusics, data.error);
        break;
      case 'onMusicDetailResult':
        final data = OnMusicDetailResultData.fromMap(dic);
        onMusicDetailResult?.call(data.music, data.error);
        break;
      case 'onClearCacheResult':
        final data = OnClearCacheResultData.fromMap(dic);
        onClearCacheResult?.call(data.error);
        break;
      default:
        break;
    }
  }
}

class OnMusicListResultData {
  final KTVErrorCode error;
  final int totalSize;
  final List<MusicInfo>? musics;

  const OnMusicListResultData({
    required this.error,
    required this.totalSize,
    this.musics,
  });

  factory OnMusicListResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnMusicListResultData(
      error: (map['error'] as int?).ktvError,
      totalSize: map['totalSize'],
      musics: (map['musicInfos'] as List?)
          ?.map((e) => MusicInfo.fromMap(e))
          .toList(growable: false),
    );
  }
}

class OnHotMusicResultData {
  final KTVErrorCode error;
  final List<HotMusicInfo>? hotMusics;

  const OnHotMusicResultData({
    required this.error,
    this.hotMusics,
  });

  factory OnHotMusicResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnHotMusicResultData(
      error: (map['error'] as int?).ktvError,
      hotMusics: (map['hotMusics'] as List?)
          ?.map((e) => HotMusicInfo.fromMap(e))
          .toList(growable: false),
    );
  }
}

class OnMusicDetailResultData {
  final KTVErrorCode error;
  final MusicInfo? music;

  const OnMusicDetailResultData({
    required this.error,
    this.music,
  });

  factory OnMusicDetailResultData.fromMap(Map<dynamic, dynamic> map) {
    MusicInfo? music;
    Map<dynamic, dynamic>? m = map['musicInfo'];
    if (m != null) {
      music = MusicInfo.fromMap(m);
    }
    return OnMusicDetailResultData(
      error: (map['error'] as int?).ktvError,
      music: music,
    );
  }
}

class OnClearCacheResultData {
  final KTVErrorCode error;

  const OnClearCacheResultData({
    required this.error,
  });

  factory OnClearCacheResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnClearCacheResultData(
      error: (map['error'] as int?).ktvError,
    );
  }
}
