// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

// ignore_for_file: public_member_api_docs
import 'dart:typed_data';

import '../api/bytertc_audio_defines.dart';
import '../api/bytertc_event_define.dart';
import '../api/bytertc_media_defines.dart';
import '../api/bytertc_rts_defines.dart';
import '../api/bytertc_video_defines.dart';
import '../api/bytertc_video_event_handler.dart';
import 'base/bytertc_enum_convert.dart';
import 'base/bytertc_event_serialize.dart';

class RTCVideoEventValue {
  RTCVideoEventValue({
    OnSysStatsType? onSysStats,
  }) {
    this.onSysStats = onSysStats;
  }

  void Function(Map<String, dynamic>)? _valueObserver;

  set valueObserver(void Function(Map<String, dynamic>)? valueObserver) {
    _valueObserver = valueObserver;
    if (valueObserver == null) return;
    _valueObserver?.call({'enableSysStats': onSysStats != null});
  }

  OnSysStatsType? _onSysStats;

  OnSysStatsType? get onSysStats => _onSysStats;

  set onSysStats(OnSysStatsType? onSysStats) {
    _onSysStats = onSysStats;
    _valueObserver?.call({'enableSysStats': onSysStats != null});
  }
}

extension RTCVideoEventProcessor on RTCVideoEventHandler {
  void process(String methodName, Map<dynamic, dynamic> dic) {
    switch (methodName) {
      case 'onWarning':
        final data = OnWarningData.fromMap(dic);
        onWarning?.call(data.code);
        break;
      case 'onError':
        final data = OnErrorData.fromMap(dic);
        onError?.call(data.code);
        break;
      case 'onCreateRoomStateChanged':
        final data = OnCreateRoomStateChangedData.fromMap(dic);
        onCreateRoomStateChanged?.call(data.roomId, data.errorCode);
        break;
      case 'onConnectionStateChanged':
        final data = OnConnectionStateChangedData.fromMap(dic);
        onConnectionStateChanged?.call(data.state);
        break;
      case 'onNetworkTypeChanged':
        final data = OnNetworkTypeChangedData.fromMap(dic);
        onNetworkTypeChanged?.call(data.type);
        break;
      case 'onUserStartAudioCapture':
        final data = OnUserOperateMediaCaptureData.fromMap(dic);
        onUserStartAudioCapture?.call(data.roomId, data.uid);
        break;
      case 'onUserStopAudioCapture':
        final data = OnUserOperateMediaCaptureData.fromMap(dic);
        onUserStopAudioCapture?.call(data.roomId, data.uid);
        break;
      case 'onFirstLocalAudioFrame':
        final data = OnFirstLocalAudioFrameData.fromMap(dic);
        onFirstLocalAudioFrame?.call(data.index);
        break;
      case 'onFirstRemoteAudioFrame':
        final data = OnFirstRemoteAudioFrameData.fromMap(dic);
        onFirstRemoteAudioFrame?.call(data.streamKey);
        break;
      case 'onLocalAudioPropertiesReport':
        final data = OnLocalAudioPropertiesReportData.fromMap(dic);
        onLocalAudioPropertiesReport?.call(data.audioPropertiesInfos);
        break;
      case 'onRemoteAudioPropertiesReport':
        final data = OnRemoteAudioPropertiesReportData.fromMap(dic);
        onRemoteAudioPropertiesReport?.call(
            data.audioPropertiesInfos, data.totalRemoteVolume);
        break;
      case 'onActiveSpeaker':
        final data = OnActiveSpeakerData.fromMap(dic);
        onActiveSpeaker?.call(data.roomId, data.uid);
        break;
      case 'onUserStartVideoCapture':
        final data = OnUserOperateMediaCaptureData.fromMap(dic);
        onUserStartVideoCapture?.call(data.roomId, data.uid);
        break;
      case 'onUserStopVideoCapture':
        final data = OnUserOperateMediaCaptureData.fromMap(dic);
        onUserStopVideoCapture?.call(data.roomId, data.uid);
        break;
      case 'onFirstLocalVideoFrameCaptured':
        final data = OnFirstLocalVideoFrameCapturedData.fromMap(dic);
        onFirstLocalVideoFrameCaptured?.call(data.index, data.videoFrameInfo);
        break;
      case 'onFirstRemoteVideoFrameRendered':
        final data = OnFirstRemoteVideoFrameRenderedData.fromMap(dic);
        onFirstRemoteVideoFrameRendered?.call(
            data.streamKey, data.videoFrameInfo);
        break;
      case 'onFirstRemoteVideoFrameDecoded':
        final data = OnFirstRemoteVideoFrameRenderedData.fromMap(dic);
        onFirstRemoteVideoFrameDecoded?.call(
            data.streamKey, data.videoFrameInfo);
        break;
      case 'onRemoteVideoSizeChanged':
        final data = OnRemoteVideoSizeChangedData.fromMap(dic);
        onRemoteVideoSizeChanged?.call(data.streamKey, data.videoFrameInfo);
        break;
      case 'onLocalVideoSizeChanged':
        final data = OnLocalVideoSizeChangedData.fromMap(dic);
        onLocalVideoSizeChanged?.call(data.streamIndex, data.videoFrameInfo);
        break;
      case 'onAudioDeviceStateChanged':
        final data = OnAudioDeviceStateChangedData.fromMap(dic);
        onAudioDeviceStateChanged?.call(
            data.deviceId, data.deviceType, data.deviceState, data.deviceError);
        break;
      case 'onVideoDeviceStateChanged':
        final data = OnVideoDeviceStateChangedData.fromMap(dic);
        onVideoDeviceStateChanged?.call(
            data.deviceId, data.deviceType, data.deviceState, data.deviceError);
        break;
      case 'onAudioDeviceWarning':
        final data = OnAudioDeviceWarningData.fromMap(dic);
        onAudioDeviceWarning?.call(
            data.deviceId, data.deviceType, data.deviceWarning);
        break;
      case 'onVideoDeviceWarning':
        final data = OnVideoDeviceWarningData.fromMap(dic);
        onVideoDeviceWarning?.call(
            data.deviceId, data.deviceType, data.deviceWarning);
        break;
      case 'onAudioFrameSendStateChanged':
        final data = OnMediaFrameSendStateChangedData.fromMap(dic);
        onAudioFrameSendStateChanged?.call(
            data.roomId, data.userInfo, data.state);
        break;
      case 'onVideoFrameSendStateChanged':
        final data = OnMediaFrameSendStateChangedData.fromMap(dic);
        onVideoFrameSendStateChanged?.call(
            data.roomId, data.userInfo, data.state);
        break;
      case 'onScreenVideoFrameSendStateChanged':
        final data = OnMediaFrameSendStateChangedData.fromMap(dic);
        onScreenVideoFrameSendStateChanged?.call(
            data.roomId, data.userInfo, data.state);
        break;
      case 'onAudioFramePlayStateChanged':
        final data = OnMediaFramePlayStateChangedData.fromMap(dic);
        onAudioFramePlayStateChanged?.call(
            data.roomId, data.userInfo, data.state);
        break;
      case 'onVideoFramePlayStateChanged':
        final data = OnMediaFramePlayStateChangedData.fromMap(dic);
        onVideoFramePlayStateChanged?.call(
            data.roomId, data.userInfo, data.state);
        break;
      case 'onScreenVideoFramePlayStateChanged':
        final data = OnMediaFramePlayStateChangedData.fromMap(dic);
        onScreenVideoFramePlayStateChanged?.call(
            data.roomId, data.userInfo, data.state);
        break;
      case 'onAudioRouteChanged':
        final data = OnAudioRouteChangedData.fromMap(dic);
        onAudioRouteChanged?.call(data.route);
        break;
      case 'onSEIMessageReceived':
        final data = OnSEIMessageReceivedData.fromMap(dic);
        onSEIMessageReceived?.call(data.streamKey, data.message);
        break;
      case 'onSEIStreamUpdate':
        final data = OnSEIStreamUpdateData.fromMap(dic);
        onSEIStreamUpdate?.call(data.streamKey, data.event);
        break;
      case 'onStreamSyncInfoReceived':
        final data = OnStreamSyncInfoReceivedData.fromMap(dic);
        onStreamSyncInfoReceived?.call(
            data.streamKey, data.streamType, data.data);
        break;
      case 'onSysStats':
        final data = OnSysStatsData.fromMap(dic);
        onSysStats?.call(data.stats);
        break;
      case 'onLocalAudioStateChanged':
        final data = OnLocalAudioStateChangedData.fromMap(dic);
        onLocalAudioStateChanged?.call(data.state, data.error);
        break;
      case 'onRemoteAudioStateChanged':
        final data = OnRemoteAudioStateChangedData.fromMap(dic);
        onRemoteAudioStateChanged?.call(
            data.streamKey, data.state, data.reason);
        break;
      case 'onLocalVideoStateChanged':
        final data = OnLocalVideoStateChangedData.fromMap(dic);
        onLocalVideoStateChanged?.call(data.index, data.state, data.error);
        break;
      case 'onRemoteVideoStateChanged':
        final data = OnRemoteVideoStateChangedData.fromMap(dic);
        onRemoteVideoStateChanged?.call(
            data.streamKey, data.state, data.reason);
        break;
      case 'onRemoteVideoSuperResolutionModeChanged':
        final data = OnRemoteVideoSuperResolutionModeChangedData.fromMap(dic);
        onRemoteVideoSuperResolutionModeChanged?.call(
            data.streamKey, data.mode, data.reason);
        break;
      case 'onVideoDenoiseModeChanged':
        final data = OnVideoDenoiseModeChangedData.fromMap(dic);
        onVideoDenoiseModeChanged?.call(data.mode, data.reason);
        break;
      case 'onLoginResult':
        final data = OnLoginResultData.fromMap(dic);
        onLoginResult?.call(data.uid, data.errorCode, data.elapsed);
        break;
      case 'onLogout':
        final data = OnLogoutData.fromMap(dic);
        onLogout?.call(data.reason);
        break;
      case 'onServerParamsSetResult':
        final data = OnServerParamsSetResultData.fromMap(dic);
        onServerParamsSetResult?.call(data.error);
        break;
      case 'onGetPeerOnlineStatus':
        final data = OnGetPeerOnlineStatusData.fromMap(dic);
        onGetPeerOnlineStatus?.call(data.peerUid, data.status);
        break;
      case 'onUserMessageReceivedOutsideRoom':
        final data = OnMessageReceivedData.fromMap(dic);
        onUserMessageReceivedOutsideRoom?.call(data.uid, data.message);
        break;
      case 'onUserBinaryMessageReceivedOutsideRoom':
        final data = OnBinaryMessageReceivedData.fromMap(dic);
        onUserBinaryMessageReceivedOutsideRoom?.call(data.uid, data.message);
        break;
      case 'onUserMessageSendResultOutsideRoom':
        final data = OnMessageSendResultData.fromMap(dic);
        onUserMessageSendResultOutsideRoom?.call(data.msgid, data.error);
        break;
      case 'onServerMessageSendResult':
        final data = OnServerMessageSendResultData.fromMap(dic);
        onServerMessageSendResult?.call(data.msgid, data.error, data.message);
        break;
      case 'onNetworkDetectionResult':
        final data = OnNetworkDetectionResultData.fromMap(dic);
        onNetworkDetectionResult?.call(data.type, data.quality, data.rtt,
            data.lostRate, data.bitrate, data.jitter);
        break;
      case 'onNetworkDetectionStopped':
        final data = OnNetworkDetectionStoppedData.fromMap(dic);
        onNetworkDetectionStopped?.call(data.reason);
        break;
      case 'onAudioMixingStateChanged':
        final data = OnAudioMixingStateChangedData.fromMap(dic);
        onAudioMixingStateChanged?.call(data.mixId, data.state, data.error);
        break;
      case 'onAudioMixingPlayingProgress':
        final data = OnAudioMixingPlayingProgressData.fromMap(dic);
        onAudioMixingPlayingProgress?.call(data.mixId, data.progress);
        break;
      case 'onPerformanceAlarms':
        final data = OnPerformanceAlarmsData.fromMap(dic);
        onPerformanceAlarms?.call(
            data.mode, data.roomId, data.reason, data.data);
        break;
      case 'onSimulcastSubscribeFallback':
        final data = OnSimulcastSubscribeFallbackData.fromMap(dic);
        onSimulcastSubscribeFallback?.call(data.event);
        break;
      case 'onHttpProxyState':
        final data = OnHttpProxyStateData.fromMap(dic);
        onHttpProxyState?.call(data.state);
        break;
      case 'onHttpsProxyState':
        final data = OnHttpsProxyStateData.fromMap(dic);
        onHttpsProxyState?.call(data.state);
        break;
      case 'onSocks5ProxyState':
        final data = OnSocks5ProxyStateData.fromMap(dic);
        onSocks5ProxyState?.call(data.state, data.cmd, data.proxyAddress,
            data.localAddress, data.remoteAddress);
        break;
      case 'onRecordingStateUpdate':
        final data = OnRecordingStateUpdateData.fromMap(dic);
        onRecordingStateUpdate?.call(
            data.type, data.state, data.errorCode, data.info);
        break;
      case 'onRecordingProgressUpdate':
        final data = OnRecordingProgressUpdateData.fromMap(dic);
        onRecordingProgressUpdate?.call(data.type, data.progress, data.info);
        break;
      case 'onAudioRecordingStateUpdate':
        final data = OnAudioRecordingStateUpdateData.fromMap(dic);
        onAudioRecordingStateUpdate?.call(data.state, data.errorCode);
        break;
      case 'onPushPublicStreamResult':
        final data = OnPushPublicStreamResultData.fromMap(dic);
        onPushPublicStreamResult?.call(
            data.roomId, data.publicStreamId, data.errorCode);
        break;
      case 'onPlayPublicStreamResult':
        final data = OnPlayPublicStreamResultData.fromMap(dic);
        onPlayPublicStreamResult?.call(data.publicStreamId, data.errorCode);
        break;
      case 'onPublicStreamSEIMessageReceived':
        final data = OnPublicStreamDataMessageReceivedData.fromMap(dic);
        onPublicStreamSEIMessageReceived?.call(
            data.publicStreamId, data.message, data.sourceType);
        break;
      case 'onPublicStreamDataMessageReceived':
        final data = OnPublicStreamDataMessageReceivedData.fromMap(dic);
        onPublicStreamDataMessageReceived?.call(
            data.publicStreamId, data.message, data.sourceType);
        break;
      case 'onFirstPublicStreamVideoFrameDecoded':
        final data = OnFirstPublicStreamVideoFrameDecodedData.fromMap(dic);
        onFirstPublicStreamVideoFrameDecoded?.call(
            data.publicStreamId, data.videoFrameInfo);
        break;
      case 'onFirstPublicStreamAudioFrame':
        final data = OnFirstPublicStreamAudioFrameData.fromMap(dic);
        onFirstPublicStreamAudioFrame?.call(data.publicStreamId);
        break;
      case 'onCloudProxyConnected':
        final data = OnCloudProxyConnectedData.fromMap(dic);
        onCloudProxyConnected?.call(data.interval);
        break;
      case 'onEchoTestResult':
        final data = OnEchoTestResultData.fromMap(dic);
        onEchoTestResult?.call(data.result);
        break;
      case 'onNetworkTimeSynchronized':
        onNetworkTimeSynchronized?.call();
        break;
      case 'onLicenseWillExpire':
        final data = OnLicenseWillExpireData.fromMap(dic);
        onLicenseWillExpire?.call(data.days);
        break;
      case 'onInvokeExperimentalAPI':
        final data = OnInvokeExperimentalAPIData.fromMap(dic);
        onInvokeExperimentalAPI?.call(data.param);
        break;
      case 'onHardwareEchoDetectionResult':
        final data = OnHardwareEchoDetectionResultData.fromMap(dic);
        onHardwareEchoDetectionResult?.call(data.result);
        break;
      case 'onExtensionAccessError':
        final data = OnExtensionAccessErrorData.fromMap(dic);
        onExtensionAccessError?.call(data.extensionName, data.msg);
        break;
      case 'onLocalProxyStateChanged':
        final data = OnLocalProxyStateChangedData.fromMap(dic);
        onLocalProxyStateChanged?.call(
            data.localProxyType, data.localProxyState, data.localProxyError);
        break;
      default:
        break;
    }
  }
}

class OnWarningData {
  final WarningCode code;

  const OnWarningData({required this.code});

  factory OnWarningData.fromMap(Map<dynamic, dynamic> map) {
    return OnWarningData(code: (map['code'] as int).warningCode);
  }
}

class OnErrorData {
  final ErrorCode code;

  const OnErrorData({required this.code});

  factory OnErrorData.fromMap(Map<dynamic, dynamic> map) {
    return OnErrorData(code: (map['code'] as int).errorCode);
  }
}

class OnCreateRoomStateChangedData {
  final String roomId;
  final int errorCode;

  const OnCreateRoomStateChangedData(
      {required this.roomId, required this.errorCode});

  factory OnCreateRoomStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnCreateRoomStateChangedData(
        roomId: map['roomId'] as String, errorCode: map['errorCode'] as int);
  }
}

class OnConnectionStateChangedData {
  final RTCConnectionState state;

  const OnConnectionStateChangedData({required this.state});

  factory OnConnectionStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnConnectionStateChangedData(
        state: (map['state'] as int).connectionState);
  }
}

class OnNetworkTypeChangedData {
  final NetworkType type;

  const OnNetworkTypeChangedData({required this.type});

  factory OnNetworkTypeChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnNetworkTypeChangedData(type: (map['type'] as int).networkType);
  }
}

class OnUserOperateMediaCaptureData {
  final String roomId;
  final String uid;

  const OnUserOperateMediaCaptureData(
      {required this.roomId, required this.uid});

  factory OnUserOperateMediaCaptureData.fromMap(Map<dynamic, dynamic> map) {
    return OnUserOperateMediaCaptureData(
        roomId: map['roomId'] as String, uid: map['uid'] as String);
  }
}

class OnFirstLocalAudioFrameData {
  final StreamIndex index;

  const OnFirstLocalAudioFrameData({required this.index});

  factory OnFirstLocalAudioFrameData.fromMap(Map<dynamic, dynamic> map) {
    return OnFirstLocalAudioFrameData(index: (map['index'] as int).streamIndex);
  }
}

class OnFirstRemoteAudioFrameData {
  final RemoteStreamKey streamKey;

  const OnFirstRemoteAudioFrameData({required this.streamKey});

  factory OnFirstRemoteAudioFrameData.fromMap(Map<dynamic, dynamic> map) {
    return OnFirstRemoteAudioFrameData(
        streamKey: RemoteStreamKey.fromMap(map['streamKey']));
  }
}

class OnLocalAudioPropertiesReportData {
  final List<LocalAudioPropertiesInfo> audioPropertiesInfos;

  const OnLocalAudioPropertiesReportData({required this.audioPropertiesInfos});

  factory OnLocalAudioPropertiesReportData.fromMap(Map<dynamic, dynamic> map) {
    List<LocalAudioPropertiesInfo> infos = (map['infos'] as List<dynamic>)
        .map((e) => LocalAudioPropertiesInfo.fromMap(e))
        .toList();

    return OnLocalAudioPropertiesReportData(
      audioPropertiesInfos: infos,
    );
  }
}

class OnRemoteAudioPropertiesReportData {
  final List<RemoteAudioPropertiesInfo> audioPropertiesInfos;
  final int totalRemoteVolume;

  const OnRemoteAudioPropertiesReportData(
      {required this.audioPropertiesInfos, required this.totalRemoteVolume});

  factory OnRemoteAudioPropertiesReportData.fromMap(Map<dynamic, dynamic> map) {
    List<RemoteAudioPropertiesInfo> infos = (map['infos'] as List<dynamic>)
        .map((e) => RemoteAudioPropertiesInfo.fromMap(e))
        .toList();
    return OnRemoteAudioPropertiesReportData(
        audioPropertiesInfos: infos,
        totalRemoteVolume: map['totalRemoteVolume']);
  }
}

class OnActiveSpeakerData {
  final String roomId;
  final String uid;

  const OnActiveSpeakerData({required this.roomId, required this.uid});

  factory OnActiveSpeakerData.fromMap(Map<dynamic, dynamic> map) {
    return OnActiveSpeakerData(roomId: map['roomId'], uid: map['uid']);
  }
}

class OnFirstLocalVideoFrameCapturedData {
  final StreamIndex index;
  final VideoFrameInfo videoFrameInfo;

  const OnFirstLocalVideoFrameCapturedData(
      {required this.index, required this.videoFrameInfo});

  factory OnFirstLocalVideoFrameCapturedData.fromMap(
      Map<dynamic, dynamic> map) {
    return OnFirstLocalVideoFrameCapturedData(
        index: (map['index'] as int).streamIndex,
        videoFrameInfo: VideoFrameInfo.fromMap(map['videoFrame']));
  }
}

class OnFirstRemoteVideoFrameRenderedData {
  final RemoteStreamKey streamKey;
  final VideoFrameInfo videoFrameInfo;

  const OnFirstRemoteVideoFrameRenderedData(
      {required this.streamKey, required this.videoFrameInfo});

  factory OnFirstRemoteVideoFrameRenderedData.fromMap(
      Map<dynamic, dynamic> map) {
    return OnFirstRemoteVideoFrameRenderedData(
        streamKey: RemoteStreamKey.fromMap(map['streamKey']),
        videoFrameInfo: VideoFrameInfo.fromMap(map['videoFrame']));
  }
}

class OnRemoteVideoSizeChangedData {
  final RemoteStreamKey streamKey;
  final VideoFrameInfo videoFrameInfo;

  const OnRemoteVideoSizeChangedData(
      {required this.streamKey, required this.videoFrameInfo});

  factory OnRemoteVideoSizeChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnRemoteVideoSizeChangedData(
        streamKey: RemoteStreamKey.fromMap(map['streamKey']),
        videoFrameInfo: VideoFrameInfo.fromMap(map['videoFrame']));
  }
}

class OnLocalVideoSizeChangedData {
  final StreamIndex streamIndex;
  final VideoFrameInfo videoFrameInfo;

  const OnLocalVideoSizeChangedData(
      {required this.streamIndex, required this.videoFrameInfo});

  factory OnLocalVideoSizeChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnLocalVideoSizeChangedData(
        streamIndex: (map['index'] as int).streamIndex,
        videoFrameInfo: VideoFrameInfo.fromMap(map['videoFrame']));
  }
}

class OnAudioDeviceStateChangedData {
  final String deviceId;
  final AudioDeviceType deviceType;
  final MediaDeviceState deviceState;
  final MediaDeviceError deviceError;

  const OnAudioDeviceStateChangedData(
      {required this.deviceId,
      required this.deviceType,
      required this.deviceState,
      required this.deviceError});

  factory OnAudioDeviceStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnAudioDeviceStateChangedData(
        deviceId: map['deviceId'],
        deviceType: (map['deviceType'] as int).audioDeviceType,
        deviceState: (map['deviceState'] as int).mediaDeviceState,
        deviceError: (map['deviceError'] as int).mediaDeviceError);
  }
}

class OnVideoDeviceStateChangedData {
  final String deviceId;
  final VideoDeviceType deviceType;
  final MediaDeviceState deviceState;
  final MediaDeviceError deviceError;

  const OnVideoDeviceStateChangedData(
      {required this.deviceId,
      required this.deviceType,
      required this.deviceState,
      required this.deviceError});

  factory OnVideoDeviceStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnVideoDeviceStateChangedData(
        deviceId: map['deviceId'],
        deviceType: (map['deviceType'] as int).videoDeviceType,
        deviceState: (map['deviceState'] as int).mediaDeviceState,
        deviceError: (map['deviceError'] as int).mediaDeviceError);
  }
}

class OnAudioDeviceWarningData {
  final String deviceId;
  final AudioDeviceType deviceType;
  final MediaDeviceWarning deviceWarning;

  const OnAudioDeviceWarningData(
      {required this.deviceId,
      required this.deviceType,
      required this.deviceWarning});

  factory OnAudioDeviceWarningData.fromMap(Map<dynamic, dynamic> map) {
    return OnAudioDeviceWarningData(
        deviceId: map['deviceId'],
        deviceType: (map['deviceType'] as int).audioDeviceType,
        deviceWarning: (map['deviceWarning'] as int).mediaDeviceWarning);
  }
}

class OnVideoDeviceWarningData {
  final String deviceId;
  final VideoDeviceType deviceType;
  final MediaDeviceWarning deviceWarning;

  const OnVideoDeviceWarningData(
      {required this.deviceId,
      required this.deviceType,
      required this.deviceWarning});

  factory OnVideoDeviceWarningData.fromMap(Map<dynamic, dynamic> map) {
    return OnVideoDeviceWarningData(
        deviceId: map['deviceId'],
        deviceType: (map['deviceType'] as int).videoDeviceType,
        deviceWarning: (map['deviceWarning'] as int).mediaDeviceWarning);
  }
}

class OnMediaFrameSendStateChangedData {
  final String roomId;
  final UserInfo userInfo;
  final FirstFrameSendState state;

  const OnMediaFrameSendStateChangedData(
      {required this.roomId, required this.userInfo, required this.state});

  factory OnMediaFrameSendStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnMediaFrameSendStateChangedData(
        roomId: map['roomId'] as String,
        userInfo: UserInfo.fromMap(map['userInfo']),
        state: (map['state'] as int).firstFrameSendState);
  }
}

class OnMediaFramePlayStateChangedData {
  final String roomId;
  final UserInfo userInfo;
  final FirstFramePlayState state;

  const OnMediaFramePlayStateChangedData(
      {required this.roomId, required this.userInfo, required this.state});

  factory OnMediaFramePlayStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnMediaFramePlayStateChangedData(
        roomId: map['roomId'] as String,
        userInfo: UserInfo.fromMap(map['userInfo']),
        state: (map['state'] as int).firstFramePlayState);
  }
}

class OnAudioRouteChangedData {
  final AudioRoute route;

  const OnAudioRouteChangedData({required this.route});

  factory OnAudioRouteChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnAudioRouteChangedData(route: (map['route'] as int).audioRoute);
  }
}

class OnSEIMessageReceivedData {
  final RemoteStreamKey streamKey;
  final Uint8List message;

  const OnSEIMessageReceivedData(
      {required this.streamKey, required this.message});

  factory OnSEIMessageReceivedData.fromMap(Map<dynamic, dynamic> map) {
    return OnSEIMessageReceivedData(
        streamKey: RemoteStreamKey.fromMap(map['streamKey']),
        message: map['message']);
  }
}

class OnSEIStreamUpdateData {
  final RemoteStreamKey streamKey;
  final SEIStreamUpdateEvent event;

  const OnSEIStreamUpdateData({required this.streamKey, required this.event});

  factory OnSEIStreamUpdateData.fromMap(Map<dynamic, dynamic> map) {
    return OnSEIStreamUpdateData(
        streamKey: RemoteStreamKey.fromMap(map['streamKey']),
        event: (map['event'] as int).seiStreamUpdateEvent);
  }
}

class OnStreamSyncInfoReceivedData {
  final RemoteStreamKey streamKey;
  final SyncInfoStreamType streamType;
  final Uint8List data;

  const OnStreamSyncInfoReceivedData(
      {required this.streamKey, required this.streamType, required this.data});

  factory OnStreamSyncInfoReceivedData.fromMap(Map<dynamic, dynamic> map) {
    return OnStreamSyncInfoReceivedData(
      streamKey: RemoteStreamKey.fromMap(map['streamKey']),
      streamType: (map['streamType'] as int).syncInfoStreamType,
      data: map['data'],
    );
  }
}

class OnSysStatsData {
  final SysStats stats;

  const OnSysStatsData({required this.stats});

  factory OnSysStatsData.fromMap(Map<dynamic, dynamic> map) {
    return OnSysStatsData(stats: SysStats.fromMap(map['stats']));
  }
}

class OnLocalAudioStateChangedData {
  final LocalAudioStreamState state;
  final LocalAudioStreamError error;

  const OnLocalAudioStateChangedData(
      {required this.state, required this.error});

  factory OnLocalAudioStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnLocalAudioStateChangedData(
        state: (map['state'] as int).localAudioStreamState,
        error: (map['error'] as int).localAudioStreamError);
  }
}

class OnRemoteAudioStateChangedData {
  final RemoteStreamKey streamKey;
  final RemoteAudioState state;
  final RemoteAudioStateChangeReason reason;

  const OnRemoteAudioStateChangedData(
      {required this.streamKey, required this.state, required this.reason});

  factory OnRemoteAudioStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnRemoteAudioStateChangedData(
        streamKey: RemoteStreamKey.fromMap(map['streamKey']),
        state: (map['state'] as int).remoteAudioState,
        reason: (map['reason'] as int).remoteAudioStateChangeReason);
  }
}

class OnLocalVideoStateChangedData {
  final StreamIndex index;
  final LocalVideoStreamState state;
  final LocalVideoStreamError error;

  const OnLocalVideoStateChangedData(
      {required this.index, required this.state, required this.error});

  factory OnLocalVideoStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnLocalVideoStateChangedData(
        index: (map['index'] as int).streamIndex,
        state: (map['state'] as int).localVideoStreamState,
        error: (map['error'] as int).localVideoStreamError);
  }
}

class OnRemoteVideoStateChangedData {
  final RemoteStreamKey streamKey;
  final RemoteVideoState state;
  final RemoteVideoStateChangeReason reason;

  const OnRemoteVideoStateChangedData(
      {required this.streamKey, required this.state, required this.reason});

  factory OnRemoteVideoStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnRemoteVideoStateChangedData(
        streamKey: RemoteStreamKey.fromMap(map['streamKey']),
        state: (map['state'] as int).remoteVideoState,
        reason: (map['reason'] as int).remoteVideoStateChangedReason);
  }
}

class OnRemoteVideoSuperResolutionModeChangedData {
  final RemoteStreamKey streamKey;
  final VideoSuperResolutionMode mode;
  final VideoSuperResolutionModeChangedReason reason;

  const OnRemoteVideoSuperResolutionModeChangedData(
      {required this.streamKey, required this.mode, required this.reason});

  factory OnRemoteVideoSuperResolutionModeChangedData.fromMap(
      Map<dynamic, dynamic> map) {
    return OnRemoteVideoSuperResolutionModeChangedData(
        streamKey: RemoteStreamKey.fromMap(map['streamKey']),
        mode: (map['mode'] as int).videoSuperResolutionMode,
        reason: (map['reason'] as int).videoSuperResolutionModeChangedReason);
  }
}

class OnVideoDenoiseModeChangedData {
  final VideoDenoiseMode mode;
  final VideoDenoiseModeChangedReason reason;

  const OnVideoDenoiseModeChangedData(
      {required this.mode, required this.reason});

  factory OnVideoDenoiseModeChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnVideoDenoiseModeChangedData(
        mode: (map['mode'] as int).videoDenoiseMode,
        reason: (map['reason'] as int).videoDenoiseModeChangedReason);
  }
}

class OnLoginResultData {
  final String uid;
  final LoginErrorCode errorCode;
  final int elapsed;

  const OnLoginResultData(
      {required this.uid, required this.errorCode, required this.elapsed});

  factory OnLoginResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnLoginResultData(
        uid: map['uid'],
        errorCode: (map['errorCode'] as int).loginErrorCode,
        elapsed: map['elapsed']);
  }
}

class OnLogoutData {
  final LogoutReason reason;
  const OnLogoutData({required this.reason});

  factory OnLogoutData.fromMap(Map<dynamic, dynamic> map) {
    return OnLogoutData(reason: (map['reason'] as int).logoutReason);
  }
}

class OnServerParamsSetResultData {
  final int error;

  const OnServerParamsSetResultData({required this.error});

  factory OnServerParamsSetResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnServerParamsSetResultData(
      error: map['error'],
    );
  }
}

class OnGetPeerOnlineStatusData {
  final String peerUid;
  final UserOnlineStatus status;

  const OnGetPeerOnlineStatusData(
      {required this.peerUid, required this.status});

  factory OnGetPeerOnlineStatusData.fromMap(Map<dynamic, dynamic> map) {
    return OnGetPeerOnlineStatusData(
      peerUid: map['uid'],
      status: (map['status'] as int).userOnlineStatus,
    );
  }
}

class OnServerMessageSendResultData {
  final int msgid;
  final UserMessageSendResult error;
  final Uint8List message;

  const OnServerMessageSendResultData(
      {required this.msgid, required this.error, required this.message});

  factory OnServerMessageSendResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnServerMessageSendResultData(
        msgid: map['msgid'],
        error: (map['error'] as int).userMessageSendResult,
        message: map['message']);
  }
}

class OnNetworkDetectionResultData {
  final NetworkDetectionLinkType type;
  final NetworkQuality quality;
  final int rtt;
  final double lostRate;
  final int bitrate;
  final int jitter;

  const OnNetworkDetectionResultData(
      {required this.type,
      required this.quality,
      required this.rtt,
      required this.lostRate,
      required this.bitrate,
      required this.jitter});

  factory OnNetworkDetectionResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnNetworkDetectionResultData(
      type: (map['type'] as int).networkDetectionLinkType,
      quality: (map['quality'] as int).networkQuality,
      rtt: map['rtt'],
      lostRate: map['lostRate'],
      bitrate: map['bitrate'],
      jitter: map['jitter'],
    );
  }
}

class OnNetworkDetectionStoppedData {
  final NetworkDetectionStopReason reason;

  const OnNetworkDetectionStoppedData({required this.reason});

  factory OnNetworkDetectionStoppedData.fromMap(Map<dynamic, dynamic> map) {
    return OnNetworkDetectionStoppedData(
        reason: (map['reason'] as int).networkDetectionStopReason);
  }
}

class OnAudioMixingStateChangedData {
  final int mixId;
  final AudioMixingState state;
  final AudioMixingError error;

  const OnAudioMixingStateChangedData(
      {required this.mixId, required this.state, required this.error});

  factory OnAudioMixingStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnAudioMixingStateChangedData(
        mixId: map['mixId'],
        state: (map['state'] as int).audioMixingState,
        error: (map['error'] as int).audioMixingError);
  }
}

class OnAudioMixingPlayingProgressData {
  final int mixId;
  final int progress;

  const OnAudioMixingPlayingProgressData(
      {required this.mixId, required this.progress});

  factory OnAudioMixingPlayingProgressData.fromMap(Map<dynamic, dynamic> map) {
    return OnAudioMixingPlayingProgressData(
        mixId: map['mixId'], progress: map['progress']);
  }
}

class OnPerformanceAlarmsData {
  final PerformanceAlarmMode mode;
  final String roomId;
  final PerformanceAlarmReason reason;
  final SourceWantedData data;

  const OnPerformanceAlarmsData(
      {required this.mode,
      required this.roomId,
      required this.reason,
      required this.data});

  factory OnPerformanceAlarmsData.fromMap(Map<dynamic, dynamic> map) {
    return OnPerformanceAlarmsData(
      mode: (map['mode'] as int).performanceAlarmMode,
      roomId: map['roomId'],
      reason: (map['reason'] as int).performanceAlarmReason,
      data: SourceWantedData.fromMap(map['data']),
    );
  }
}

class OnSimulcastSubscribeFallbackData {
  final RemoteStreamSwitch event;

  const OnSimulcastSubscribeFallbackData({required this.event});

  factory OnSimulcastSubscribeFallbackData.fromMap(Map<dynamic, dynamic> map) {
    return OnSimulcastSubscribeFallbackData(
        event: RemoteStreamSwitch.fromMap(map['event']));
  }
}

class OnHttpProxyStateData {
  final int state;

  const OnHttpProxyStateData({required this.state});

  factory OnHttpProxyStateData.fromMap(Map<dynamic, dynamic> map) {
    return OnHttpProxyStateData(
      state: map['state'],
    );
  }
}

class OnHttpsProxyStateData {
  final int state;

  const OnHttpsProxyStateData({required this.state});

  factory OnHttpsProxyStateData.fromMap(Map<dynamic, dynamic> map) {
    return OnHttpsProxyStateData(
      state: map['state'],
    );
  }
}

class OnSocks5ProxyStateData {
  final int state;
  final String cmd;
  final String proxyAddress;
  final String localAddress;
  final String remoteAddress;

  const OnSocks5ProxyStateData(
      {required this.state,
      required this.cmd,
      required this.proxyAddress,
      required this.localAddress,
      required this.remoteAddress});

  factory OnSocks5ProxyStateData.fromMap(Map<dynamic, dynamic> map) {
    return OnSocks5ProxyStateData(
      state: map['state'],
      cmd: map['cmd'],
      proxyAddress: map['proxyAddress'],
      localAddress: map['localAddress'],
      remoteAddress: map['remoteAddress'],
    );
  }
}

class OnRecordingStateUpdateData {
  final StreamIndex type;
  final RecordingState state;
  final RecordingErrorCode errorCode;
  final RecordingInfo info;

  const OnRecordingStateUpdateData(
      {required this.state,
      required this.type,
      required this.errorCode,
      required this.info});

  factory OnRecordingStateUpdateData.fromMap(Map<dynamic, dynamic> map) {
    return OnRecordingStateUpdateData(
      state: (map['state'] as int).recordingState,
      type: (map['type'] as int).streamIndex,
      errorCode: (map['errorCode'] as int).recordingErrorCode,
      info: RecordingInfo.fromMap(map['info']),
    );
  }
}

class OnRecordingProgressUpdateData {
  final StreamIndex type;
  final RecordingProgress progress;
  final RecordingInfo info;

  const OnRecordingProgressUpdateData(
      {required this.type, required this.progress, required this.info});

  factory OnRecordingProgressUpdateData.fromMap(Map<dynamic, dynamic> map) {
    return OnRecordingProgressUpdateData(
      type: (map['type'] as int).streamIndex,
      progress: RecordingProgress.fromMap(map['progress']),
      info: RecordingInfo.fromMap(map['info']),
    );
  }
}

class OnAudioRecordingStateUpdateData {
  final AudioRecordingState state;
  final AudioRecordingErrorCode errorCode;

  const OnAudioRecordingStateUpdateData(
      {required this.state, required this.errorCode});

  factory OnAudioRecordingStateUpdateData.fromMap(Map<dynamic, dynamic> map) {
    return OnAudioRecordingStateUpdateData(
        state: (map['state'] as int).audioRecordingState,
        errorCode: (map['errorCode'] as int).audioRecordingErrorCode);
  }
}

class OnPushPublicStreamResultData {
  final String roomId;
  final String publicStreamId;
  final PublicStreamErrorCode errorCode;

  const OnPushPublicStreamResultData(
      {required this.roomId,
      required this.publicStreamId,
      required this.errorCode});

  factory OnPushPublicStreamResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnPushPublicStreamResultData(
        roomId: map['roomId'],
        publicStreamId: map['publicStreamId'],
        errorCode: (map['errorCode'] as int).publicStreamErrorCode);
  }
}

class OnPlayPublicStreamResultData {
  final String publicStreamId;
  final PublicStreamErrorCode errorCode;

  const OnPlayPublicStreamResultData(
      {required this.publicStreamId, required this.errorCode});

  factory OnPlayPublicStreamResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnPlayPublicStreamResultData(
        publicStreamId: map['publicStreamId'],
        errorCode: (map['errorCode'] as int).publicStreamErrorCode);
  }
}

class OnPublicStreamDataMessageReceivedData {
  final String publicStreamId;
  final Uint8List message;
  final DataMessageSourceType sourceType;

  const OnPublicStreamDataMessageReceivedData(
      {required this.publicStreamId,
      required this.message,
      required this.sourceType});

  factory OnPublicStreamDataMessageReceivedData.fromMap(
      Map<dynamic, dynamic> map) {
    return OnPublicStreamDataMessageReceivedData(
        publicStreamId: map['publicStreamId'],
        message: map['message'],
        sourceType: (map['sourceType'] as int).seiMessageSourceType);
  }
}

class OnFirstPublicStreamVideoFrameDecodedData {
  final String publicStreamId;
  final VideoFrameInfo videoFrameInfo;

  const OnFirstPublicStreamVideoFrameDecodedData(
      {required this.publicStreamId, required this.videoFrameInfo});

  factory OnFirstPublicStreamVideoFrameDecodedData.fromMap(
      Map<dynamic, dynamic> map) {
    return OnFirstPublicStreamVideoFrameDecodedData(
        publicStreamId: map['publicStreamId'],
        videoFrameInfo: VideoFrameInfo.fromMap(map['videoFrame']));
  }
}

class OnFirstPublicStreamAudioFrameData {
  final String publicStreamId;

  const OnFirstPublicStreamAudioFrameData({required this.publicStreamId});

  factory OnFirstPublicStreamAudioFrameData.fromMap(Map<dynamic, dynamic> map) {
    return OnFirstPublicStreamAudioFrameData(
        publicStreamId: map['publicStreamId']);
  }
}

class OnCloudProxyConnectedData {
  final int interval;

  const OnCloudProxyConnectedData({required this.interval});

  factory OnCloudProxyConnectedData.fromMap(Map<dynamic, dynamic> map) {
    return OnCloudProxyConnectedData(interval: map['interval']);
  }
}

class OnEchoTestResultData {
  final EchoTestResult result;

  const OnEchoTestResultData({required this.result});

  factory OnEchoTestResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnEchoTestResultData(result: (map['result'] as int).echoTestResult);
  }
}

class OnLicenseWillExpireData {
  final int days;

  const OnLicenseWillExpireData({required this.days});

  factory OnLicenseWillExpireData.fromMap(Map<dynamic, dynamic> map) {
    return OnLicenseWillExpireData(days: map['days']);
  }
}

class OnInvokeExperimentalAPIData {
  final String param;

  const OnInvokeExperimentalAPIData({required this.param});

  factory OnInvokeExperimentalAPIData.fromMap(Map<dynamic, dynamic> map) {
    return OnInvokeExperimentalAPIData(param: map['param']);
  }
}

class OnHardwareEchoDetectionResultData {
  final HardwareEchoDetectionResult result;

  const OnHardwareEchoDetectionResultData({required this.result});

  factory OnHardwareEchoDetectionResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnHardwareEchoDetectionResultData(
        result: (map['result'] as int?).hardwareEchoDetectionResult);
  }
}

class OnExtensionAccessErrorData {
  final String extensionName;
  final String msg;

  const OnExtensionAccessErrorData({
    required this.extensionName,
    required this.msg,
  });

  factory OnExtensionAccessErrorData.fromMap(Map<dynamic, dynamic> map) {
    return OnExtensionAccessErrorData(
      extensionName: map['extensionName'],
      msg: map['msg'],
    );
  }
}

class OnLocalProxyStateChangedData {
  final LocalProxyType localProxyType;
  final LocalProxyState localProxyState;
  final LocalProxyError localProxyError;

  const OnLocalProxyStateChangedData({
    required this.localProxyType,
    required this.localProxyState,
    required this.localProxyError,
  });

  factory OnLocalProxyStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnLocalProxyStateChangedData(
      localProxyType: (map['localProxyType'] as int?).localProxyType,
      localProxyState: (map['localProxyState'] as int?).localProxyState,
      localProxyError: (map['localProxyError'] as int?).localProxyError,
    );
  }
}
